@@ -529,6 +529,7 @@ QString mouseKeysShortcut (Display *display) {
      return ""; // Somehow the keycode -> keysym mapping is flawed
 
   XEvent ev;
+  ev.type = KeyPress;
   ev.xkey.display = display;
   ev.xkey.keycode = code;
   ev.xkey.state = 0;
@@ -588,7 +589,6 @@ void KAccessApp::createDialogContents() {
       dialog->setEscapeButton( KDialog::Close );
       dialog->setObjectName( "AccessXWarning" );
       dialog->setModal( true );
-      dialog->showButtonSeparator( true );
 
       KVBox *topcontents = new KVBox (dialog);
       topcontents->setSpacing(KDialog::spacingHint()*2);
@@ -634,7 +634,6 @@ void KAccessApp::createDialogContents() {
       showModeCombobox->setCurrentIndex (1);
 
       dialog->setMainWidget(topcontents);
-      dialog->showButtonSeparator(false);
 
       connect (dialog, SIGNAL(yesClicked()), this, SLOT(yesClicked()));
       connect (dialog, SIGNAL(noClicked()), this, SLOT(noClicked()));
@@ -779,7 +778,7 @@ void KAccessApp::xkbControlsNotify(XkbControlsNotifyEvent *event)
 
         createDialogContents();
         featuresLabel->setText ( question+"\n\n"+explanation
-              +" "+i18n("These AccessX settings are needed for some users with motion impairments and can be configured in the KDE Control Center. You can also turn them on and off with standardized keyboard gestures.\n\nIf you do not need them, you can select \"Deactivate all AccessX features and gestures\".") );
+              +" "+i18n("These AccessX settings are needed for some users with motion impairments and can be configured in the KDE System Settings. You can also turn them on and off with standardized keyboard gestures.\n\nIf you do not need them, you can select \"Deactivate all AccessX features and gestures\".") );
 
         KWindowSystem::setState( dialog->winId(), NET::KeepAbove );
         kapp->updateUserTimestamp();
