@@ -23,6 +23,7 @@
 #include "../krdb/krdb.h"
 
 #include <QtCore/QFileInfo>
+#include <QtCore/QTimer>
 #include <QtGui/QHeaderView>
 #include <QtGui/QStackedWidget>
 #include <QtGui/QPainter>
@@ -41,7 +42,8 @@
 #include <KMessageBox>
 #include <KStandardDirs>
 #include <kio/netaccess.h>
-#include <knewstuff2/engine.h>
+#include <knewstuff3/downloaddialog.h>
+#include <knewstuff3/uploaddialog.h>
 
 K_PLUGIN_FACTORY( KolorFactory, registerPlugin<KColorCm>(); )
 K_EXPORT_PLUGIN( KolorFactory("kcmcolors") )
@@ -56,10 +58,10 @@ void KColorCm::WindecoColors::load(const KSharedConfigPtr &config)
 {
     // NOTE: keep this in sync with kdelibs/kdeui/kernel/kglobalsettings.cpp
     KConfigGroup group(config, "WM");
-    m_colors[ActiveBackground] = group.readEntry("activeBackground", QColor(96, 148, 207));
+    m_colors[ActiveBackground] = group.readEntry("activeBackground", QColor(48, 174, 232));
     m_colors[ActiveForeground] = group.readEntry("activeForeground", QColor(255, 255, 255));
     m_colors[InactiveBackground] = group.readEntry("inactiveBackground", QColor(224, 223, 222));
-    m_colors[InactiveForeground] = group.readEntry("inactiveForeground", QColor(20, 19, 18));
+    m_colors[InactiveForeground] = group.readEntry("inactiveForeground", QColor(75, 71, 67));
 }
 
 QColor KColorCm::WindecoColors::color(WindecoColors::Role role) const
@@ -69,7 +71,9 @@ QColor KColorCm::WindecoColors::color(WindecoColors::Role role) const
 //END WindecoColors
 
 KColorCm::KColorCm(QWidget *parent, const QVariantList &)
-    : KCModule( KolorFactory::componentData(), parent ), m_disableUpdates(false)
+    : KCModule( KolorFactory::componentData(), parent ), m_disableUpdates(false),
+      m_loadedSchemeHasUnsavedChanges(false), m_dontLoadSelectedScheme(false),
+      m_previousSchemeItem(0)
 {
     KAboutData* about = new KAboutData(
         "kcmcolors", 0, ki18n("Colors"), 0, KLocalizedString(),
@@ -85,8 +89,10 @@ KColorCm::KColorCm(QWidget *parent, const QVariantList &)
 
     setupUi(this);
     schemeKnsButton->setIcon( KIcon("get-hot-new-stuff") );
+    schemeKnsUploadButton->setIcon( KIcon("get-hot-new-stuff") );
     connect(colorSet, SIGNAL(currentIndexChanged(int)), this, SLOT(updateColorTable()));
-    connect(schemeList, SIGNAL(currentRowChanged(int)), this, SLOT(loadScheme()));
+    connect(schemeList, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)),
+            this, SLOT(loadScheme(QListWidgetItem*, QListWidgetItem*)));
     connect(applyToAlien, SIGNAL(toggled(bool)), this, SLOT(emitChanged()));
 
     // only needs to be called once
@@ -178,7 +184,7 @@ void KColorCm::updateEffectsPage()
     {
         disabledColorSlider->setValue(int(groupD.readEntry("ColorAmount", 0.0) * 20.0) + 20);
     }
-    disabledColorButton->setColor(groupD.readEntry("Color", QColor(112, 111, 110)));
+    disabledColorButton->setColor(groupD.readEntry("Color", QColor(56, 56, 56)));
     disabledContrastBox->setCurrentIndex(groupD.readEntry("ContrastEffect", 1));
     disabledContrastSlider->setValue(int(groupD.readEntry("ContrastAmount", 0.65) * 20.0));
 
@@ -210,18 +216,50 @@ void KColorCm::loadScheme(KSharedConfigPtr config) // const QString &path)
     updateColorTable();
     updatePreviews();
 
+    m_loadedSchemeHasUnsavedChanges = false;
     //m_changed = false;
 }
 
-void KColorCm::loadScheme()
+void KColorCm::selectPreviousSchemeAgain()
 {
-    if (schemeList->currentItem() != NULL)
+    m_dontLoadSelectedScheme = true;
+    schemeList->setCurrentItem(m_previousSchemeItem);
+    m_dontLoadSelectedScheme = false;
+}
+
+void KColorCm::loadScheme(QListWidgetItem *currentItem, QListWidgetItem *previousItem)
+{
+    m_previousSchemeItem = previousItem;
+
+    if (m_dontLoadSelectedScheme)
     {
-        const QString name = schemeList->currentItem()->text();
-        const QString fileBaseName = schemeList->currentItem()->data(Qt::UserRole).toString();
+        return;
+    }
+
+    if (currentItem != NULL)
+    {
+        if (m_loadedSchemeHasUnsavedChanges) // if changes made to loaded scheme
+        {
+            if (KMessageBox::Continue != KMessageBox::warningContinueCancel(this,
+                i18n("Selecting another scheme will discard any changes you have made"),
+                i18n("Are you sure?"),
+                KStandardGuiItem::cont(),
+                KStandardGuiItem::cancel(),
+                "noDiscardWarning")) // if the user decides to cancel
+            {
+                QTimer::singleShot(0, this, SLOT(selectPreviousSchemeAgain()));
+                return;
+            }
+        }
+
+        // load it
+
+        const QString name = currentItem->text();
+        const QString fileBaseName = currentItem->data(Qt::UserRole).toString();
         if (name == i18nc("Default color scheme", "Default"))
         {
             schemeRemoveButton->setEnabled(false);
+            schemeKnsUploadButton->setEnabled(false);
 
             KSharedConfigPtr config = m_config;
             config->setReadDefaults(true);
@@ -233,6 +271,7 @@ void KColorCm::loadScheme()
         else if (name == i18nc("Current color scheme", "Current"))
         {
             schemeRemoveButton->setEnabled(false);
+            schemeKnsUploadButton->setEnabled(false);
             loadInternal(false);
         }
         else
@@ -244,19 +283,7 @@ void KColorCm::loadScheme()
             const bool canWrite = (permissions & QFile::WriteUser);
             kDebug() << "checking permissions of " << path;
             schemeRemoveButton->setEnabled(canWrite);
-
-            if (0) // TODO if changes made to loaded scheme
-            {
-                if (KMessageBox::Continue != KMessageBox::warningContinueCancel(this,
-                    i18n("Selecting another scheme will discard any changes you have made"),
-                    i18n("Are you sure?"),
-                    KStandardGuiItem::cont(),
-                    KStandardGuiItem::cancel(),
-                    "noDiscardWarning"))
-                {
-                    return;
-                }
-            }
+            schemeKnsUploadButton->setEnabled(true);
 
             KSharedConfigPtr config = KSharedConfig::openConfig(path);
             loadScheme(config);
@@ -360,15 +387,40 @@ void KColorCm::on_schemeImportButton_clicked()
 
 void KColorCm::on_schemeKnsButton_clicked()
 {
-    KNS::Engine engine(this);
-    if (engine.init("colorschemes.knsrc")) {
+    KNS3::DownloadDialog dialog("colorschemes.knsrc", this);
+    dialog.exec();
+    if ( ! dialog.changedEntries().isEmpty() )
+    {
+        populateSchemeList();
+    }
+}
 
-
-        KNS::Entry::List entries = engine.downloadDialogModal(this);
-        if (!entries.isEmpty())
+void KColorCm::on_schemeKnsUploadButton_clicked()
+{
+    if (schemeList->currentItem() != NULL)
+    {
+        // check if the currently loaded scheme has unsaved changes
+        if (m_loadedSchemeHasUnsavedChanges)
         {
-            populateSchemeList();
+            KMessageBox::sorry(this, i18n("Please save the color scheme before uploading it."),
+                               i18n("Please save"));
+            return;
         }
+
+        // find path
+        const QString basename = schemeList->currentItem()->data(Qt::UserRole).toString();
+        const QString path = KGlobal::dirs()->findResource("data",
+            "color-schemes/" + basename + ".colors");
+        if (path.isEmpty() ) // if the color scheme file wasn't found
+        {
+            kDebug() << "path for color scheme " << basename << " couldn't be found";
+            return;
+        }
+
+        // upload
+        KNS3::UploadDialog dialog("colorschemes.knsrc", this);
+        dialog.setUploadFile(KUrl(path) );
+        dialog.exec();
     }
 }
 
@@ -423,7 +475,7 @@ void KColorCm::saveScheme(const QString &name)
                 return;
             }
         }
-        
+
         // go ahead and save it
         QString newpath = KGlobal::dirs()->saveLocation("data", "color-schemes/");
         newpath += filename + ".colors";
@@ -437,6 +489,8 @@ void KColorCm::saveScheme(const QString &name)
         // sync it
         m_config->sync();
 
+        m_loadedSchemeHasUnsavedChanges = false;
+
         QList<QListWidgetItem*> foundItems = schemeList->findItems(name, Qt::MatchExactly);
         QIcon icon = createSchemePreviewIcon(m_config);
         if (foundItems.size() < 1)
@@ -458,6 +512,7 @@ void KColorCm::saveScheme(const QString &name)
 
         // set m_config back to the system one
         m_config = temp;
+
         emit changed(true);
     }
     else if (!canWrite && file.exists())
@@ -739,6 +794,7 @@ void KColorCm::setCommonForeground(KColorScheme::ForegroundRole role, int stackI
 
     m_stackedWidgets[stackIndex]->setCurrentIndex(0);
     m_commonColorButtons[buttonIndex]->setColor(color);
+    m_loadedSchemeHasUnsavedChanges = true;
 }
 
 void KColorCm::setCommonDecoration(KColorScheme::DecorationRole role, int stackIndex,
@@ -756,6 +812,7 @@ void KColorCm::setCommonDecoration(KColorScheme::DecorationRole role, int stackI
 
     m_stackedWidgets[stackIndex]->setCurrentIndex(0);
     m_commonColorButtons[buttonIndex]->setColor(color);
+    m_loadedSchemeHasUnsavedChanges = true;
 }
 
 void KColorCm::updateColorTable()
@@ -1005,6 +1062,8 @@ void KColorCm::changeColor(int row, const QColor &newColor)
     updateColorSchemes();
     updatePreviews();
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
@@ -1088,7 +1147,20 @@ void KColorCm::load()
     applyToAlien->blockSignals(false);
 }
 
-void KColorCm::loadInternal(bool loadOptions)
+void KColorCm::loadOptions()
+{
+    contrastSlider->setValue(KGlobalSettings::contrast());
+    shadeSortedColumn->setCheckState(KGlobalSettings::shadeSortColumn() ? Qt::Checked : Qt::Unchecked);
+
+    KConfigGroup group(m_config, "ColorEffects:Inactive");
+    useInactiveEffects->setCheckState(group.readEntry("Enable", false) ? Qt::Checked : Qt::Unchecked);
+    // NOTE: keep this in sync with kdelibs/kdeui/colors/kcolorscheme.cpp
+    // NOTE: remove extra logic from updateFromOptions and on_useInactiveEffects_stateChanged when this changes!
+    inactiveSelectionEffect->setCheckState(group.readEntry("ChangeSelectionColor", group.readEntry("Enable", true))
+                                           ? Qt::Checked : Qt::Unchecked);
+}
+
+void KColorCm::loadInternal(bool loadOptions_)
 {
     // clean the config, in case we have changed the in-memory kconfig
     m_config->markAsClean();
@@ -1101,23 +1173,15 @@ void KColorCm::loadInternal(bool loadOptions)
     // fill in the color scheme list
     populateSchemeList();
 
-    if (loadOptions)
-    {
-        contrastSlider->setValue(KGlobalSettings::contrast());
-        shadeSortedColumn->setCheckState(KGlobalSettings::shadeSortColumn() ? Qt::Checked : Qt::Unchecked);
-
-        KConfigGroup group(m_config, "ColorEffects:Inactive");
-        useInactiveEffects->setCheckState(group.readEntry("Enable", false) ? Qt::Checked : Qt::Unchecked);
-        // NOTE: keep this in sync with kdelibs/kdeui/colors/kcolorscheme.cpp
-        // NOTE: remove extra logic from updateFromOptions and on_useInactiveEffects_stateChanged when this changes!
-        inactiveSelectionEffect->setCheckState(group.readEntry("ChangeSelectionColor", group.readEntry("Enable", false))
-                                               ? Qt::Checked : Qt::Unchecked);
-    }
+    if (loadOptions_)
+        loadOptions();
 
     updateEffectsPage();
 
     updatePreviews();
 
+    m_loadedSchemeHasUnsavedChanges = false;
+
     emit changed(false);
 }
 
@@ -1158,12 +1222,25 @@ void KColorCm::save()
 
 void KColorCm::defaults()
 {
+    // Switch to default scheme
     for(int i = 0; i < schemeList->count(); ++i) {
-        if(schemeList->item(i)->text() == i18nc("Default color scheme", "Default")) {
-            schemeList->setCurrentItem(schemeList->item(i));
+        QListWidgetItem *item = schemeList->item(i);
+        if(item->text() == i18nc("Default color scheme", "Default")) {
+            // If editing the default scheme, force a reload, else select the default scheme
+            if(schemeList->currentItem() == item)
+                loadScheme(item, item);
+            else
+                schemeList->setCurrentItem(item);
             break;
         }
     }
+
+    // Reset options (not part of scheme)
+    m_config->setReadDefaults(true);
+    loadOptions();
+    m_config->setReadDefaults(false);
+    applyToAlien->setChecked(Qt::Checked);
+
     KCModule::defaults();
     emit changed(true);
 }
@@ -1177,70 +1254,84 @@ void KColorCm::emitChanged()
 void KColorCm::on_inactiveIntensityBox_currentIndexChanged(int index)
 {
     Q_UNUSED( index );
-    
+
     updateFromEffectsPage();
     inactivePreview->setPalette(m_config, QPalette::Inactive);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
 void KColorCm::on_inactiveIntensitySlider_valueChanged(int value)
 {
     Q_UNUSED( value );
-    
+
     updateFromEffectsPage();
     inactivePreview->setPalette(m_config, QPalette::Inactive);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
 void KColorCm::on_inactiveColorBox_currentIndexChanged(int index)
 {
     Q_UNUSED( index );
-    
+
     updateFromEffectsPage();
     inactivePreview->setPalette(m_config, QPalette::Inactive);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
 void KColorCm::on_inactiveColorSlider_valueChanged(int value)
 {
     Q_UNUSED( value );
-    
+
     updateFromEffectsPage();
     inactivePreview->setPalette(m_config, QPalette::Inactive);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
 void KColorCm::on_inactiveColorButton_changed(const QColor& color)
 {
     Q_UNUSED( color );
-    
+
     updateFromEffectsPage();
     inactivePreview->setPalette(m_config, QPalette::Inactive);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
 void KColorCm::on_inactiveContrastBox_currentIndexChanged(int index)
 {
     Q_UNUSED( index );
-    
+
     updateFromEffectsPage();
     inactivePreview->setPalette(m_config, QPalette::Inactive);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
 void KColorCm::on_inactiveContrastSlider_valueChanged(int value)
 {
     Q_UNUSED( value );
-    
+
     updateFromEffectsPage();
     inactivePreview->setPalette(m_config, QPalette::Inactive);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
@@ -1248,70 +1339,84 @@ void KColorCm::on_inactiveContrastSlider_valueChanged(int value)
 void KColorCm::on_disabledIntensityBox_currentIndexChanged(int index)
 {
     Q_UNUSED( index );
-    
+
     updateFromEffectsPage();
     disabledPreview->setPalette(m_config, QPalette::Disabled);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
 void KColorCm::on_disabledIntensitySlider_valueChanged(int value)
 {
     Q_UNUSED( value );
-    
+
     updateFromEffectsPage();
     disabledPreview->setPalette(m_config, QPalette::Disabled);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
 void KColorCm::on_disabledColorBox_currentIndexChanged(int index)
 {
     Q_UNUSED( index );
-    
+
     updateFromEffectsPage();
     disabledPreview->setPalette(m_config, QPalette::Disabled);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
 void KColorCm::on_disabledColorSlider_valueChanged(int value)
 {
     Q_UNUSED( value );
-    
+
     updateFromEffectsPage();
     disabledPreview->setPalette(m_config, QPalette::Disabled);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
 void KColorCm::on_disabledColorButton_changed(const QColor& color)
 {
     Q_UNUSED( color );
-    
+
     updateFromEffectsPage();
     disabledPreview->setPalette(m_config, QPalette::Disabled);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
 void KColorCm::on_disabledContrastBox_currentIndexChanged(int index)
 {
     Q_UNUSED( index );
-    
+
     updateFromEffectsPage();
     disabledPreview->setPalette(m_config, QPalette::Disabled);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
 void KColorCm::on_disabledContrastSlider_valueChanged(int value)
 {
     Q_UNUSED( value );
-    
+
     updateFromEffectsPage();
     disabledPreview->setPalette(m_config, QPalette::Disabled);
 
+    m_loadedSchemeHasUnsavedChanges = true;
+
     emit changed(true);
 }
 
