@@ -28,6 +28,7 @@
 #include "ui_colorsettings.h"
 
 class QStackedWidget;
+class QListWidgetItem;
 
 /**
  * The Desktop/Colors tab in kcontrol.
@@ -63,7 +64,10 @@ private slots:
     void variesClicked();
 
     /** slot called when the schemeList selection changes */
-    void loadScheme();
+    void loadScheme(QListWidgetItem *currentItem, QListWidgetItem *previousItem);
+
+    /** reselect the previously selected scheme in schemeList without loading it */
+    void selectPreviousSchemeAgain();
 
     /** slot called when the remove button is clicked*/
     void on_schemeRemoveButton_clicked();
@@ -76,6 +80,9 @@ private slots:
 
     /** slot called when the get new schemes button is clicked */
     void on_schemeKnsButton_clicked();
+    
+    /** slot called when the upload scheme button is clicked */
+    void on_schemeKnsUploadButton_clicked();
 
     /** null slot to emit changed(true) */
     void emitChanged();
@@ -126,6 +133,9 @@ private:
     /** create a preview of a color scheme */
     static QPixmap createSchemePreviewIcon(const KSharedConfigPtr &config);
 
+    /** load options from global */
+    void loadOptions();
+
     /** load from global */
     void loadInternal(bool loadOptions);
 
@@ -192,6 +202,12 @@ private:
     KSharedConfigPtr m_config;
 
     bool m_disableUpdates;
+    bool m_loadedSchemeHasUnsavedChanges;
+    // don't (re)load the scheme, only select it in schemeList
+    bool m_dontLoadSelectedScheme;
+
+    // the item previously selected in schemeList
+    QListWidgetItem *m_previousSchemeItem;
 };
 
 #endif
