@@ -54,12 +54,12 @@ static const int TAB_HARDWARE = 0;
 static const int TAB_ADVANCED = 2;
 
 KCMKeyboardWidget::KCMKeyboardWidget(Rules* rules_, KeyboardConfig* keyboardConfig_, const KComponentData componentData_, QWidget* /*parent*/):
+	rules(rules_),
 	componentData(componentData_),
 	actionCollection(NULL),
 	uiUpdating(false)
 {
 	flags = new Flags();
-	rules = rules_;
 	keyboardConfig = keyboardConfig_;
 
 	uiWidget = new Ui::TabWidget;
@@ -118,7 +118,7 @@ void KCMKeyboardWidget::uiChanged()
 
 	keyboardConfig->configureLayouts = uiWidget->configureLayoutsChk->isChecked();
 	keyboardConfig->keyboardModel = uiWidget->keyboardModelComboBox->itemData(uiWidget->keyboardModelComboBox->currentIndex()).toString();
-	keyboardConfig->showFlag = uiWidget->showFlagChk->isChecked();
+	keyboardConfig->showFlag = uiWidget->showFlagRadioBtn->isChecked();
 
 	keyboardConfig->resetOldXkbOptions = uiWidget->configureKeyboardOptionsChk->isChecked();
 //    if( keyboardConfig->resetOldXkbOptions ) {
@@ -184,13 +184,20 @@ void KCMKeyboardWidget::addLayout()
 void KCMKeyboardWidget::initializeLayoutsUI()
 {
 	layoutsTableModel = new LayoutsTableModel(rules, flags, keyboardConfig, uiWidget->layoutsTableView);
+	uiWidget->layoutsTableView->setEditTriggers(QAbstractItemView::SelectedClicked | QAbstractItemView::DoubleClicked | QAbstractItemView::EditKeyPressed | QAbstractItemView::AnyKeyPressed);
 	uiWidget->layoutsTableView->setModel(layoutsTableModel);
+	uiWidget->layoutsTableView->setIconSize( flags->getTransparentPixmap().size() );
+
 	//TODO: do we need to delete this delegate or parent will take care of it?
-	VariantComboDelegate* delegate = new VariantComboDelegate(keyboardConfig, rules, uiWidget->layoutsTableView);
-	uiWidget->layoutsTableView->setItemDelegateForColumn(LayoutsTableModel::VARIANT_COLUMN, delegate);
+	VariantComboDelegate* variantDelegate = new VariantComboDelegate(keyboardConfig, rules, uiWidget->layoutsTableView);
+	uiWidget->layoutsTableView->setItemDelegateForColumn(LayoutsTableModel::VARIANT_COLUMN, variantDelegate);
+
+	LabelEditDelegate* labelDelegate = new LabelEditDelegate(keyboardConfig, uiWidget->layoutsTableView);
+	uiWidget->layoutsTableView->setItemDelegateForColumn(LayoutsTableModel::DISPLAY_NAME_COLUMN, labelDelegate);
+
 	//TODO: is it ok to hardcode sizes? any better approach?
 	uiWidget->layoutsTableView->setColumnWidth(LayoutsTableModel::MAP_COLUMN, 70);
-	uiWidget->layoutsTableView->setColumnWidth(LayoutsTableModel::LAYOUT_COLUMN, 150);
+	uiWidget->layoutsTableView->setColumnWidth(LayoutsTableModel::LAYOUT_COLUMN, 200);
 	uiWidget->layoutsTableView->setColumnWidth(LayoutsTableModel::VARIANT_COLUMN, 250);
 
 	connect(layoutsTableModel, SIGNAL(dataChanged(const QModelIndex &, const QModelIndex &)), this, SLOT(uiChanged()));
@@ -241,7 +248,8 @@ void KCMKeyboardWidget::initializeLayoutsUI()
 
 	connect(uiWidget->showIndicatorChk, SIGNAL(clicked(bool)), this, SLOT(uiChanged()));
 	connect(uiWidget->showIndicatorChk, SIGNAL(toggled(bool)), uiWidget->showSingleChk, SLOT(setEnabled(bool)));
-	connect(uiWidget->showFlagChk, SIGNAL(clicked(bool)), this, SLOT(uiChanged()));
+	connect(uiWidget->showFlagRadioBtn, SIGNAL(clicked(bool)), this, SLOT(uiChanged()));
+	connect(uiWidget->showLabelRadioBtn, SIGNAL(clicked(bool)), this, SLOT(uiChanged()));
 	connect(uiWidget->showSingleChk, SIGNAL(toggled(bool)), this, SLOT(uiChanged()));
 }
 
@@ -353,12 +361,18 @@ void KCMKeyboardWidget::moveSelectedLayouts(int shift)
 void KCMKeyboardWidget::scrollToGroupShortcut()
 {
     this->setCurrentIndex(TAB_ADVANCED);
+    if( ! uiWidget->configureKeyboardOptionsChk->isChecked() ) {
+    	uiWidget->configureKeyboardOptionsChk->setChecked(true);
+    }
 	((XkbOptionsTreeModel*)uiWidget->xkbOptionsTreeView->model())->gotoGroup(GROUP_SWITCH_GROUP_NAME, uiWidget->xkbOptionsTreeView);
 }
 
 void KCMKeyboardWidget::scrollTo3rdLevelShortcut()
 {
 	this->setCurrentIndex(TAB_ADVANCED);
+    if( ! uiWidget->configureKeyboardOptionsChk->isChecked() ) {
+    	uiWidget->configureKeyboardOptionsChk->setChecked(true);
+    }
 	((XkbOptionsTreeModel*)uiWidget->xkbOptionsTreeView->model())->gotoGroup(LV3_SWITCH_GROUP_NAME, uiWidget->xkbOptionsTreeView);
 }
 
@@ -489,7 +503,8 @@ void KCMKeyboardWidget::updateLayoutsUI()
 	uiWidget->configureLayoutsChk->setChecked(keyboardConfig->configureLayouts);
 	uiWidget->showIndicatorChk->setChecked(keyboardConfig->showIndicator);
 	uiWidget->showSingleChk->setChecked(keyboardConfig->showSingle);
-	uiWidget->showFlagChk->setChecked(keyboardConfig->showFlag);
+	uiWidget->showFlagRadioBtn->setChecked(keyboardConfig->showFlag);
+	uiWidget->showLabelRadioBtn->setChecked(!keyboardConfig->showFlag);
 }
 
 void KCMKeyboardWidget::updateHardwareUI()
