@@ -426,18 +426,27 @@ void RandROutput::save(KConfig &config)
 
 void RandROutput::proposeRefreshRate(float rate)
 {
+	if (!m_crtc->isValid())
+		slotEnable();
+	
 	m_originalRate = refreshRate();
 	m_proposedRate = rate;
 }
 
 void RandROutput::proposeRect(const QRect &r)
 {
+	if (!m_crtc->isValid())
+		slotEnable();
+	
 	m_originalRect = rect();
 	m_proposedRect = r;
 }
 
 void RandROutput::proposeRotation(int r)
 {
+	if (!m_crtc->isValid())
+		slotEnable();
+	
 	m_originalRotation = rotation();
 	m_proposedRotation = r;
 }
@@ -602,7 +611,6 @@ bool RandROutput::applyProposed(int changes, bool confirm)
 
 bool RandROutput::setCrtc(RandRCrtc *crtc, bool applyNow)
 {
-    Q_UNUSED(applyNow);
 	if( !crtc || (m_crtc && crtc->id() == m_crtc->id()) )
 		return false;
 	
@@ -615,7 +623,8 @@ bool RandROutput::setCrtc(RandRCrtc *crtc, bool applyNow)
 		           this, SLOT(slotCrtcChanged(RRCrtc, int)));
 				 
 		m_crtc->removeOutput(m_id);
-		m_crtc->applyProposed();
+		if( applyNow )
+			m_crtc->applyProposed();
 	}
 	m_crtc = crtc;
 	if (!m_crtc->isValid())
@@ -628,6 +637,11 @@ bool RandROutput::setCrtc(RandRCrtc *crtc, bool applyNow)
 	return true;
 }
 
+void RandROutput::disconnectFromCrtc()
+{ // but don't apply now
+	setCrtc(m_screen->crtc(None), false);
+}
+
 void RandROutput::slotCrtcChanged(RRCrtc c, int changes)
 {
 	Q_UNUSED(c);
