@@ -43,6 +43,7 @@
 #include <KLocale>
 #include <KGlobalSettings>
 #include <KPushButton>
+#include <KStyle>
 #include <KTitleWidget>
 #include <KWindowSystem>
 
@@ -172,9 +173,9 @@ Interface::Interface(Plasma::RunnerManager *runnerManager, QWidget *parent)
 
     KrunnerTabFilter *krunnerTabFilter = new KrunnerTabFilter(m_resultsScene, lineEdit, this);
     m_searchTerm->installEventFilter(krunnerTabFilter);
-    m_singleRunnerSearchTerm->installEventFilter(krunnerTabFilter);
     connect(krunnerTabFilter, SIGNAL(historyRotated(QString)), this, SLOT(queryTextEdited(QString)));
     lineEdit->installEventFilter(this);
+    m_singleRunnerSearchTerm->installEventFilter(this);
 
     themeUpdated();
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeUpdated()));
@@ -373,7 +374,7 @@ void Interface::resetInterface()
         m_resultsScene->clearQuery();
     }
     resetResultsArea();
-    resize(qMax(minimumSizeHint().width(), m_defaultSize.width()), minimumSizeHint().height());
+    m_minimumHeight = height();
 }
 
 void Interface::showHelp()
@@ -472,6 +473,10 @@ void Interface::run(ResultItem *item)
         return;
     }
 
+    // We need to keep a copy of the query text before calling close(),
+    // in order to add it to history later on.
+    QString currentQuery = m_searchTerm->currentText().trimmed();
+
     m_running = true;
     // must run the result first before clearing the interface
     // in a way that will cause the results scene to be cleared and
@@ -483,7 +488,7 @@ void Interface::run(ResultItem *item)
     //TODO: check if run is succesful before adding the term to history
     // Notice that we must add items to history after calling m_resultScene->run()
     // otherwise the query result could be reset to QString() too early.
-    m_searchTerm->addToHistory(m_searchTerm->currentText().trimmed());
+    m_searchTerm->addToHistory(currentQuery);
 
     resetInterface();
 }
@@ -546,13 +551,26 @@ void Interface::matchCountChanged(int count)
         //kDebug() << "showing!" << minimumSizeHint();
 
         QSize s = m_defaultSize;
-        const int minHeight = minimumSizeHint().height();
         const int resultsHeight = m_resultsScene->viewableHeight() + 2;
-        //kDebug() << minHeight << resultsHeight << s.height();
+        int spacing = m_layout->spacing();
+        if (spacing < 0) {
+            // KStyles allow for variable spacing via the layoutSpacingImplementation() method;
+            // in this case m_layout->spacing() returns -1 and we should ask for the
+            // spacing by ourselves.
+            // This is quite ugly, but at least gives the right guess, so that we avoid
+            // multiple resize events
+            spacing = style()->layoutSpacing(QSizePolicy::DefaultType, QSizePolicy::DefaultType, Qt::Vertical);
+        }
 
-        if (minHeight + resultsHeight + m_layout->spacing() < s.height()) {
-            s.setHeight(minHeight + resultsHeight + m_layout->spacing());
+        //kDebug() << m_minimumHeight << resultsHeight << spacing << s.height();
+
+        if (m_minimumHeight + resultsHeight + spacing < s.height()) {
+            s.setHeight(m_minimumHeight + resultsHeight + spacing);
             m_resultsView->setMinimumHeight(resultsHeight);
+            // The layout will activate on the next event cycle, but
+            // we need to update the minimum size now, as we are going to
+            // resize the krunner window right away.
+            m_layout->activate();
         }
 
         resize(s);
@@ -579,14 +597,13 @@ void Interface::hideResultsArea()
 {
     searchTermSetFocus();
     resetResultsArea();
-    resize(qMax(minimumSizeHint().width(), m_defaultSize.width()), minimumSizeHint().height());
 }
 
 void Interface::resetResultsArea()
 {
     m_resultsView->hide();
     setMinimumSize(QSize(MIN_WIDTH, 0));
-    adjustSize();
+    resize(qMax(minimumSizeHint().width(), m_defaultSize.width()), minimumSizeHint().height());
 }
 
 #include "interface.moc"
