@@ -29,13 +29,15 @@
 #include <Plasma/Svg>
 
 
+#include "resultscene.h"
 #include "resultsview.h"
 #include "resultitem.h"
 
-ResultsView::ResultsView(QWidget * parent)
-    : QGraphicsView(parent)
+ResultsView::ResultsView(ResultScene *scene, SharedResultData *resultData, QWidget *parent)
+    : QGraphicsView(scene, parent),
+      m_resultScene(scene),
+      m_resultData(resultData)
 {
-
     setFrameStyle(QFrame::NoFrame);
     viewport()->setAutoFillBackground(false);
     setInteractive(true);
@@ -45,46 +47,48 @@ ResultsView::ResultsView(QWidget * parent)
     setAlignment(Qt::AlignLeft | Qt::AlignTop);
 
     m_arrowSvg = new Plasma::Svg(this);
-    {
-        // lock because setImagePath uses KSycoca
-        QMutexLocker lock(Plasma::AbstractRunner::bigLock());
-        m_arrowSvg->setImagePath("widgets/arrows");
-    }
+    m_arrowSvg->setImagePath("widgets/arrows");
 
     m_previousPage = new QToolButton(this);
-    m_previousPage->setIcon(m_arrowSvg->pixmap("up-arrow"));
     m_previousPage->setAutoRaise(true);
     m_previousPage->setVisible(false);
-    m_previousPage->adjustSize();
     connect(m_previousPage, SIGNAL(clicked(bool)), SLOT(previousPage()));
 
     m_nextPage = new QToolButton(this);
-    m_nextPage->setIcon(m_arrowSvg->pixmap("down-arrow"));
     m_nextPage->setAutoRaise(true);
     m_nextPage->setVisible(false);
-    m_nextPage->adjustSize();
     connect(m_nextPage, SIGNAL(clicked(bool)), SLOT(nextPage()));
 
-    updateArrowsVisibility();
+    connect(m_arrowSvg, SIGNAL(repaintNeeded()), this, SLOT(updateArrowsIcons()));
+    updateArrowsIcons();
 
     connect(verticalScrollBar(), SIGNAL(rangeChanged(int, int)), this, SLOT(updateArrowsVisibility()));
     connect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(updateArrowsVisibility()));
+    connect(m_resultScene, SIGNAL(ensureVisibility(QGraphicsItem *)), this, SLOT(ensureVisibility(QGraphicsItem *)));
 }
 
 ResultsView::~ResultsView()
 {
 }
 
+void ResultsView::ensureVisibility(QGraphicsItem* item)
+{
+    m_resultData->processHoverEvents = false;
+    ensureVisible(item, 0, 0);
+    m_resultData->processHoverEvents = true;
+}
+
 void ResultsView::previousPage()
 {
-    QGraphicsItem * currentItem = scene()->selectedItems().first();
-    QGraphicsItem * item = itemAt(0,-height()*0.4);
+    QGraphicsItem *currentItem = m_resultScene->selectedItems().first();
+    QGraphicsItem *item = itemAt(0, -height()*0.4);
 
     if (!item) {
-        item = scene()->itemAt(0,0);
+        item = m_resultScene->itemAt(0,0);
     }
+
     if (item && (item != currentItem)) {
-        scene()->setFocusItem(item);
+        m_resultScene->setFocusItem(item);
     } else {
         verticalScrollBar()->setValue(verticalScrollBar()->value()-height()*0.4);
     }
@@ -93,33 +97,58 @@ void ResultsView::previousPage()
 
 void ResultsView::nextPage()
 {
-    QGraphicsItem * currentItem = scene()->selectedItems().first();
-    QGraphicsItem * item = itemAt(0,height()*1.4);
+    QGraphicsItem *currentItem = m_resultScene->selectedItems().first();
+    QGraphicsItem *item = itemAt(0, height()*1.4);
     if (!item) {
-        item = scene()->itemAt(0,sceneRect().height()-1);
+        item = m_resultScene->itemAt(0,sceneRect().height()-1);
     }
+
+    ResultItem *rItem = dynamic_cast<ResultItem *>(item);
+    if (rItem && !rItem->isValid()) {
+        item = m_resultScene->itemAt(0, m_resultScene->viewableHeight() - 1);
+    }
+
     if (item && (item != currentItem)) {
-        scene()->setFocusItem(item);
+        m_resultScene->setFocusItem(item);
     } else {
-        verticalScrollBar()->setValue(verticalScrollBar()->value()+height()*0.4);
+        verticalScrollBar()->setValue(qMin(m_resultScene->viewableHeight(),
+                                           int(verticalScrollBar()->value()+ (height() * 0.4))));
     }
 }
 
-void ResultsView::resizeEvent( QResizeEvent * event)
+void ResultsView::resizeEvent(QResizeEvent * event)
 {
     updateArrowsVisibility();
     QGraphicsView::resizeEvent(event);
 }
 
+void ResultsView::updateArrowsIcons()
+{
+    m_previousPage->setIcon(m_arrowSvg->pixmap("up-arrow"));
+    m_previousPage->adjustSize();
+
+    m_nextPage->setIcon(m_arrowSvg->pixmap("down-arrow"));
+    m_nextPage->adjustSize();
+
+    updateArrowsVisibility();
+}
+
 void ResultsView::updateArrowsVisibility()
 {
     m_previousPage->move(width()/2-m_previousPage->width()/2,0);
     m_nextPage->move(width()/2-m_nextPage->width()/2,height()-m_nextPage->height());
 
-    if (scene()) {
-        m_previousPage->setVisible(mapFromScene(QPointF(0,0)).y()<0);
-        m_nextPage->setVisible(mapFromScene(QPointF(0,sceneRect().height())).y()>height());
+    m_previousPage->setVisible(mapFromScene(QPointF(0,0)).y()<0);
+    m_nextPage->setVisible(mapFromScene(QPointF(0, m_resultScene->viewableHeight())).y() > height());
+}
+
+void ResultsView::wheelEvent(QWheelEvent *e)
+{
+    if (e->delta() < 0 && !m_nextPage->isVisible()) {
+        return;
     }
+
+    QGraphicsView::wheelEvent(e);
 }
 
 void ResultsView::paintEvent(QPaintEvent *event)
@@ -141,13 +170,9 @@ void ResultsView::paintEvent(QPaintEvent *event)
 
     if (m_previousFadeout.isNull() || m_previousFadeout.width() != width()) {
         QLinearGradient g(0, 0, 0, m_previousPage->height());
-        QColor col=Qt::white;
-        g.setColorAt(1, col);
-        col.setAlphaF(0.25);
-        g.setColorAt(0.5, col);
-        col.setAlphaF(0);
-        g.setColorAt(0, col);
-        m_previousFadeout = QPixmap(width(), m_previousPage->height()*1.5);
+        g.setColorAt(1, Qt::white );
+        g.setColorAt(0, Qt::transparent );
+        m_previousFadeout = QPixmap(width(), m_previousPage->height());
         m_previousFadeout.fill(Qt::transparent);
         QPainter p(&m_previousFadeout);
         p.setCompositionMode(QPainter::CompositionMode_Source);
@@ -156,13 +181,9 @@ void ResultsView::paintEvent(QPaintEvent *event)
 
     if (m_nextFadeout.isNull() || m_nextFadeout.width() != width()) {
         QLinearGradient g(0, 0, 0, m_nextPage->height());
-        QColor col=Qt::white;
-        g.setColorAt(0, col);
-        col.setAlphaF(0.25);
-        g.setColorAt(0.5, col);
-        col.setAlphaF(0);
-        g.setColorAt(1, col);
-        m_nextFadeout = QPixmap(width(), m_nextPage->height()*1.5);
+        g.setColorAt(0, Qt::white );
+        g.setColorAt(1, Qt::transparent );
+        m_nextFadeout = QPixmap(width(), m_nextPage->height());
         m_nextFadeout.fill(Qt::transparent);
         QPainter p(&m_nextFadeout);
         p.setCompositionMode(QPainter::CompositionMode_Source);
@@ -173,6 +194,7 @@ void ResultsView::paintEvent(QPaintEvent *event)
     if (m_previousPage->isVisible()) {
         backBufferPainter.drawPixmap(QPoint(0,0), m_previousFadeout);
     }
+
     if (m_nextPage->isVisible()) {
         backBufferPainter.drawPixmap(QPoint(0,height()-m_nextFadeout.height()), m_nextFadeout);
     }
