@@ -31,12 +31,14 @@
 #include <QTimeLine>
 #include <QTimer>
 #include <QToolButton>
+#include <QShortcut>
+#include <QClipboard>
+#include <QApplication>
 
 #include <KDebug>
 #include <KIcon>
 #include <KLineEdit>
 #include <KLocale>
-#include <KWindowSystem>
 
 #include <Plasma/Theme>
 
@@ -109,6 +111,7 @@ QsMatchView::QsMatchView(QWidget *parent)
     d->m_currentItem = 0;
 
     d->m_lineEdit = new KLineEdit(this);
+    new QShortcut(QKeySequence("Ctrl+V"), this, SLOT(pasteClipboard()));
     d->m_compBox = new QuickSand::QsCompletionBox(this);
     d->m_compBox->setTabHandling(false);
 
@@ -117,8 +120,15 @@ QsMatchView::QsMatchView(QWidget *parent)
     d->m_stack->addWidget(d->m_lineEdit);
     d->m_stack->setCurrentIndex(0);
 
+    const QColor textColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
     d->m_titleLabel = new QLabel(this);
     d->m_itemCountLabel = new QLabel(this);
+
+    QPalette labelPallete =  d->m_itemCountLabel->palette();
+    labelPallete.setColor(QPalette::WindowText, textColor);
+    d->m_itemCountLabel->setPalette(labelPallete);
+    d->m_titleLabel->setPalette(labelPallete);
+
     d->m_itemCountSuffixItems = true;
 
     d->m_arrowButton = new QToolButton(this);
@@ -131,12 +141,16 @@ QsMatchView::QsMatchView(QWidget *parent)
     d->m_arrowButton->setStyleSheet(buttonStyleSheet);
 
     QHBoxLayout *topLayout = new QHBoxLayout();
+    topLayout->setSpacing(0);
+    topLayout->setMargin(0);
     topLayout->addWidget(d->m_titleLabel);
     topLayout->addStretch();
     topLayout->addWidget(d->m_itemCountLabel);
     topLayout->addWidget(d->m_arrowButton);
 
     QVBoxLayout *layout = new QVBoxLayout(this);
+    layout->setSpacing(0);
+    layout->setMargin(0);
     layout->addLayout(topLayout);
     layout->addWidget(d->m_stack);
 
@@ -273,8 +287,8 @@ void QsMatchView::setCountingActions(bool actions)
 
 void QsMatchView::setDescriptionText(const QString &text)
 {
-    QColor color(Qt::white);
-    setDescriptionText(text, color);
+    const QColor textColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
+    setDescriptionText(text, textColor);
 }
 
 void QsMatchView::setDescriptionText(const QString &text, const QColor &color)
@@ -342,9 +356,9 @@ void QsMatchView::toggleView()
 void QsMatchView::showLoading()
 {
     clear(true);
-
+    const QColor textColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
     d->m_descText = new QGraphicsTextItem(i18n("Loading..."), d->m_descRect);
-    d->m_descText->setDefaultTextColor(QColor(Qt::white));
+    d->m_descText->setDefaultTextColor(textColor);
     QFontMetrics fm(d->m_descText->font());
 
     //Center text
@@ -394,6 +408,7 @@ void QsMatchView::showSelected()
     d->m_stack->setCurrentIndex(0);
 
     QGraphicsPixmapItem *pixmap = new QGraphicsPixmapItem(it->icon().pixmap(64));
+
     pixmap->setPos(-WIDTH/2 + 5, LARGE_ICON_PADDING);
 
     Plasma::Theme *theme = Plasma::Theme::defaultTheme();
@@ -441,7 +456,7 @@ void QsMatchView::focusItem(int index)
         } else {
             description = QString("%1 (%2)").arg(it->name()).arg(it->description());
         }
-        setDescriptionText(description, it->backgroundColor());
+        setDescriptionText(description);
         emit selectionChanged(it);
     }
 }
@@ -590,6 +605,17 @@ void QsMatchView::focusOutEvent(QFocusEvent *event)
     showSelected();
 }
 
+void QsMatchView::pasteClipboard()
+{
+    QClipboard *clip = qApp->clipboard();
+    QString text = clip->text();
+
+    if (!text.isEmpty()) {
+        d->m_lineEdit->setText(text);
+        d->m_searchTerm = text;
+    }
+}
+
 //TODO: Make it possible to disable text mode
 void QsMatchView::keyPressEvent(QKeyEvent *e)
 {
@@ -659,18 +685,6 @@ void QsMatchView::keyPressEvent(QKeyEvent *e)
     QWidget::keyPressEvent(e);
 }
 
-bool QsMatchView::eventFilter(QObject *obj, QEvent *event)
-{
-    if (event->type() == QEvent::MouseButtonPress) {
-        if (KWindowSystem::activeWindow() != winId()) {
-            // this overcomes problems with click-to-focus and being a Dock window
-            KWindowSystem::forceActiveWindow(winId());
-        }
-    }
-
-    return QWidget::eventFilter(obj, event);
-}
-
 } // namespace QuickSand
 
 #include "qs_matchview.moc"
