@@ -51,9 +51,9 @@
 #endif
 
 KRunnerDialog::KRunnerDialog(Plasma::RunnerManager *runnerManager, QWidget *parent, Qt::WindowFlags f)
-    : KDialog(parent, f),
+    : QWidget(parent, f),
       m_runnerManager(runnerManager),
-      m_configDialog(0),
+      m_configWidget(0),
       m_lastPressPos(-1),
       m_oldScreen(-1),
       m_floating(!KRunnerSettings::freeFloating()),
@@ -63,7 +63,7 @@ KRunnerDialog::KRunnerDialog(Plasma::RunnerManager *runnerManager, QWidget *pare
 {
     setAttribute(Qt::WA_TranslucentBackground);
     setMouseTracking(true);
-    setButtons(0);
+    //setButtons(0);
     setWindowTitle(i18n("Run Command"));
     setWindowIcon(KIcon("system-run"));
 
@@ -73,7 +73,6 @@ KRunnerDialog::KRunnerDialog(Plasma::RunnerManager *runnerManager, QWidget *pare
 
     m_iconSvg = new Plasma::Svg(this);
     m_iconSvg->setImagePath("widgets/configuration-icons");
-    m_iconSvg->setContainsMultipleImages(true);
 
     m_background = new Plasma::FrameSvg(this);
     m_background->setImagePath("dialogs/krunner");
@@ -87,21 +86,19 @@ KRunnerDialog::KRunnerDialog(Plasma::RunnerManager *runnerManager, QWidget *pare
             this, SLOT(screenChanged(Kephal::Screen*)));
     connect(KWindowSystem::self(), SIGNAL(workAreaChanged()), this, SLOT(resetScreenPos()));
 
-    connect(m_background, SIGNAL(repaintNeeded()), this, SLOT(update()));
-
-    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeUpdated()));
+    connect(m_background, SIGNAL(repaintNeeded()), this, SLOT(themeUpdated()));
     themeUpdated();
 }
 
 KRunnerDialog::~KRunnerDialog()
 {
-    kDebug( )<< "!!!!!!!!!! deleting" << m_floating << m_screenPos.count();
+    //kDebug( )<< "!!!!!!!!!! deleting" << m_floating << m_screenPos.count();
     if (!m_floating) {
         KConfigGroup cg(KGlobal::config(), "EdgePositions");
         QHashIterator<int, QPoint> it(m_screenPos);
         while (it.hasNext()) {
             it.next();
-            kDebug() << "saving" << "Screen" + QString::number(it.key()) << it.value();
+            //kDebug() << "saving" << "Screen" + QString::number(it.key()) << it.value();
             cg.writeEntry("Screen" + QString::number(it.key()), it.value());
         }
     }
@@ -159,14 +156,28 @@ void KRunnerDialog::positionOnScreen()
     if (m_oldScreen != screen) {
         //kDebug() << "old screen be the new screen" << m_oldScreen << screen;
         if (m_oldScreen != -1) {
-            m_screenPos.insert(m_oldScreen, pos());
+            QRect oldRect = Kephal::ScreenUtils::screenGeometry(m_oldScreen);
+            // Store the position relative to the screen topLeft corner.
+            // Since the geometry of screens might change between sessions
+            // storing the absolute position might lead to issues such as bug #243898
+            m_screenPos.insert(m_oldScreen, pos() - oldRect.topLeft());
         }
 
         m_oldScreen = screen;
         if (m_screenPos.contains(screen)) {
             //kDebug() << "moving to" << m_screenPos[screen];
-            move(m_screenPos[screen]);
-        } else {
+
+            // Checks that the stored position is still a valid position on screen
+            // if not, remove the stored position so that it is reset later
+            if (r.contains(m_screenPos[screen] + r.topLeft()) &&
+                r.contains(m_screenPos[screen] + r.topLeft() + QPoint(width()-1, 0))) {
+                move(m_screenPos[screen] + r.topLeft());
+            } else {
+                m_screenPos.remove(screen);
+            }
+        }
+
+        if (!m_screenPos.contains(screen)) {
             const int w = width();
             const int dx = r.left() + (r.width() / 2) - (w / 2);
             int dy = r.top();
@@ -186,7 +197,6 @@ void KRunnerDialog::positionOnScreen()
 
     if (m_floating) {
         KWindowSystem::setOnDesktop(winId(), KWindowSystem::currentDesktop());
-        Plasma::WindowEffects::slideWindow(this, Plasma::Floating);
     } else {
         KWindowSystem::setOnAllDesktops(winId(), true);
         Plasma::WindowEffects::slideWindow(this, Plasma::TopEdge);
@@ -198,7 +208,7 @@ void KRunnerDialog::positionOnScreen()
 
 void KRunnerDialog::moveEvent(QMoveEvent *)
 {
-    m_screenPos.insert(m_oldScreen, pos());
+    m_screenPos.insert(m_oldScreen, pos() - Kephal::ScreenUtils::screenGeometry(m_oldScreen).topLeft());
 }
 
 void KRunnerDialog::setFreeFloating(bool floating)
@@ -210,9 +220,11 @@ void KRunnerDialog::setFreeFloating(bool floating)
     m_floating = floating;
     m_screenPos.clear();
     m_oldScreen = -1;
+    unsetCursor();
 
     if (m_floating) {
         m_background->setEnabledBorders(Plasma::FrameSvg::AllBorders);
+        KWindowSystem::setType(winId(), NET::Normal);
     } else {
         // load the positions for each screen from our config
         const int numScreens = Kephal::ScreenUtils::numScreens();
@@ -224,9 +236,13 @@ void KRunnerDialog::setFreeFloating(bool floating)
                 m_screenPos.insert(i, QPoint(p.x(), r.top()));
             }
         }
+        QRect r = Kephal::ScreenUtils::screenGeometry(m_oldScreen > -1 ? m_oldScreen : 0);
+        checkBorders(r);
+        KWindowSystem::setType(winId(), NET::Dock);
     }
 
-    m_iconSvg->resize(KIconLoader::SizeSmall, KIconLoader::SizeSmall);
+    // recalc the contents margins
+    themeUpdated();
 
     if (isVisible()) {
         positionOnScreen();
@@ -278,35 +294,52 @@ void KRunnerDialog::switchUser()
     }
 }
 
-void KRunnerDialog::showConfigDialog()
+void KRunnerDialog::toggleConfigDialog()
 {
-    if (!m_configDialog) {
-        m_configDialog = new KRunnerConfigDialog(m_runnerManager, this);
-        connect(this, SIGNAL(okClicked()), m_configDialog, SLOT(accept()));
-        setButtons(Ok | Cancel);
-        setConfigWidget(m_configDialog);
+    if (m_configWidget) {
+        delete m_configWidget;
+        m_configWidget = 0;
+
+        if (!m_floating) {
+            KWindowSystem::setType(winId(), NET::Dock);
+        }
+    } else {
+        m_configWidget = new KRunnerConfigWidget(m_runnerManager, this);
+        connect(m_configWidget, SIGNAL(finished()), this, SLOT(configCompleted()));
+        setConfigWidget(m_configWidget);
+        KWindowSystem::setType(winId(), NET::Normal);
     }
 }
 
 void KRunnerDialog::configCompleted()
 {
-    if (m_configDialog) {
-        disconnect(this, SIGNAL(finished()), this, SLOT(configCompleted()));
-        setButtons(0);
-        m_configDialog->deleteLater();
-        m_configDialog = 0;
+    if (m_configWidget) {
+        m_configWidget->deleteLater();
+        m_configWidget = 0;
+    }
+
+    if (!m_floating) {
+        KWindowSystem::setType(winId(), NET::Dock);
     }
 }
 
 void KRunnerDialog::themeUpdated()
 {
-    int margin = marginHint();
-    const int topHeight = qMax(0, int(m_background->marginSize(Plasma::TopMargin)) - margin);
-    m_leftBorderWidth = qMax(0, int(m_background->marginSize(Plasma::LeftMargin)) - margin);
-    m_rightBorderWidth = qMax(0, int(m_background->marginSize(Plasma::RightMargin)) - margin);
-    m_bottomBorderHeight = qMax(0, int(m_background->marginSize(Plasma::BottomMargin)) - margin);
+    m_leftBorderWidth = qMax(0, int(m_background->marginSize(Plasma::LeftMargin)));
+    m_rightBorderWidth = qMax(0, int(m_background->marginSize(Plasma::RightMargin)));
+    m_bottomBorderHeight = qMax(0, int(m_background->marginSize(Plasma::BottomMargin)));
+    // the -2 in the non-floating case is not optimal, but it gives it a bit of a "more snug to the
+    // top" feel; best would be if we could tell exactly where the edge/shadow of the frame svg was
+    // but this works nicely
+    const int topHeight = m_floating ? qMax(0, int(m_background->marginSize(Plasma::TopMargin)))
+                                     : Plasma::Theme::defaultTheme()->windowTranslucencyEnabled() ?
+                                         qMax(1, m_bottomBorderHeight / 2)
+                                       : qMax(1, m_bottomBorderHeight - 2);
 
-    setContentsMargins(m_leftBorderWidth, topHeight, m_rightBorderWidth, m_bottomBorderHeight);
+    //kDebug() << m_leftBorderWidth<< topHeight<< m_rightBorderWidth<< m_bottomBorderHeight;
+    // the +1 gives us the extra mouseMoveEvent needed to always reset the resize cursor
+    setContentsMargins(m_leftBorderWidth + 1, topHeight, m_rightBorderWidth + 1, m_bottomBorderHeight + 1);
+    update();
 }
 
 void KRunnerDialog::paintEvent(QPaintEvent *e)
@@ -327,7 +360,7 @@ bool KRunnerDialog::event(QEvent *event)
         p.fillRect(rect(), Qt::transparent);
     }
 
-    return KDialog::event(event);
+    return QWidget::event(event);
 }
 
 void KRunnerDialog::showEvent(QShowEvent *)
@@ -341,24 +374,11 @@ void KRunnerDialog::showEvent(QShowEvent *)
     m_runnerManager->setupMatchSession();
 }
 
-void KRunnerDialog::slotButtonClicked(int button)
-{
-    if (button == KDialog::Ok) {
-        if (m_configDialog) {
-            m_configDialog->accept();
-        }
-        configCompleted();
-    } else if (button == KDialog::Cancel) {
-        configCompleted();
-    } else {
-        KDialog::slotButtonClicked(button);
-    }
-}
-
-void KRunnerDialog::hideEvent(QHideEvent *event)
+void KRunnerDialog::hideEvent(QHideEvent *)
 {
     m_runnerManager->matchSessionComplete();
-    KDialog::hideEvent(event);
+    delete m_configWidget;
+    m_configWidget = 0;
 }
 
 void KRunnerDialog::updateMask()
@@ -371,12 +391,9 @@ void KRunnerDialog::updateMask()
     if (!QX11Info::isCompositingManagerRunning()) {
         setMask(m_background->mask());
     } else {
-        //HACK: we set a trivial mask in order to fool kwin
-        //      to think we are shaped and thus avoid drawing
-        //      shadows behind us.
-        //      This is also done in Oxygen and Aurorae windecos
-        //      to obtain the same result
-        setMask(QRect(QPoint(0, 0), size()));
+        const QRegion mask = m_background->mask();
+        Plasma::WindowEffects::enableBlurBehind(winId(), true, mask);
+        Plasma::WindowEffects::overrideShadow(winId(), true);
     }
 
 #else
@@ -395,8 +412,8 @@ void KRunnerDialog::resizeEvent(QResizeEvent *e)
         if (borders & Plasma::FrameSvg::LeftBorder) {
             const int dx = x() + (e->oldSize().width() / 2) - (width() / 2);
             int dy = r.top();
-            move(qBound(r.left(), dx, r.right() - width()), dy);
-            m_screenPos.insert(m_oldScreen, pos());
+            move(qBound(r.left(), dx, r.right() - width() + 1), dy);
+            m_screenPos.insert(m_oldScreen, pos() - Kephal::ScreenUtils::screenGeometry(m_oldScreen).topLeft());
             if (!checkBorders(r)) {
                 updateMask();
             }
@@ -406,8 +423,6 @@ void KRunnerDialog::resizeEvent(QResizeEvent *e)
     } else {
         updateMask();
     }
-
-    KDialog::resizeEvent(e);
 }
 
 void KRunnerDialog::mousePressEvent(QMouseEvent *e)
@@ -476,6 +491,11 @@ bool KRunnerDialog::checkBorders(const QRect &screenGeom)
     return false;
 }
 
+void KRunnerDialog::leaveEvent(QEvent *)
+{
+    unsetCursor();
+}
+
 void KRunnerDialog::mouseMoveEvent(QMouseEvent *e)
 {
     //kDebug() << e->x() << m_leftBorderWidth << width() << m_rightBorderWidth;
@@ -493,16 +513,15 @@ void KRunnerDialog::mouseMoveEvent(QMouseEvent *e)
                 QRect r = Kephal::ScreenUtils::screenGeometry(m_oldScreen);
                 const int deltaX = (m_rightResize ? -1 : 1) * (m_lastPressPos - e->globalX());
                 int newWidth = width() + deltaX;
-                const Plasma::FrameSvg::EnabledBorders borders = m_background->enabledBorders();
                 // don't let it grow beyond the opposite screen edge
                 if (m_rightResize) {
-                    if (borders & Plasma::FrameSvg::LeftBorder) {
+                    if (m_leftBorderWidth > 0) {
                         newWidth += qMin(deltaX, x() - r.left());
                     }
-                } else if (borders & Plasma::FrameSvg::RightBorder) {
+                } else if (m_rightBorderWidth > 0) {
                     newWidth += qBound(0, deltaX, r.right() - (x() + width() - 1));
                 } else if (newWidth > minimumWidth() && newWidth < width()) {
-                    move(x() - deltaX, y());
+                    move(r.right() - newWidth + 1, y());
                 }
 
                 if (newWidth > minimumWidth()) {
@@ -512,7 +531,7 @@ void KRunnerDialog::mouseMoveEvent(QMouseEvent *e)
             }
         } else {
             QRect r = Kephal::ScreenUtils::screenGeometry(m_oldScreen);
-            int newX = qBound(r.left(), x() - (m_lastPressPos - e->globalX()), r.right() - width());
+            int newX = qBound(r.left(), x() - (m_lastPressPos - e->globalX()), r.right() - width() + 1);
             if (abs(r.center().x() - (newX + (width() / 2))) < 20) {
                 newX = r.center().x() - (width() / 2);
             } else {
@@ -522,13 +541,43 @@ void KRunnerDialog::mouseMoveEvent(QMouseEvent *e)
             move(newX, y());
             checkBorders(r);
         }
-    } else if (e->x() < qMax(5, m_leftBorderWidth) || e->x() > width() - qMax(5, m_rightBorderWidth)) {
-        setCursor(Qt::SizeHorCursor);
-    } else if ((e->y() > height() - qMax(5, m_bottomBorderHeight)) && (e->y() < height())) {
-        setCursor(Qt::SizeVerCursor);
     } else {
-        unsetCursor();
+        checkCursor(e->pos());
     }
 }
 
+void KRunnerDialog::timerEvent(QTimerEvent *event)
+{
+    killTimer(event->timerId());
+    if (checkCursor(mapFromGlobal(QCursor::pos()))) {
+        startTimer(100);
+    }
+}
+
+bool KRunnerDialog::checkCursor(const QPoint &pos)
+{
+    //Plasma::FrameSvg borders = m_background->enabledBoders();
+    if ((m_leftBorderWidth > 0 && pos.x() < qMax(5, m_leftBorderWidth)) ||
+        (m_rightBorderWidth > 0 && pos.x() > width() - qMax(5, m_rightBorderWidth))) {
+        if (cursor().shape() != Qt::SizeHorCursor) {
+            setCursor(Qt::SizeHorCursor);
+            startTimer(100);
+            return false;
+        }
+
+        return true;
+    } else if ((pos.y() > height() - qMax(5, m_bottomBorderHeight)) && (pos.y() < height())) {
+        if (cursor().shape() != Qt::SizeVerCursor) {
+            setCursor(Qt::SizeVerCursor);
+            startTimer(100);
+            return false;
+        }
+
+        return true;
+    }
+
+    unsetCursor();
+    return false;
+}
+
 #include "krunnerdialog.moc"
