@@ -30,9 +30,9 @@ namespace Plasma
     class Svg;
 }
 
-class KRunnerConfigDialog;
+class KRunnerConfigWidget;
 
-class KRunnerDialog : public KDialog
+class KRunnerDialog : public QWidget
 {
     Q_OBJECT
 
@@ -59,6 +59,7 @@ class KRunnerDialog : public KDialog
         void mousePressEvent(QMouseEvent *event);
         void mouseReleaseEvent(QMouseEvent *event);
         void mouseMoveEvent(QMouseEvent *event);
+        void leaveEvent(QEvent *e);
         bool event(QEvent *event);
         void showEvent(QShowEvent *);
         void hideEvent(QHideEvent *);
@@ -68,8 +69,8 @@ class KRunnerDialog : public KDialog
         virtual void setStaticQueryMode(bool staticQuery);
 
     protected Q_SLOTS:
-        void slotButtonClicked(int button);
-        void showConfigDialog();
+        void toggleConfigDialog();
+        void timerEvent(QTimerEvent *event);
 
         /**
          * React to configuration being done
@@ -95,10 +96,11 @@ class KRunnerDialog : public KDialog
 
     private:
         bool checkBorders(const QRect &screenGeom);
+        bool checkCursor(const QPoint &pos);
         void updateMask();
         void paintBackground(QPainter* painter, const QRect &exposedRect);
 
-        KRunnerConfigDialog *m_configDialog;
+        KRunnerConfigWidget *m_configWidget;
         Plasma::FrameSvg *m_background;
         QPixmap *m_cachedBackground;
         QHash<int, QPoint> m_screenPos;
