@@ -37,6 +37,7 @@
 #include <KGlobalSettings>
 #include <KLocale>
 #include <KLibrary>
+#include <KNotification>
 #include <KPushButton>
 #include <KStandardGuiItem>
 #include <KAuthorized>
@@ -44,6 +45,7 @@
 #include <kservicetypetrader.h>
 #include <kmacroexpander.h>
 #include <kshell.h>
+#include <kxerrorhandler.h>
 
 #include <QtGui/QFrame>
 #include <QLabel>
@@ -59,6 +61,7 @@
 #include <QDBusConnection>
 #include <QDBusConnectionInterface>
 #include <QDBusInterface>
+#include <QDBusServiceWatcher>
 
 #include <QDateTime>
 
@@ -123,6 +126,7 @@ int XSelectInput( Display* dpy, Window w, long e )
 }
 #endif
 
+static QString s_overlayServiceName("org.kde.plasma-overlay");
 
 //===========================================================================
 //
@@ -134,6 +138,7 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
       mLocked(false),
       mBusy(false),
       mPlasmaDBus(0),
+      mServiceWatcher(0),
       mSetupMode(false),
       mOpenGLVisual(false),
       child_saver(child),
@@ -204,6 +209,25 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
 
     mSuppressUnlock.setSingleShot(true);
     connect(&mSuppressUnlock, SIGNAL(timeout()), SLOT(deactivatePlasma()));
+
+    // read the initial information about all toplevel windows
+    Window r, p;
+    Window* real;
+    unsigned nreal;
+    if( XQueryTree( x11Info().display(), x11Info().appRootWindow(), &r, &p, &real, &nreal )
+        && real != NULL ) {
+        KXErrorHandler err; // ignore X errors here
+        for( unsigned i = 0; i < nreal; ++i ) {
+            XWindowAttributes winAttr;
+            if (XGetWindowAttributes(QX11Info::display(), real[ i ], &winAttr)) {
+                WindowInfo info;
+                info.window = real[ i ];
+                info.viewable = ( winAttr.map_state == IsViewable );
+                windowInfo.append( info ); // ordered bottom to top
+            }
+        }
+        XFree( real );
+    }
 }
 
 //---------------------------------------------------------------------------
@@ -245,29 +269,19 @@ void LockProcess::timerEvent(QTimerEvent *ev)
 void LockProcess::setupSignals()
 {
     struct sigaction act;
+    sigemptyset(&(act.sa_mask));
+    act.sa_flags = 0;
     // ignore SIGINT
     act.sa_handler=SIG_IGN;
-    sigemptyset(&(act.sa_mask));
-    sigaddset(&(act.sa_mask), SIGINT);
-    act.sa_flags = 0;
     sigaction(SIGINT, &act, 0L);
     // ignore SIGQUIT
-    act.sa_handler=SIG_IGN;
-    sigemptyset(&(act.sa_mask));
-    sigaddset(&(act.sa_mask), SIGQUIT);
-    act.sa_flags = 0;
+    //act.sa_handler=SIG_IGN;
     sigaction(SIGQUIT, &act, 0L);
     // exit cleanly on SIGTERM
     act.sa_handler= sigterm_handler;
-    sigemptyset(&(act.sa_mask));
-    sigaddset(&(act.sa_mask), SIGTERM);
-    act.sa_flags = 0;
     sigaction(SIGTERM, &act, 0L);
     // SIGHUP forces lock
     act.sa_handler= sighup_handler;
-    sigemptyset(&(act.sa_mask));
-    sigaddset(&(act.sa_mask), SIGHUP);
-    act.sa_flags = 0;
     sigaction(SIGHUP, &act, 0L);
 
     pipe(signal_pipe);
@@ -755,6 +769,7 @@ bool LockProcess::startSaver()
     setVRoot( winId(), winId() );
     startHack();
     startPlasma();
+    KNotification::event( "savingstarted" );
     return true;
 }
 
@@ -771,13 +786,17 @@ void LockProcess::stopSaver()
     hideSaverWindow();
     mVisibility = false;
     if (!child_saver) {
-        if (mLocked)
+        if (mLocked) {
             KDisplayManager().setLock( false );
+            mLocked = false;
+            KNotification::event( "unlocked" );
+        }
         ungrabInput();
         const char *out = "GOAWAY!";
         for (QList<int>::ConstIterator it = child_sockets.constBegin(); it != child_sockets.constEnd(); ++it)
             write(*it, out, sizeof(out));
     }
+    KNotification::event( "savingstopped" );
 }
 
 // private static
@@ -812,6 +831,7 @@ bool LockProcess::startLock()
         mLocked = true;
         KDisplayManager().setLock(true);
         lockPlasma();
+        KNotification::event( "locked" );
         return true;
     }
     return false;
@@ -916,7 +936,7 @@ void LockProcess::hackExited()
 {
     // Hack exited while we're supposed to be saving the screen.
     // Make sure the saver window is black.
-    XSetWindowBackground(QX11Info::display(), winId(), 0);
+    XSetWindowBackground(QX11Info::display(), winId(), BlackPixel( QX11Info::display(), QX11Info::appScreen()));
     XClearWindow(QX11Info::display(), winId());
 }
 
@@ -935,9 +955,8 @@ bool LockProcess::startPlasma()
     kDebug() << "looking for plasma-overlay";
     if (!mPlasmaDBus) {
         //try to get it, in case it's already running somehow
-        //FIXME I don't like hardcoded strings
-        //mPlasmaDBus = new QDBusInterface("org.kde.plasma-overlay", "/MainApplication", QString(),
-        mPlasmaDBus = new org::kde::plasmaoverlay::App("org.kde.plasma-overlay", "/App",
+        //mPlasmaDBus = new QDBusInterface(s_overlayServiceName, "/MainApplication", QString(),
+        mPlasmaDBus = new org::kde::plasmaoverlay::App(s_overlayServiceName, "/App",
                                                        QDBusConnection::sessionBus(), this);
         //FIXME this might-already-be-running stuff seems really really Wrong.
     }
@@ -952,8 +971,12 @@ bool LockProcess::startPlasma()
     delete mPlasmaDBus;
     mPlasmaDBus = 0;
 
-    connect(QDBusConnection::sessionBus().interface(), SIGNAL(serviceOwnerChanged(QString, QString, QString)),
-            this, SLOT(newService(QString, QString, QString)));
+    if (!mServiceWatcher) {
+        mServiceWatcher = new QDBusServiceWatcher(s_overlayServiceName, QDBusConnection::sessionBus(),
+                                                   QDBusServiceWatcher::WatchForOwnerChange, this);
+        connect(mServiceWatcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
+                this, SLOT(newService(QString,QString,QString)));
+    }
 
     KProcess *plasmaProc = new KProcess;
     plasmaProc->setProgram("plasma-overlay");
@@ -1028,10 +1051,8 @@ void LockProcess::stopPlasma()
 
 void LockProcess::newService(QString name, QString oldOwner, QString newOwner)
 {
+    Q_UNUSED(name);
     Q_UNUSED(oldOwner);
-    if (name != "org.kde.plasma-overlay") {
-        return;
-    }
 
     if (mPlasmaDBus) {
         if (newOwner.isEmpty()) {
@@ -1044,8 +1065,7 @@ void LockProcess::newService(QString name, QString oldOwner, QString newOwner)
     }
 
     kDebug() << "plasma! yaay!";
-    mPlasmaDBus = new org::kde::plasmaoverlay::App(name, "/App",
-            QDBusConnection::sessionBus(), this);
+    mPlasmaDBus = new org::kde::plasmaoverlay::App(s_overlayServiceName, "/App", QDBusConnection::sessionBus(), this);
 
     //XXX this isn't actually used any more iirc
     connect(mPlasmaDBus, SIGNAL(hidden()), SLOT(unSuppressUnlock()));
@@ -1093,8 +1113,15 @@ void LockProcess::suspend()
     if( !mSuspended && mHackProc.state() == QProcess::Running )
     {
         ::kill(mHackProc.pid(), SIGSTOP);
-        QApplication::syncX();
+        // We actually want to wait for the stopped hack's X commands
+        // having been handled, but that would require a custom
+        // protocol which would cause the hack to call XSync() and
+        // freeze itself. So just go to sleep and hope that the X
+        // server will have enough time ...
+        usleep(100000);
         mSavedScreen = QPixmap::grabWindow( winId());
+        mSnapshotTimer.setSingleShot(true);
+        mSnapshotTimer.start(1000);
     }
     mSuspended = true;
 }
@@ -1106,10 +1133,13 @@ void LockProcess::resume( bool force )
     if( mSuspended && mHackProc.state() == QProcess::Running )
     {
         QPainter p( this );
-        p.drawPixmap( 0, 0, mSavedScreen );
+        if (!mSavedScreen.isNull())
+            p.drawPixmap( 0, 0, mSavedScreen );
+        else
+            p.fillRect( rect(), Qt::black );
         p.end();
-        mSavedScreen = QPixmap();
         QApplication::syncX();
+        mSavedScreen = QPixmap();
         ::kill(mHackProc.pid(), SIGCONT);
     }
     mSuspended = false;
@@ -1159,7 +1189,7 @@ bool LockProcess::checkPass(const QString &reason)
     }
     PasswordDlg passDlg(this, &greetPlugin, reason);
     const int ret = execDialog( &passDlg );
-    kDebug() << ret;
+//    kDebug() << ret;
 
     //FIXME do we need to copy&paste that SubstructureNotifyMask code above?
     if (ret == QDialog::Accepted) {
@@ -1167,6 +1197,7 @@ bool LockProcess::checkPass(const QString &reason)
         //so that the user doesn't have to type their password twice
         mLocked = false;
         KDisplayManager().setLock(false);
+        KNotification::event( "unlocked" );
         //FIXME while suppressUnlock *should* always be running, if it isn't
         //(say if someone's doing things they shouldn't with dbus) then it won't get started by this
         //which means that a successful unlock will never re-lock
@@ -1326,47 +1357,65 @@ bool LockProcess::x11Event(XEvent *event)
                 mVisibility = !(event->xvisibility.state == VisibilityFullyObscured);
                 if (!mVisibility) {
                     mSuspendTimer.start(2000);
-                    kDebug() << "fully obscured";
+                    kDebug(1204) << "fully obscured";
                 } else {
-                    kDebug() << "not fully obscured";
+                    kDebug(1204) << "not fully obscured";
                     mSuspendTimer.stop();
                     resume( false );
                 }
                 if (mForeignWindows.isEmpty() && event->xvisibility.state != VisibilityUnobscured) {
-                    kDebug() << "no plasma; saver obscured";
+                    kDebug(1204) << "no plasma; saver obscured";
                     stayOnTop();
                 }
             } else if (!mForeignWindows.isEmpty() && event->xvisibility.window == mForeignWindows.last() &&
                     event->xvisibility.state != VisibilityUnobscured) {
                 //FIXME now that we have several plasma winids this doesn't feel valid
                 //but I don't know what to do about it!
-                kDebug() << "plasma obscured!";
+                kDebug(1204) << "plasma obscured!";
                 stayOnTop();
             }
             break;
 
         case ConfigureNotify: // from SubstructureNotifyMask on the root window
             if(event->xconfigure.event == QX11Info::appRootWindow()) {
+                int index = findWindowInfo( event->xconfigure.window );
+                if( index >= 0 ) {
+                    int index2 = event->xconfigure.above ? findWindowInfo( event->xconfigure.above ) : 0;
+                    if( index2 < 0 )
+                        kDebug(1204) << "Unknown above for ConfigureNotify";
+                    else { // move just above the other window
+                        if( index2 < index )
+                            ++index2;
+                        windowInfo.move( index, index2 );
+                    }
+                } else
+                    kDebug(1204) << "Unknown toplevel for ConfigureNotify";
                 //kDebug() << "ConfigureNotify:";
-                //the stacking order changed, so let's change the stacking order!
+                //the stacking order changed, so let's change the stacking order again to what we want
                 stayOnTop();
             }
             break;
         case MapNotify: // from SubstructureNotifyMask on the root window
             if( event->xmap.event == QX11Info::appRootWindow()) {
-                kDebug() << "MapNotify:" << event->xmap.window;
+                kDebug(1204) << "MapNotify:" << event->xmap.window;
+                int index = findWindowInfo( event->xmap.window );
+                if( index >= 0 )
+                    windowInfo[ index ].viewable = true;
+                else
+                    kDebug(1204) << "Unknown toplevel for MapNotify";
+                KXErrorHandler err; // ignore X errors here
                 WindowType type = windowType(event->xmap.window);
                 if (type != IgnoreWindow) {
                     if (mForeignWindows.contains(event->xmap.window)) {
-                        kDebug() << "uhoh! duplicate!";
+                        kDebug(1204) << "uhoh! duplicate!";
                     } else {
                         //ordered youngest-on-top
                         mForeignWindows.prepend(event->xmap.window);
                     }
                     if (type & InputWindow) {
-                        kDebug() << "input window";
+                        kDebug(1204) << "input window";
                         if (mForeignInputWindows.contains(event->xmap.window)) {
-                            kDebug() << "uhoh! duplicate again"; //never happens
+                            kDebug(1204) << "uhoh! duplicate again"; //never happens
                         } else {
                             //ordered youngest-on-top
                             mForeignInputWindows.prepend(event->xmap.window);
@@ -1379,13 +1428,70 @@ bool LockProcess::x11Event(XEvent *event)
             }
             break;
         case UnmapNotify:
-            if (event->xmap.event == QX11Info::appRootWindow()) {
-                kDebug() << "UnmapNotify:" << event->xunmap.window;
+            if (event->xunmap.event == QX11Info::appRootWindow()) {
+                kDebug(1204) << "UnmapNotify:" << event->xunmap.window;
+                int index = findWindowInfo( event->xunmap.window );
+                if( index >= 0 )
+                    windowInfo[ index ].viewable = false;
+                else
+                    kDebug(1204) << "Unknown toplevel for MapNotify";
                 mForeignWindows.removeAll(event->xunmap.window);
                 if (mForeignInputWindows.removeAll(event->xunmap.window)) {
                     updateFocus();
                 }
             }
+            break;
+        case CreateNotify:
+            if (event->xcreatewindow.parent == QX11Info::appRootWindow()) {
+                kDebug(1204) << "CreateNotify:" << event->xcreatewindow.window;
+                int index = findWindowInfo( event->xcreatewindow.window );
+                if( index >= 0 )
+                    kDebug(1204) << "Already existing toplevel for CreateNotify";
+                else {
+                    WindowInfo info;
+                    info.window = event->xcreatewindow.window;
+                    info.viewable = false;
+                    windowInfo.append( info );
+                }
+            }
+            break;
+        case DestroyNotify:
+            if (event->xdestroywindow.event == QX11Info::appRootWindow()) {
+                int index = findWindowInfo( event->xdestroywindow.window );
+                if( index >= 0 )
+                    windowInfo.removeAt( index );
+                else
+                    kDebug(1204) << "Unknown toplevel for DestroyNotify";
+            }
+            break;
+        case ReparentNotify:
+            if (event->xreparent.event == QX11Info::appRootWindow() && event->xreparent.parent != QX11Info::appRootWindow()) {
+                int index = findWindowInfo( event->xreparent.window );
+                if( index >= 0 )
+                    windowInfo.removeAt( index );
+                else
+                    kDebug(1204) << "Unknown toplevel for ReparentNotify away";
+            } else if (event->xreparent.parent == QX11Info::appRootWindow()) {
+                int index = findWindowInfo( event->xreparent.window );
+                if( index >= 0 )
+                    kDebug(1204) << "Already existing toplevel for ReparentNotify";
+                else {
+                    WindowInfo info;
+                    info.window = event->xreparent.window;
+                    info.viewable = false;
+                    windowInfo.append( info );
+                }
+            }
+            break;
+        case CirculateNotify:
+            if (event->xcirculate.event == QX11Info::appRootWindow()) {
+                int index = findWindowInfo( event->xcirculate.window );
+                if( index >= 0 ) {
+                    windowInfo.move( index, event->xcirculate.place == PlaceOnTop ? windowInfo.size() - 1 : 0 );
+                } else
+                    kDebug(1204) << "Unknown toplevel for CirculateNotify";
+            }
+            break;
     }
 
     // We have grab with the grab window being the root window.
@@ -1416,8 +1522,33 @@ bool LockProcess::x11Event(XEvent *event)
             {
                 //kDebug() << "forward to plasma";
                 XEvent ev2 = *event;
-                ev2.xkey.window = ev2.xkey.subwindow = mForeignInputWindows.first();
-                XSendEvent(QX11Info::display(), ev2.xkey.window, False, NoEventMask, &ev2);
+                Window root_return;
+                int x_return, y_return;
+                unsigned int width_return, height_return, border_width_return, depth_return;
+                WId targetWindow = 0;
+                //kDebug() << "root is" << winId();
+                //kDebug() << "search window under pointer with" << mForeignInputWindows.size() << "windows";
+                KXErrorHandler err; // ignore X errors
+                foreach(WId window, mForeignInputWindows)
+                {
+                    if( XGetGeometry(QX11Info::display(), window, &root_return,
+                                &x_return, &y_return,
+                                &width_return, &height_return,
+                                &border_width_return, &depth_return)
+                        &&
+                        (event->xkey.x>=x_return && event->xkey.x<=x_return+(int)width_return)
+                        &&
+                        (event->xkey.y>=y_return && event->xkey.y<=y_return+(int)height_return) )
+                    {
+                        //kDebug() << "found window" << window;
+                        targetWindow = window;
+                        ev2.xkey.window = ev2.xkey.subwindow = targetWindow;
+                        ev2.xkey.x = event->xkey.x - x_return;
+                        ev2.xkey.y = event->xkey.y - y_return;
+                        break;
+                    }
+                }
+                XSendEvent(QX11Info::display(), targetWindow, False, NoEventMask, &ev2);
                 ret = true;
             }
         default:
@@ -1444,9 +1575,9 @@ LockProcess::WindowType LockProcess::windowType(WId id)
     WindowType type = IgnoreWindow;
     if (result == Success && actualType == tag) {
         if (nitems != 1 || actualFormat != 8) {
-            kDebug() << "malformed property";
+            kDebug(1204) << "malformed property";
         } else {
-            kDebug() << "i can haz plasma window?" << data[0];
+            kDebug(1204) << "i can haz plasma window?" << data[0];
             switch (data[0]) {
                 case 0: //FIXME magic numbers
                     type = SimpleWindow;
@@ -1491,36 +1622,37 @@ void LockProcess::stayOnTop()
     // don't repaint as necessary. Therefore, if a window is detected above any of the windows
     // related to screenlocking, I don't see any better possibility than to completely
     // erase the screenlocker window.
-    // It is important to first detect, than restack and then erase.
+    // It is important to first detect, then restack and then erase.
+    // Another catch here is that only viewable windows matter, but checking here whether
+    // a window is viewable is a race condition, since a window may map, paint and unmap
+    // before we reach this point, thus making this code fail to detect the need to do
+    // a repaint. Therefore we track all relevant X events about mapping state of toplevel
+    // windows (which ensures proper ordering) and here just consult the information.
     bool needs_erase = false;
-    Window r, p;
-    Window* real;
-    unsigned int nreal;
-    if( XQueryTree( x11Info().display(), x11Info().appRootWindow(), &r, &p, &real, &nreal )
-        && real != NULL ) {
-        bool found_ours = false;
-        for( unsigned int i = 0;
-             i < nreal;
-             ++i ) {
-            if( stack.contains( real[ i ] ))
-                found_ours = true;
-            else {
-                if( found_ours ) {
-                    needs_erase = true;
-                    break;
-                }
-            }
+    bool found_ours = false;
+    foreach( const WindowInfo& info, windowInfo ) {
+        if( stack.contains( info.window )) {
+            found_ours = true;
+        } else if( found_ours && info.viewable ) {
+            kDebug(1204) << "found foreign window above screensaver";
+            needs_erase = true;
+            break;
         }
-        XFree( real );
     }
     // do the actual restacking if needed
     XRaiseWindow( x11Info().display(), stack[ 0 ] );
     if( count > 1 )
         XRestackWindows( x11Info().display(), stack.data(), count );
     if( needs_erase ) {
-        kDebug( 1024 ) << "Window above screensaver, raising, erasing";
+        // if the snapshot was taken recently it is possible that the rogue
+        // window was snapshotted at well.
+        if (mSnapshotTimer.isActive())
+            mSavedScreen = QPixmap();
         QPainter p( this );
-        p.fillRect( rect(), Qt::black );
+        if (!mSavedScreen.isNull())
+            p.drawPixmap( 0, 0, mSavedScreen );
+        else
+            p.fillRect( rect(), Qt::black );
         p.end();
         QApplication::syncX();
     }
@@ -1534,12 +1666,9 @@ void LockProcess::checkDPMSActive()
     DPMSInfo(QX11Info::display(), &state, &on);
     //kDebug() << "checkDPMSActive " << on << " " << state;
     if (state == DPMSModeStandby || state == DPMSModeSuspend || state == DPMSModeOff)
-    {
-       suspend();
-    } else if ( mSuspended )
-    {
-        resume( true );
-    }
+        suspend();
+    else
+        resume( false );
 #endif
 }
 
@@ -1552,8 +1681,9 @@ void LockProcess::lockXF86()
 {
     if( can_do_xf86_lock == Unknown )
     {
-        int major, minor;
-        if( XF86MiscQueryVersion( QX11Info::display(), &major, &minor )
+        int major, minor, dummy;
+        if( XF86MiscQueryExtension( QX11Info::display(), &dummy, &dummy )
+            && XF86MiscQueryVersion( QX11Info::display(), &major, &minor )
             && (major > 0 || minor >= 5) )
             can_do_xf86_lock = Yes;
         else
@@ -1609,4 +1739,14 @@ void LockProcess::msgBox( QWidget *parent, QMessageBox::Icon type, const QString
     execDialog( &box );
 }
 
+int LockProcess::findWindowInfo( Window w )
+{
+    for( int i = 0;
+         i < windowInfo.size();
+         ++i )
+        if( windowInfo[ i ].window == w )
+            return i;
+    return -1;
+}
+
 #include "lockprocess.moc"
