@@ -82,6 +82,12 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <klauncher_iface.h>
 #include "kcminit_interface.h"
 
+//#define KSMSERVER_STARTUP_DEBUG1
+
+#ifdef KSMSERVER_STARTUP_DEBUG1
+static QTime t;
+#endif
+
 /*!  Restores the previous session. Ensures the window manager is
   running (if specified).
  */
@@ -89,6 +95,9 @@ void KSMServer::restoreSession( const QString &sessionName )
 {
     if( state != Idle )
         return;
+#ifdef KSMSERVER_STARTUP_DEBUG1
+    t.start();
+#endif
     state = LaunchingWM;
 
     kDebug( 1218 ) << "KSMServer::restoreSession " << sessionName;
@@ -99,7 +108,6 @@ void KSMServer::restoreSession( const QString &sessionName )
 
     int count =  configSessionGroup.readEntry( "count", 0 );
     appsToStart = count;
-    publishProgress( appsToStart, true );
     upAndRunning( "ksmserver" );
     connect( klauncherSignals, SIGNAL( autoStart0Done()), SLOT( autoStart0Done()));
     connect( klauncherSignals, SIGNAL( autoStart1Done()), SLOT( autoStart1Done()));
@@ -131,8 +139,10 @@ void KSMServer::startDefaultSession()
     if( state != Idle )
         return;
     state = LaunchingWM;
+#ifdef KSMSERVER_STARTUP_DEBUG1
+    t.start();
+#endif
     sessionGroup = "";
-    publishProgress( 0, true );
     upAndRunning( "ksmserver" );
     connect( klauncherSignals, SIGNAL( autoStart0Done()), SLOT( autoStart0Done()));
     connect( klauncherSignals, SIGNAL( autoStart1Done()), SLOT( autoStart1Done()));
@@ -162,6 +172,24 @@ void KSMServer::clientSetProgram( KSMClient* client )
 {
     if( client->program() == wm )
         autoStart0();
+#ifndef KSMSERVER_STARTUP_DEBUGl
+    if( state == Idle )
+        {
+        static int cnt = 0;
+        if( client->program() == "gedit" && ( cnt == 0 ))
+            ++cnt;
+        else if( client->program() == "konqueror" && ( cnt == 1 ))
+            ++cnt;
+        else if( client->program() == "kspaceduel" && ( cnt == 2 ))
+            ++cnt;
+        else if( client->program() == "gedit" && ( cnt == 3 ))
+            ++cnt;
+        else
+            cnt = 0;
+        if( cnt == 4 )
+            KMessageBox::information( NULL, "drat" );
+        }
+#endif
 }
 
 void KSMServer::wmProcessChange()
@@ -192,6 +220,9 @@ void KSMServer::autoStart0()
     if( !checkStartupSuspend())
         return;
     state = AutoStart0;
+#ifdef KSMSERVER_STARTUP_DEBUG1
+    kDebug() << t.elapsed();
+#endif
     org::kde::KLauncher klauncher("org.kde.klauncher", "/KLauncher", QDBusConnection::sessionBus());
     klauncher.autoStart((int)0);
 }
@@ -204,6 +235,9 @@ void KSMServer::autoStart0Done()
     if( !checkStartupSuspend())
         return;
     kDebug( 1218 ) << "Autostart 0 done";
+#ifdef KSMSERVER_STARTUP_DEBUG1
+    kDebug() << t.elapsed();
+#endif
     upAndRunning( "desktop" );
     kcminitSignals = new QDBusInterface("org.kde.kcminit", "/kcminit", "org.kde.KCMInit", QDBusConnection::sessionBus(), this );
     if( !kcminitSignals->isValid())
@@ -238,6 +272,9 @@ void KSMServer::autoStart1()
     if( state != KcmInitPhase1 )
         return;
     state = AutoStart1;
+#ifdef KSMSERVER_STARTUP_DEBUG1
+    kDebug() << t.elapsed();
+#endif
     org::kde::KLauncher klauncher("org.kde.klauncher", "/KLauncher", QDBusConnection::sessionBus());
     klauncher.autoStart((int)1);
 }
@@ -250,9 +287,13 @@ void KSMServer::autoStart1Done()
     if( !checkStartupSuspend())
         return;
     kDebug( 1218 ) << "Autostart 1 done";
+    setupShortcuts(); // done only here, because it needs kglobalaccel :-/
     lastAppStarted = 0;
     lastIdStarted.clear();
     state = Restoring;
+#ifdef KSMSERVER_STARTUP_DEBUG1
+    kDebug() << t.elapsed();
+#endif
     if( defaultSession()) {
         autoStart2();
         return;
@@ -275,7 +316,6 @@ void KSMServer::tryRestoreNext()
     KConfigGroup config(KGlobal::config(), sessionGroup );
 
     while ( lastAppStarted < appsToStart ) {
-        publishProgress ( appsToStart - lastAppStarted );
         lastAppStarted++;
         QString n = QString::number(lastAppStarted);
         QStringList restartCommand = config.readEntry( QString("restartCommand")+n, QStringList() );
@@ -300,7 +340,6 @@ void KSMServer::tryRestoreNext()
 
     appsToStart = 0;
     lastIdStarted.clear();
-    publishProgress( 0 );
 
     autoStart2();
 }
@@ -312,6 +351,9 @@ void KSMServer::autoStart2()
     if( !checkStartupSuspend())
         return;
     state = FinishingStartup;
+#ifdef KSMSERVER_STARTUP_DEBUG1
+    kDebug() << t.elapsed();
+#endif
     waitAutoStart2 = true;
     waitKcmInit2 = true;
     org::kde::KLauncher klauncher("org.kde.klauncher", "/KLauncher", QDBusConnection::sessionBus());
@@ -393,6 +435,9 @@ void KSMServer::finishStartup()
         return;
 
     upAndRunning( "ready" );
+#ifdef KSMSERVER_STARTUP_DEBUG1
+    kDebug() << t.elapsed();
+#endif
 
     state = Idle;
     setupXIOErrorHandler(); // From now on handle X errors as normal shutdown.
@@ -459,23 +504,6 @@ void KSMServer::resumeStartupInternal()
     }
 }
 
-void KSMServer::publishProgress( int progress, bool max )
-{
-    Q_UNUSED( progress );
-    Q_UNUSED( max );
-    // KSplash now goes away before restoring of session-saved apps starts, in order
-    // to make the startup visually faster, so this is not needed now. Also, there's
-    // no DBUS interface anymore.
-#if 0
-    org::kde::KSplash ksplash("org.kde.ksplash", "/KSplash", QDBusConnection::sessionBus());
-    if(max)
-	ksplash.setMaxProgress(progress);
-    else
-        ksplash.setProgress(progress);
-#endif
-}
-
-
 void KSMServer::upAndRunning( const QString& msg )
 {
     XEvent e;
