@@ -143,6 +143,7 @@ static void create_pixmaps()
     QPainter maskPainter;
     int i, x, y;
     titlePix = new QPixmap(33, 12);
+    titlePix->fill( Qt::transparent );
     QBitmap mask(33, 12);
     mask.fill(Qt::color0);
 
@@ -253,6 +254,7 @@ static void delete_pixmaps()
 LaptopButton::LaptopButton(ButtonType type, LaptopClient *parent, const char *name)
     : KCommonDecorationButton(type, parent)
 {
+    Q_UNUSED( name );
     setAttribute(Qt::WA_NoSystemBackground, true);
 }
 
@@ -337,10 +339,16 @@ void LaptopButton::drawButton(QPainter *p)
         p->drawLine(0, h-1, w-1, h-1);
     }
 
-    p->setPen(btnForeground);
-    int xOff = (width()-8)/2;
-    int yOff = (height()-8)/2;
-    p->drawPixmap(isDown() ? xOff+1: xOff, isDown() ? yOff+1 : yOff, deco);
+    QPainterPath path;
+    path.addRegion( deco );
+    
+    QPoint offset( (width()-8)/2, (height()-8)/2 );
+    if( isDown() ) offset += QPoint( 1, 1 );
+    p->translate( offset );
+    p->setPen( Qt::NoPen );
+    p->setBrush( btnForeground );
+    p->drawPath( path );
+    
 }
 
 // =====================================
