@@ -217,6 +217,13 @@ void PresentWindowsEffect::postPaintScreen()
             }
         m_windowData.clear();
 
+        foreach( EffectWindow *w, effects->stackingOrder() )
+            {
+            if( w->isDock() )
+                {
+                w->setData( WindowForceBlurRole, QVariant( false ) );
+                }
+            }
         effects->setActiveFullScreenEffect( NULL );
         }
 
@@ -318,7 +325,7 @@ void PresentWindowsEffect::paintWindow( EffectWindow *w, int mask, QRegion regio
             {
             m_motionManager.apply( w, data );
 
-            effects->paintWindow( w, mask, region, data );
+            effects->paintWindow( w, mask | PAINT_WINDOW_LANCZOS, region, data );
 
             QRect rect = m_motionManager.transformedGeometry( w ).toRect();
             if( m_showIcons )
@@ -744,7 +751,7 @@ void PresentWindowsEffect::tabBoxKeyEvent( QKeyEvent* event )
 // Atom handling
 void PresentWindowsEffect::propertyNotify( EffectWindow* w, long a )
     {
-    if( a != m_atomDesktop && a != m_atomWindows )
+    if( !w || ( a != m_atomDesktop && a != m_atomWindows ))
         return; // Not our atom
 
     if( a == m_atomDesktop )
@@ -1607,6 +1614,14 @@ void PresentWindowsEffect::setActive( bool active, bool closingTab )
             setHighlightedWindow( effects->currentTabBoxWindow() );
         else
             setHighlightedWindow( effects->activeWindow() );
+
+        foreach( EffectWindow *w, effects->stackingOrder() )
+            {
+            if( w->isDock() )
+                {
+                w->setData( WindowForceBlurRole, QVariant( true ) );
+                }
+            }
         }
     else
         {
@@ -1672,6 +1687,8 @@ bool PresentWindowsEffect::isSelectableWindow( EffectWindow *w )
         return false;
     if( !w->visibleInClientGroup() )
         return false;
+    if( w->isSkipSwitcher() )
+        return false;
     switch( m_mode )
         {
         case ModeAllDesktops:
