@@ -321,6 +321,13 @@ bool Workspace::workspaceEvent( XEvent * e )
                 if( c->windowEvent( e ))
                     return true;
                 }
+
+        // We want to pass root window property events to effects
+        if( e->type == PropertyNotify && e->xany.window == rootWindow() && effects )
+            {
+            XPropertyEvent* re = &e->xproperty;
+            static_cast< EffectsHandlerImpl* >( effects )->propertyNotify( NULL, re->atom );
+            }
         }
     if( movingClient != NULL && movingClient->moveResizeGrabWindow() == e->xany.window
         && ( e->type == MotionNotify || e->type == ButtonPress || e->type == ButtonRelease ))
@@ -1390,7 +1397,7 @@ bool Client::buttonReleaseEvent( Window w, int /*button*/, int state, int x, int
             {
             finishMoveResize( false );
             // mouse position is still relative to old Client position, adjust it
-            QPoint mousepos( x_root - x, y_root - y );
+            QPoint mousepos( x_root - x + padding_left, y_root - y + padding_top );
             mode = mousePosition( mousepos );
             }
         else if( workspace()->decorationSupportsClientGrouping() )
@@ -1474,7 +1481,10 @@ bool Client::motionNotifyEvent( Window w, int /*state*/, int x, int y, int x_roo
         return true; // care only about the whole frame
     if ( !buttonDown ) 
         {
-        Position newmode = mousePosition( QPoint( x, y ));
+        QPoint mousePos( x, y );
+        if( w == frameId() ) 
+            mousePos += QPoint( padding_left, padding_top );
+        Position newmode = mousePosition( mousePos );
         if( newmode != mode )
             {
             mode = newmode;
