@@ -50,6 +50,7 @@ namespace KWin
 RulesWidget::RulesWidget( QWidget* parent )
     : detect_dlg( NULL )
     {
+    Q_UNUSED( parent );
     setupUi(this);
 
     QString enableDesc =
@@ -95,10 +96,15 @@ RulesWidget::RulesWidget( QWidget* parent )
     SETUP( noborder, set );
     SETUP( skiptaskbar, set );
     SETUP( skippager, set );
+    SETUP( skipswitcher, set );
     SETUP( acceptfocus, force );
     SETUP( closeable, force );
+    SETUP( autogroup, force );
+    SETUP( autogroupfg, force );
+    SETUP( autogroupid, force );
     SETUP( opacityactive, force );
     SETUP( opacityinactive, force );
+    SETUP( tilingoption, force );
     SETUP( shortcut, force );
     // workarounds tab
     SETUP( fsplevel, force );
@@ -142,10 +148,15 @@ UPDATE_ENABLE_SLOT( below )
 UPDATE_ENABLE_SLOT( noborder )
 UPDATE_ENABLE_SLOT( skiptaskbar )
 UPDATE_ENABLE_SLOT( skippager )
+UPDATE_ENABLE_SLOT( skipswitcher )
 UPDATE_ENABLE_SLOT( acceptfocus )
 UPDATE_ENABLE_SLOT( closeable )
+UPDATE_ENABLE_SLOT( autogroup )
+UPDATE_ENABLE_SLOT( autogroupfg )
+UPDATE_ENABLE_SLOT( autogroupid )
 UPDATE_ENABLE_SLOT( opacityactive )
 UPDATE_ENABLE_SLOT( opacityinactive )
+UPDATE_ENABLE_SLOT( tilingoption )
 void RulesWidget::updateEnableshortcut()
     {
     shortcut->setEnabled( enable_shortcut->isChecked() && rule_shortcut->currentIndex() != 0 );
@@ -262,6 +273,16 @@ int RulesWidget::comboToDesktop( int val ) const
     return val + 1;
     }
 
+int RulesWidget::tilingToCombo( int t ) const
+    {
+    return qBound(0, t, 1);
+    }
+
+int RulesWidget::comboToTiling( int val ) const
+    {
+    return val; // 0 is tiling, 1 is floating
+    }
+
 static int placementToCombo( Placement::Policy placement )
     {
     static const int conv[] = 
@@ -417,10 +438,15 @@ void RulesWidget::setRules( Rules* rules )
     CHECKBOX_SET_RULE( noborder, );
     CHECKBOX_SET_RULE( skiptaskbar, );
     CHECKBOX_SET_RULE( skippager, );
+    CHECKBOX_SET_RULE( skipswitcher, );
     CHECKBOX_FORCE_RULE( acceptfocus, );
     CHECKBOX_FORCE_RULE( closeable, );
+    CHECKBOX_FORCE_RULE( autogroup, );
+    CHECKBOX_FORCE_RULE( autogroupfg, );
+    LINEEDIT_FORCE_RULE( autogroupid, );
     LINEEDIT_FORCE_RULE( opacityactive, intToStr );
     LINEEDIT_FORCE_RULE( opacityinactive, intToStr );
+    COMBOBOX_FORCE_RULE( tilingoption, tilingToCombo );
     LINEEDIT_SET_RULE( shortcut, );
     COMBOBOX_FORCE_RULE( fsplevel, );
     COMBOBOX_FORCE_RULE( moveresizemode, moveresizeToCombo );
@@ -476,16 +502,16 @@ Rules* RulesWidget::rules() const
         rules->types = NET::AllTypesMask;
     else
         {
-        rules->types |= types->item(0)->isSelected() ? NET::NormalMask : 0;
-        rules->types |= types->item(1)->isSelected() ? NET::DialogMask : 0;
-        rules->types |= types->item(2)->isSelected() ? NET::UtilityMask : 0;
-        rules->types |= types->item(3)->isSelected() ? NET::DockMask : 0;
-        rules->types |= types->item(4)->isSelected() ? NET::ToolbarMask : 0;
-        rules->types |= types->item(5)->isSelected() ? NET::MenuMask : 0;
-        rules->types |= types->item(6)->isSelected() ? NET::SplashMask : 0;
-        rules->types |= types->item(7)->isSelected() ? NET::DesktopMask : 0;
-        rules->types |= types->item(8)->isSelected() ? NET::OverrideMask : 0;
-        rules->types |= types->item(9)->isSelected() ? NET::TopMenuMask : 0;
+        rules->types |= types->item(0)->isSelected() ? NET::NormalMask : 0U;
+        rules->types |= types->item(1)->isSelected() ? NET::DialogMask : 0U;
+        rules->types |= types->item(2)->isSelected() ? NET::UtilityMask : 0U;
+        rules->types |= types->item(3)->isSelected() ? NET::DockMask : 0U;
+        rules->types |= types->item(4)->isSelected() ? NET::ToolbarMask : 0U;
+        rules->types |= types->item(5)->isSelected() ? NET::MenuMask : 0U;
+        rules->types |= types->item(6)->isSelected() ? NET::SplashMask : 0U;
+        rules->types |= types->item(7)->isSelected() ? NET::DesktopMask : 0U;
+        rules->types |= types->item(8)->isSelected() ? NET::OverrideMask : 0U;
+        rules->types |= types->item(9)->isSelected() ? NET::TopMenuMask : 0U;
         }
     rules->title = title->text();
     rules->titlematch = static_cast< Rules::StringMatch >( title_match->currentIndex());
@@ -507,10 +533,15 @@ Rules* RulesWidget::rules() const
     CHECKBOX_SET_RULE( noborder, );
     CHECKBOX_SET_RULE( skiptaskbar, );
     CHECKBOX_SET_RULE( skippager, );
+    CHECKBOX_SET_RULE( skipswitcher, );
     CHECKBOX_FORCE_RULE( acceptfocus, );
     CHECKBOX_FORCE_RULE( closeable, );
+    CHECKBOX_FORCE_RULE( autogroup, );
+    CHECKBOX_FORCE_RULE( autogroupfg, );
+    LINEEDIT_FORCE_RULE( autogroupid, );
     LINEEDIT_FORCE_RULE( opacityactive, strToInt );
     LINEEDIT_FORCE_RULE( opacityinactive, strToInt );
+    COMBOBOX_FORCE_RULE( tilingoption, comboToTiling );
     LINEEDIT_SET_RULE( shortcut, );
     COMBOBOX_FORCE_RULE( fsplevel, );
     COMBOBOX_FORCE_RULE( moveresizemode, comboToMoveResize );
@@ -624,10 +655,15 @@ void RulesWidget::prefillUnusedValues( const KWindowInfo& info )
     CHECKBOX_PREFILL( noborder,, info.frameGeometry() == info.geometry() );
     CHECKBOX_PREFILL( skiptaskbar,, info.state() & NET::SkipTaskbar );
     CHECKBOX_PREFILL( skippager,, info.state() & NET::SkipPager );
+    CHECKBOX_PREFILL( skipswitcher,, false );
     //CHECKBOX_PREFILL( acceptfocus, );
     //CHECKBOX_PREFILL( closeable, );
+    //CHECKBOX_PREFILL( autogroup, );
+    //CHECKBOX_PREFILL( autogroupfg, );
+    //LINEEDIT_PREFILL( autogroupid, );
     LINEEDIT_PREFILL( opacityactive, intToStr, 100 /*get the actual opacity somehow*/);
     LINEEDIT_PREFILL( opacityinactive, intToStr, 100 /*get the actual opacity somehow*/);
+    COMBOBOX_PREFILL( tilingoption, tilingToCombo, 0 );
     //LINEEDIT_PREFILL( shortcut, );
     //COMBOBOX_PREFILL( fsplevel, );
     //COMBOBOX_PREFILL( moveresizemode, moveresizeToCombo );
@@ -680,10 +716,11 @@ void RulesWidget::prepareWindowSpecific( WId window )
 
 void RulesWidget::shortcutEditClicked()
     {
-    EditShortcutDialog dlg( window());
-    dlg.setShortcut( shortcut->text());
-    if( dlg.exec() == QDialog::Accepted )
-        shortcut->setText( dlg.shortcut());
+    QPointer<EditShortcutDialog> dlg = new EditShortcutDialog( window());
+    dlg->setShortcut( shortcut->text());
+    if( dlg->exec() == QDialog::Accepted )
+        shortcut->setText( dlg->shortcut());
+    delete dlg;
     }
 
 RulesDialog::RulesDialog( QWidget* parent, const char* name )
@@ -693,6 +730,7 @@ RulesDialog::RulesDialog( QWidget* parent, const char* name )
       setModal( true );
       setCaption( i18n( "Edit Window-Specific Settings" ) );
       setButtons( Ok | Cancel );
+      setWindowIcon(KIcon("preferences-system-windows-actions"));
 
       widget = new RulesWidget( this );
       setMainWidget( widget );
@@ -741,9 +779,10 @@ EditShortcut::EditShortcut( QWidget* parent )
 
 void EditShortcut::editShortcut()
     {
-    ShortcutDialog dlg( QKeySequence( shortcut->text()), window());
-    if( dlg.exec() == QDialog::Accepted )
-        shortcut->setText( dlg.shortcut().toString());
+    QPointer< ShortcutDialog > dlg = new ShortcutDialog( QKeySequence( shortcut->text()), window());
+    if( dlg->exec() == QDialog::Accepted )
+        shortcut->setText( dlg->shortcut().toString());
+    delete dlg;
     }
 
 void EditShortcut::clearShortcut()
@@ -778,6 +817,8 @@ ShortcutDialog::ShortcutDialog( const QKeySequence& cut, QWidget* parent )
     , widget( new KKeySequenceWidget( this ))
     {
     widget->setKeySequence( cut );
+    // It's a global shortcut so don't allow multikey shortcuts
+    widget->setMultiKeyShortcutsAllowed(false);
     setMainWidget( widget );
     }
 
