@@ -185,7 +185,14 @@ void KWinScreenEdgesConfig::monitorInit()
     monitorAddItem( i18n( "No Action" ));
     monitorAddItem( i18n( "Show Dashboard" ));
     monitorAddItem( i18n( "Show Desktop" ));
-
+    monitorAddItem( i18n( "Lock Screen" ));
+    monitorAddItem( i18n( "Prevent Screen Locking" ));
+    //Prevent Screen Locking is not supported on some edges
+    m_ui->monitor->setEdgeItemEnabled( int( Monitor::Top ), 4, false );
+    m_ui->monitor->setEdgeItemEnabled( int( Monitor::Left ), 4, false );
+    m_ui->monitor->setEdgeItemEnabled( int( Monitor::Right ), 4, false );
+    m_ui->monitor->setEdgeItemEnabled( int( Monitor::Bottom ), 4, false );
+    
     // Search the effect names
     KServiceTypeTrader* trader = KServiceTypeTrader::self();
     KService::List services;
@@ -250,6 +257,9 @@ void KWinScreenEdgesConfig::monitorLoadAction( ElectricBorder edge, const QStrin
     KConfigGroup config( m_config, "ElectricBorders" );
     QString lowerName = config.readEntry( configName, "None" ).toLower();
     if( lowerName == "dashboard" ) monitorChangeEdge( edge, int( ElectricActionDashboard ));
+    else if( lowerName == "showdesktop" ) monitorChangeEdge( edge, int( ElectricActionShowDesktop ));
+    else if( lowerName == "lockscreen" ) monitorChangeEdge( edge, int( ElectricActionLockScreen ));
+    else if( lowerName == "preventscreenlocking" ) monitorChangeEdge( edge, int( ElectricActionPreventScreenLocking ));
     }
 
 void KWinScreenEdgesConfig::monitorLoad()
@@ -347,8 +357,23 @@ void KWinScreenEdgesConfig::monitorSaveAction( int edge, const QString& configNa
         config.writeEntry( configName, "Dashboard" );
     else if( item == 2 )
         config.writeEntry( configName, "ShowDesktop" );
+    else if( item == 3)
+        config.writeEntry( configName, "LockScreen" );
+    else if ( item == 4)
+        config.writeEntry( configName, "PreventScreenLocking" );
     else // Anything else
         config.writeEntry( configName, "None" );
+
+        if ((edge == int( Monitor::TopRight )) ||
+            (edge == int( Monitor::BottomRight )) ||
+            (edge == int( Monitor::BottomLeft )) ||
+            (edge == int( Monitor::TopLeft )))
+            {
+            KConfig scrnConfig("kscreensaverrc");
+            KConfigGroup scrnGroup = scrnConfig.group("ScreenSaver");
+            scrnGroup.writeEntry("Action" + configName, (item == 4) ? 2 /* Prevent Screen Locking */ : 0 /* None */);
+            scrnGroup.sync();
+            }
     }
 
 void KWinScreenEdgesConfig::monitorSave()
