@@ -95,6 +95,8 @@ enum ElectricBorderAction
     ElectricActionNone,          // No special action, not set, desktop switch or an effect
     ElectricActionDashboard,     // Launch the Plasma dashboard
     ElectricActionShowDesktop,   // Show desktop or restore
+    ElectricActionLockScreen,   // Lock screen
+    ElectricActionPreventScreenLocking,
     ELECTRIC_ACTION_COUNT
 };
 
