@@ -33,6 +33,7 @@ namespace KWin
 class Workspace;
 class Deleted;
 class EffectWindowImpl;
+class LanczosFilter;
 
 // The base class for compositing backends.
 class Scene
@@ -81,7 +82,9 @@ class Scene
             // Clear whole background as the very first step, without optimizing it
             PAINT_SCREEN_BACKGROUND_FIRST = 1 << 6,
             // Temporary solution since (_OPAQUE | _TRANSLUCENT) is not working currently.
-            PAINT_DECORATION_ONLY = 1 << 7
+            PAINT_DECORATION_ONLY = 1 << 7,
+            // Window will be painted with a lanczos filter.
+            PAINT_WINDOW_LANCZOS = 1 << 8
             };
         // types of filtering available
         enum ImageFilterType { ImageFilterFast, ImageFilterGood };
@@ -140,6 +143,7 @@ class Scene
         Workspace* wspace;
         bool has_waitSync;
         bool selfCheckDone;
+        LanczosFilter* lanczos_filter;
     };
 
 // The base class for windows representations in composite backends
@@ -178,7 +182,9 @@ class Scene::Window
             // Window will not be painted because it is minimized
             PAINT_DISABLED_BY_MINIMIZE     = 1 << 3,
             // Window will not be painted because it is not the active window in a client group
-            PAINT_DISABLED_BY_CLIENT_GROUP = 1 << 4
+            PAINT_DISABLED_BY_CLIENT_GROUP = 1 << 4,
+            // Window will not be painted because it's not on the current activity
+            PAINT_DISABLED_BY_ACTIVITY     = 1 << 5
             };
         void enablePainting( int reason );
         void disablePainting( int reason );
