@@ -248,7 +248,7 @@ bool SceneXrender::selfCheckFinish()
             || img.pixel( 1, 1 ) != QColor( Qt::black ).rgb()
             || img.pixel( 2, 1 ) != QColor( Qt::white ).rgb())
             {
-            kError( 1212 ) << "Compositing self-check failed, disabling compositing.";
+            kError( 1212 ) << "XRender compositing self-check failed, disabling compositing.";
             ok = false;
             break;
             }
@@ -256,7 +256,7 @@ bool SceneXrender::selfCheckFinish()
     if( err.error( true ))
         ok = false;
     if( ok )
-        kDebug( 1212 ) << "Compositing self-check passed.";
+        kDebug( 1212 ) << "XRender compositing self-check passed.";
     if( !ok && options->disableCompositingChecks )
         {
         kWarning( 1212 ) << "Compositing checks disabled, proceeding regardless of self-check failure.";
@@ -684,6 +684,8 @@ void SceneXrender::Window::performPaint( int mask, QRegion region, WindowPaintDa
         { XDoubleToFixed( 0 ), XDoubleToFixed( 0 ), XDoubleToFixed( 1 ) }
     }};
  
+    XRenderPictureAttributes attr;
+
     if( mask & PAINT_WINDOW_TRANSFORMED )
         {
         xscale = data.xScale;
@@ -704,6 +706,22 @@ void SceneXrender::Window::performPaint( int mask, QRegion region, WindowPaintDa
         if( filter == ImageFilterGood )
             XRenderSetPictureFilter( display(), pic, const_cast< char* >( "good" ), NULL, 0 );
 
+        // This is needed to avoid hitting a fallback in the radeon driver.
+        // The Render specification states that sampling pixels outside the
+        // source picture results in alpha=0 pixels. This can be achieved by
+        // setting the border color to transparent black, but since the border
+        // color has the same format as the texture, it only works when the
+        // texture has an alpha channel. So the driver falls back to software
+        // when the repeat mode is RepeatNone, the picture has a non-identity
+        // transformation matrix, and doesn't have an alpha channel.
+        // Since we only scale the picture, we can work around this by setting
+        // the repeat mode to RepeatPad.
+        if (!window()->hasAlpha())
+            {
+            attr.repeat = RepeatPad;
+            XRenderChangePicture( display(), pic, CPRepeat, &attr );
+            }
+
         // transform the shape for clipping in paintTransformedScreen()
         QVector< QRect > rects = transformed_shape.rects();
         for( int i = 0;
@@ -724,29 +742,13 @@ void SceneXrender::Window::performPaint( int mask, QRegion region, WindowPaintDa
          !iterator.isDone();
          iterator.next())
         {
-        if ( !(mask & PAINT_DECORATION_ONLY) )
-            {
-            // Paint the window contents
-            if( opaque )
-                {
-                XRenderComposite( display(), PictOpSrc, pic, None, buffer, cr.x() * xscale, cr.y() * yscale,
-                                  0, 0, dr.x(), dr.y(), dr.width(), dr.height());
-                }
-            else
-                {
-                Picture alpha = alphaMask( data.opacity );
-                XRenderComposite( display(), PictOpOver, pic, alpha, buffer, cr.x() * xscale, cr.y() * yscale,
-                                  0, 0, dr.x(), dr.y(), dr.width(), dr.height());
-                transformed_shape = QRegion();
-                }
-            }
         if( client || deleted )
             {
             bool noBorder = true;
-            const QPixmap *left;
-            const QPixmap *top;
-            const QPixmap *right;
-            const QPixmap *bottom;
+            const QPixmap *left = NULL;
+            const QPixmap *top = NULL;
+            const QPixmap *right = NULL;
+            const QPixmap *bottom = NULL;
             QRect tr, lr, rr, br;
             QRect decorationRect;
             if( client && !client->noBorder() )
@@ -804,7 +806,24 @@ void SceneXrender::Window::performPaint( int mask, QRegion region, WindowPaintDa
                     }
                 }
             }
-        if( data.brightness < 1.0 )
+        if ( !(mask & PAINT_DECORATION_ONLY) )
+            {
+            // Paint the window contents
+            if( opaque )
+                {
+                XRenderComposite( display(), PictOpSrc, pic, None, buffer, cr.x() * xscale, cr.y() * yscale,
+                                  0, 0, dr.x(), dr.y(), dr.width(), dr.height());
+                }
+            else
+                {
+                Picture alpha = alphaMask( data.opacity );
+                XRenderComposite( display(), PictOpOver, pic, alpha, buffer, cr.x() * xscale, cr.y() * yscale,
+                                  0, 0, dr.x(), dr.y(), dr.width(), dr.height());
+                transformed_shape = QRegion();
+                }
+            }
+ 
+       if( data.brightness < 1.0 )
             {
             // fake brightness change by overlaying black
             XRenderColor col = { 0, 0, 0, 0xffff * ( 1 - data.brightness ) * data.opacity };
@@ -816,6 +835,11 @@ void SceneXrender::Window::performPaint( int mask, QRegion region, WindowPaintDa
         XRenderSetPictureTransform( display(), pic, &identity );
         if( filter == ImageFilterGood )
             XRenderSetPictureFilter( display(), pic, const_cast< char* >( "fast" ), NULL, 0 );
+        if (!window()->hasAlpha())
+            {
+            attr.repeat = RepeatNone;
+            XRenderChangePicture( display(), pic, CPRepeat, &attr );
+            }
         }
     }
 
