@@ -28,6 +28,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <kdecoration.h>
 #include <kdebug.h>
 
+#include "kactivityconsumer.h"
+
 #include "utils.h"
 #include "workspace.h"
 
@@ -66,6 +68,7 @@ class Toplevel
         virtual QSize clientSize() const = 0;
         virtual QRect visibleRect() const; // the area the window occupies on the screen
         virtual QRect decorationRect() const; // rect including the decoration shadows
+        virtual QRect transparentRect() const = 0;
         virtual QRegion decorationPendingRegion() const; // decoration region that needs to be repainted
 
         // prefer isXXX() instead
@@ -89,9 +92,13 @@ class Toplevel
         bool isDNDIcon() const;
 
         virtual int desktop() const = 0;
+        virtual QStringList activities() const = 0;
         bool isOnDesktop( int d ) const;
+        bool isOnActivity( const QString &activity ) const;
         bool isOnCurrentDesktop() const;
+        bool isOnCurrentActivity() const;
         bool isOnAllDesktops() const;
+        bool isOnAllActivities() const;
 
         QByteArray windowRole() const;
         QByteArray sessionId();
@@ -385,16 +392,31 @@ inline bool Toplevel::isOnAllDesktops() const
     return desktop() == NET::OnAllDesktops;
     }
 
+inline bool Toplevel::isOnAllActivities() const
+    {
+    return activities().isEmpty();
+    }
+
 inline bool Toplevel::isOnDesktop( int d ) const
     {
     return desktop() == d || /*desk == 0 ||*/ isOnAllDesktops();
     }
 
+inline bool Toplevel::isOnActivity( const QString &activity ) const
+    {
+    return activities().isEmpty() || activities().contains(activity);
+    }
+
 inline bool Toplevel::isOnCurrentDesktop() const
     {
     return isOnDesktop( workspace()->currentDesktop());
     }
 
+inline bool Toplevel::isOnCurrentActivity() const
+    {
+    return isOnActivity( Workspace::self()->currentActivity());
+    }
+
 inline QByteArray Toplevel::resourceName() const
     {
     return resource_name; // it is always lowercase
