@@ -1,5 +1,5 @@
 /*
- *   Copyright 2008 Davide Bettio <davide.bettio@kdemail.net>
+ *   Copyright 2008,2010 Davide Bettio <davide.bettio@kdemail.net>
  *   Copyright 2009 John Layt <john@layt.net>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 
 //Qt
 #include <QtCore/QDate>
+#include <QtCore/QListIterator>
 #include <QtGui/QPainter>
 #include <QtGui/QWidget>
 #include <QtGui/QGraphicsSceneWheelEvent>
@@ -29,6 +30,7 @@
 
 //KDECore
 #include <KGlobal>
+#include <KDateTime>
 #include <KDebug>
 #include <KConfigDialog>
 #include <KConfigGroup>
@@ -40,6 +42,8 @@
 
 #include "ui_calendarConfig.h"
 
+#include <cmath>
+
 namespace Plasma
 {
 
@@ -69,20 +73,20 @@ class CalendarTablePrivate
     public:
 
         CalendarTablePrivate(CalendarTable *calTable, const QDate &initialDate = QDate::currentDate())
+            : q(calTable),
+              calendarType("locale"),
+              calendar(KGlobal::locale()->calendar()),
+              displayEvents(true),
+              displayHolidays(true),
+              dataEngine(0),
+              automaticUpdates(true),
+              opacity(0.5)
         {
             svg = new Svg();
             svg->setImagePath("widgets/calendar");
             svg->setContainsMultipleImages(true);
 
-            calendarTable = calTable;
-            calendarType = "locale";
-            calendar = KGlobal::locale()->calendar();
-            dataEngine = 0;
-            displayHolidays = false;
-
             setDate(initialDate);
-
-            opacity = 0.5; //transparency for the inactive text
         }
 
         ~CalendarTablePrivate()
@@ -113,8 +117,9 @@ class CalendarTablePrivate
             // Force date update to refresh cached date componants then update display
             setDate(selectedDate);
             updateHoveredPainting(QPointF());
-            calendarTable->populateHolidays();
-            calendarTable->update();
+            q->populateHolidays();
+            q->populateEvents();
+            q->update();
         }
 
         void setDate(const QDate &setDate)
@@ -131,13 +136,15 @@ class CalendarTablePrivate
             if (daysShownInPrevMonth < 1) {
                 daysShownInPrevMonth += daysInWeek;
             }
+            viewStartDate = dateFromRowColumn(0, 0);
+            viewEndDate = dateFromRowColumn(DISPLAYED_WEEKS - 1, daysInWeek - 1);
         }
 
         //Returns the x co-ordinate of a given column to LTR order, column is 0 to (daysInWeek-1)
         //This version does not adjust for RTL, so should not be used directly for drawing
         int columnToX(int column)
         {
-            return calendarTable->boundingRect().x() +
+            return q->boundingRect().x() +
                    centeringSpace +
                    weekBarSpace +
                    cellW +
@@ -147,7 +154,7 @@ class CalendarTablePrivate
         //Returns the y co-ordinate for given row, row is 0 to (DISPLAYED_WEEKS - 1)
         int rowToY(int row)
         {
-            return (int) calendarTable->boundingRect().y() +
+            return (int) q->boundingRect().y() +
                          headerHeight +
                          headerSpace +
                          ((cellH + cellSpace) * row);
@@ -176,7 +183,7 @@ class CalendarTablePrivate
         int adjustColumn(int column)
         {
             if (column >= 0 && column < daysInWeek) {
-                if (calendarTable->layoutDirection() == Qt::RightToLeft) {
+                if (q->layoutDirection() == Qt::RightToLeft) {
                     return daysInWeek - column - 1;
                 } else {
                     return column;
@@ -253,12 +260,13 @@ class CalendarTablePrivate
             if (hoverRect != oldHoverRect) {
                 //FIXME: update only of a piece seems to paint over the old stuff
                 /*if (oldHoverRect.isValid()) {
-                    calendarTable->update(oldHoverRect);
+                    q->update(oldHoverRect);
                 }
                 if (hoverRect.isValid()) {
-                    calendarTable->update(hoverRect);
+                    q->update(hoverRect);
                 }*/
-                calendarTable->update();
+                emit q->dateHovered(dateFromRowColumn(hoverWeekRow, hoverWeekdayColumn));
+                q->update();
             }
         }
 
@@ -276,42 +284,61 @@ class CalendarTablePrivate
 
         QString defaultHolidaysRegion()
         {
-            QString tmpRegion = KGlobal::locale()->country();
-            if (tmpRegion == "C") {
-                // we assume 'C' means USA, which isn't the best of assumptions, really
-                // the user ought to set their region in system settings, however
-                tmpRegion = "us";
-            }
-
-            if (dataEngine && dataEngine->query("holidaysRegions").value("holidaysRegions").toStringList().contains(tmpRegion)) {
-                return tmpRegion;
-            }
-
-            return QString();
+            return dataEngine->query("holidaysDefaultRegion").value("holidaysDefaultRegion").toString();
         }
 
-        CalendarTable *calendarTable;
+        void insertPimOccurence(const QString &type, const QDate &date, Plasma::DataEngine::Data occurrence) {
+            if (date >= viewStartDate && date <= viewEndDate) {
+                int julian = date.toJulianDay();
+                if (type == "Event" && !events.contains(julian, occurrence)) {
+                    events.insert(julian, occurrence);
+                } else if (type == "Todo" && !todos.contains(julian, occurrence)) {
+                    todos.insert(julian, occurrence);
+                } else if (type == "Journal" && !journals.contains(julian, occurrence)) {
+                    journals.insert(julian, occurrence);
+                }
+            }
+        }
+
+        CalendarTable *q;
         QString calendarType;
         const KCalendarSystem *calendar;
 
         QDate selectedDate;
+        QDate currentDate;
         int selectedMonth;
         int selectedYear;
         int weekDayFirstOfSelectedMonth;
         int daysInWeek;
         int daysInSelectedMonth;
         int daysShownInPrevMonth;
+        QDate viewStartDate;
+        QDate viewEndDate;
 
+        bool displayEvents;
         bool displayHolidays;
         QString holidaysRegion;
         Plasma::DataEngine *dataEngine;
-        // Hash key: int = Julian Day number, QString = what's special
-        QHash<int, QString> specialDates;
+        // Hash key: int = Julian Day number of holiday, Data = details of holiday
+        QMultiHash<int, Plasma::DataEngine::Data> holidays;
+        // Index to Events/Todos/Journals that occur on a given date.
+        // Hash key: int = Julian Day number of event/todo occurrence, Data = occurence details including start date, end date and Akonadi incidence UID
+        QMultiHash<int, Plasma::DataEngine::Data> events;
+        QMultiHash<int, Plasma::DataEngine::Data> todos;
+        QMultiHash<int, Plasma::DataEngine::Data> journals;
+        // Event/Todo/Journal details.  An event may recur on multiple dates.
+        // Hask key: QString = Akonadi UID of event/todo/journal, Data = details of event/todo/journal
+        QHash<QString, Plasma::DataEngine::Data> pimEvents;
+        QString eventsQuery;
+
+        bool automaticUpdates;
+
+        QPointF lastSeenMousePos;
 
         Ui::calendarConfig calendarConfigUi;
 
         Plasma::Svg *svg;
-        float opacity;
+        float opacity; //transparency for the inactive text
         QRectF hoverRect;
         int hoverWeekRow;
         int hoverWeekdayColumn;
@@ -344,10 +371,10 @@ CalendarTable::~CalendarTable()
     delete d;
 }
 
-bool CalendarTable::setCalendar(const QString &newCalendarType)
+void CalendarTable::setCalendar(const QString &newCalendarType)
 {
     if (newCalendarType == d->calendarType) {
-        return true;
+        return;
     }
 
     if (newCalendarType == "locale") {
@@ -359,14 +386,12 @@ bool CalendarTable::setCalendar(const QString &newCalendarType)
     // Signal out date change so any dependents will update as well
     emit dateChanged(date(), date());
     emit dateChanged(date());
-
-    return true;
 }
 
-bool CalendarTable::setCalendar(const KCalendarSystem *newCalendar)
+void CalendarTable::setCalendar(const KCalendarSystem *newCalendar)
 {
     if (newCalendar == d->calendar) {
-        return true;
+        return;
     }
 
     d->setCalendar(newCalendar);
@@ -374,8 +399,6 @@ bool CalendarTable::setCalendar(const KCalendarSystem *newCalendar)
     // Signal out date change so any dependents will update as well
     emit dateChanged(date(), date());
     emit dateChanged(date());
-
-    return true;
 }
 
 const KCalendarSystem *CalendarTable::calendar() const
@@ -383,16 +406,16 @@ const KCalendarSystem *CalendarTable::calendar() const
     return d->calendar;
 }
 
-bool CalendarTable::setDate(const QDate &newDate)
+void CalendarTable::setDate(const QDate &newDate)
 {
     // New date must be valid in the current calendar system
     if (!calendar()->isValid(newDate)) {
-        return false;
+        return;
     }
 
     // If new date is the same as old date don't actually need to do anything
     if (newDate == date()) {
-        return true;
+        return;
     }
 
     int oldYear = d->selectedYear;
@@ -402,10 +425,12 @@ bool CalendarTable::setDate(const QDate &newDate)
     // now change the date
     d->setDate(newDate);
 
-    d->updateHoveredPainting(QPointF());
-
+    d->updateHoveredPainting(d->lastSeenMousePos);
+    update(); //FIXME: we shouldn't need this update here, but something in Qt is broken (but with plasmoidviewer everything work)
+    
     if (oldYear != d->selectedYear || oldMonth != d->selectedMonth) {
         populateHolidays();
+        populateEvents();
         update();
     } else {
         // only update the old and the new areas
@@ -421,7 +446,6 @@ bool CalendarTable::setDate(const QDate &newDate)
 
     emit dateChanged(newDate, oldDate);
     emit dateChanged(newDate);
-    return true;
 }
 
 const QDate& CalendarTable::date() const
@@ -435,6 +459,7 @@ void CalendarTable::setDataEngine(Plasma::DataEngine *dataEngine)
     if (d->dataEngine != dataEngine) {
         d->dataEngine = dataEngine;
         populateHolidays();
+        populateEvents();
     }
 }
 
@@ -443,27 +468,28 @@ const Plasma::DataEngine *CalendarTable::dataEngine() const
     return d->dataEngine;
 }
 
-bool CalendarTable::setDisplayHolidays(bool showHolidays)
+void CalendarTable::setDisplayHolidays(bool showHolidays)
 {
     if (showHolidays) {
-
-        if (!dataEngine() ||
-        !dataEngine()->query("holidaysRegions").value("holidaysRegions").toStringList().contains(holidaysRegion())) {
-            return false;
+        if (!dataEngine()) {
+            clearHolidays();
+            return;
         }
 
         if (holidaysRegion().isEmpty()) {
             setHolidaysRegion(d->defaultHolidaysRegion());
         }
 
+        QString queryString = "holidaysIsValidRegion:" + holidaysRegion();
+        if (!dataEngine()->query(queryString).value(queryString).toBool()) {
+            return;
+        }
     }
 
     if (d->displayHolidays != showHolidays) {
         d->displayHolidays = showHolidays;
         populateHolidays();
     }
-
-    return true;
 }
 
 bool CalendarTable::displayHolidays()
@@ -471,17 +497,42 @@ bool CalendarTable::displayHolidays()
     return d->displayHolidays && !holidaysRegion().isEmpty();
 }
 
-bool CalendarTable::setHolidaysRegion(const QString &region)
+bool CalendarTable::displayEvents()
 {
-    if(!dataEngine()->query("holidaysRegions").value("holidaysRegions").toStringList().contains(region)){
-        return false;
+    return d->displayEvents;
+}
+
+void CalendarTable::setDisplayEvents(bool display)
+{
+    if (d->displayEvents == display) {
+        return;
+    }
+
+    d->displayEvents = display;
+    if (display) {
+        populateEvents();
+    } else {
+        if (d->dataEngine) {
+            d->dataEngine->disconnectSource(d->eventsQuery, this);
+        }
+        d->events.clear();
+        d->todos.clear();
+        d->journals.clear();
+        d->pimEvents.clear();
+    }
+}
+
+void CalendarTable::setHolidaysRegion(const QString &region)
+{
+    QString queryString = "holidaysIsValidRegion:" + region;
+    if (!dataEngine()->query(queryString).value(queryString).toBool()) {
+        return;
     }
 
     if (d->holidaysRegion != region) {
         d->holidaysRegion = region;
         populateHolidays();
     }
-    return true;
 }
 
 QString CalendarTable::holidaysRegion() const
@@ -489,72 +540,198 @@ QString CalendarTable::holidaysRegion() const
     return d->holidaysRegion;
 }
 
-void CalendarTable::clearDateProperties()
+void CalendarTable::clearHolidays()
 {
-    d->specialDates.clear();
+    d->holidays.clear();
 }
 
-void CalendarTable::setDateProperty(QDate date, const QString &description)
+void CalendarTable::addHoliday(const QDate &date,  Plasma::DataEngine::Data holidayData)
 {
-    d->specialDates.insert(date.toJulianDay(), description);
+    d->holidays.insertMulti(date.toJulianDay(), holidayData);
 }
 
-QString CalendarTable::dateProperty(QDate date) const
+bool CalendarTable::dateHasDetails(const QDate &date) const
 {
-    return d->specialDates.value(date.toJulianDay());
+    int julian = date.toJulianDay();
+    return d->holidays.contains(julian) ||
+           d->events.contains(julian) ||
+           d->todos.contains(julian) ||
+           d->journals.contains(julian);
+}
+
+QString CalendarTable::dateDetails(const QDate &date) const
+{
+    QString details;
+    const int julian = date.toJulianDay();
+
+    if (d->holidays.contains(julian)) {
+        details += "<br>";
+
+        foreach (Plasma::DataEngine::Data holidayData, d->holidays.values(julian)) {
+            if (holidayData.value("observanceType").toString() == "PublicHoliday") {
+                details += i18n("<i>Holiday</i>: %1", holidayData.value("name").toString());
+                details += "<br>";
+            }
+        }
+
+        foreach (Plasma::DataEngine::Data holidayData, d->holidays.values(julian)) {
+            if (holidayData.value("observanceType").toString() == "Other") {
+                //TODO add a type when strings not frozen
+                details += holidayData.value("name").toString();
+                details += "<br>";
+            }
+        }
+    }
+
+    if (d->events.contains(julian)) {
+        details += "<br>";
+
+        foreach (const Plasma::DataEngine::Data &occurrence, d->events.values(julian)) {
+            QString eventUid = occurrence.value("OccurrenceUid").toString();
+            KDateTime occStartDate = occurrence.value("OccurrenceStartDate").value<KDateTime>();
+            KDateTime occEndDate = occurrence.value("OccurrenceEndDate").value<KDateTime>();
+            Plasma::DataEngine::Data eventData = d->pimEvents.value(eventUid);
+
+            //TODO translate this layout once strings not frozen
+            QString description;
+            if (eventData.value("AllDay").toBool()) {
+                description = QString("<br>%1").arg(eventData.value("Summary").toString());
+            } else if (!occEndDate.isValid() || occStartDate == occEndDate) {
+                description = QString("%1<br>%2").arg(KGlobal::locale()->formatTime(occStartDate.time()))
+                                                 .arg(eventData.value("Summary").toString());
+            } else {
+                description = QString("%1 - %2<br>%3").arg(KGlobal::locale()->formatTime(occStartDate.time()))
+                                                      .arg(KGlobal::locale()->formatTime(occEndDate.time()))
+                                                      .arg(eventData.value("Summary").toString());
+            }
+            details += i18n("<i>Event</i>: %1<br>", description);
+        }
+    }
+
+    if (d->todos.contains(julian)) {
+        details += "<br>";
+
+        foreach (const Plasma::DataEngine::Data &occurrence, d->todos.values(julian)) {
+            QString todoUid = occurrence.value("OccurrenceUid").toString();
+            KDateTime occStartDate = occurrence.value("OccurrenceStartDate").value<KDateTime>();
+            KDateTime occEndDate = occurrence.value("OccurrenceEndDate").value<KDateTime>();
+            Plasma::DataEngine::Data todoData = d->pimEvents.value(todoUid);
+
+            //TODO translate this layout once strings not frozen
+            QString description;
+            if (todoData.value("AllDay").toBool()) {
+                description = QString("<br>%1").arg(todoData.value("Summary").toString());
+            } else if (!occEndDate.isValid() || occStartDate == occEndDate) {
+                description = QString("%1<br>%2").arg(KGlobal::locale()->formatTime(occStartDate.time()))
+                                                 .arg(todoData.value("Summary").toString());
+            } else {
+                description = QString("%1 - %2<br>%3").arg(KGlobal::locale()->formatTime(occStartDate.time()))
+                                                      .arg(KGlobal::locale()->formatTime(occEndDate.time()))
+                                                      .arg(todoData.value("Summary").toString());
+            }
+            //TODO add Status and Percentage Complete when strings not frozen
+            details += i18n("<i>Todo</i>: %1<br>", description);
+        }
+    }
+
+    return details;
+}
+
+void CalendarTable::setAutomaticUpdateEnabled(bool enabled)
+{
+    d->automaticUpdates = enabled;
+}
+
+bool CalendarTable::isAutomaticUpdateEnabled() const
+{
+    return d->automaticUpdates;
+}
+    
+void CalendarTable::setCurrentDate(const QDate &date)
+{
+    d->currentDate = date; 
+}
+
+const QDate& CalendarTable::currentDate() const
+{
+    return d->currentDate;
 }
 
-//JPL Looks to work OK even though non-Gregorian months do not match up, probably due to fetching 3 months worth
 void CalendarTable::populateHolidays()
 {
-    clearDateProperties();
+    clearHolidays();
 
     if (!displayHolidays() || !dataEngine() || holidaysRegion().isEmpty()) {
         return;
     }
 
-    QDate queryDate = date();
-    QString prevMonthString = queryDate.addMonths(-1).toString(Qt::ISODate);
-    QString nextMonthString = queryDate.addMonths(1).toString(Qt::ISODate);
+    // Just fetch the days displayed in the grid
+    QString queryString = "holidays:" + holidaysRegion() + ':' + d->viewStartDate.toString(Qt::ISODate)
+                          + ':' + d->viewEndDate.toString(Qt::ISODate);
+    QList<QVariant> holidays = d->dataEngine->query(queryString).value(queryString).toList();
 
-    Plasma::DataEngine::Data prevMonth = d->dataEngine->query("holidaysInMonth:" + holidaysRegion() +
-    ':' + prevMonthString);
-    for (int i = -10; i < 0; i++) {
-        QDate tempDate = queryDate.addDays(i);
-        QString reason = prevMonth.value(tempDate.toString(Qt::ISODate)).toString();
-        if (!reason.isEmpty()) {
-            setDateProperty(tempDate, reason);
-        }
+    QMutableListIterator<QVariant> i(holidays);
+    while (i.hasNext()) {
+        Plasma::DataEngine::Data holidayData = i.next().toHash();
+        QDate holidayDate = QDate::fromString(holidayData.take("date").toString(), Qt::ISODate);
+        addHoliday(holidayDate, holidayData);
+    }
+}
+
+void CalendarTable::populateEvents()
+{
+    d->events.clear();
+    d->todos.clear();
+    d->journals.clear();
+    d->pimEvents.clear();
+
+    if (!d->displayEvents || !d->dataEngine) {
+        return;
     }
 
-    queryDate.setDate(queryDate.year(), queryDate.month(), 1);
-    Plasma::DataEngine::Data thisMonth = d->dataEngine->query("holidaysInMonth:" + holidaysRegion() +
-    ':' + queryDate.toString(Qt::ISODate));
-    int numDays = calendar()->daysInMonth(queryDate);
-    for (int i = 0; i < numDays; i++) {
-        QDate tempDate = queryDate.addDays(i);
-        QString reason = thisMonth.value(tempDate.toString(Qt::ISODate)).toString();
-        if (!reason.isEmpty()) {
-            setDateProperty(tempDate, reason);
-        }
-    }
+    // Just fetch the days displayed in the grid
+    d->dataEngine->disconnectSource(d->eventsQuery, this);
+    d->eventsQuery = "events:" + d->viewStartDate.toString(Qt::ISODate) + ':' + d->viewEndDate.toString(Qt::ISODate);
+    d->dataEngine->connectSource(d->eventsQuery, this);
+}
 
-    queryDate = queryDate.addMonths(1);
-    Plasma::DataEngine::Data nextMonth = d->dataEngine->query("holidaysInMonth:" + holidaysRegion() +
-    ':' + nextMonthString);
-    for (int i = 0; i < 10; i++) {
-        QDate tempDate = queryDate.addDays(i);
-        QString reason = nextMonth.value(tempDate.toString(Qt::ISODate)).toString();
-        if (!reason.isEmpty()) {
-            setDateProperty(tempDate, reason);
+void CalendarTable::dataUpdated(const QString &source, const Plasma::DataEngine::Data &data)
+{
+    Q_UNUSED(source)
+    d->events.clear();
+    d->todos.clear();
+    d->journals.clear();
+    d->pimEvents.clear();
+    foreach (const QVariant &v, data) {
+        Plasma::DataEngine::Data pimData = v.toHash();
+        QString type = pimData.value("Type").toString();
+        QString uid = pimData.value("UID").toString();
+        QDate startDate = pimData.value("StartDate").value<KDateTime>().date();
+
+        d->pimEvents.insert(uid, pimData);
+
+        QList<QVariant> occurrenceList = pimData.value("Occurrences").toList();
+        foreach (QVariant occurrence, occurrenceList) {
+            QDate occStartDate = occurrence.toHash().value("OccurrenceStartDate").value<KDateTime>().date();
+            if (pimData.value("EventMultiDay").toBool() == true) {
+                QDate occEndDate = occurrence.toHash().value("OccurrenceEndDate").value<KDateTime>().date();
+                QDate multiDate = occStartDate;
+                while (multiDate <= occEndDate) {
+                    d->insertPimOccurence(type, multiDate, occurrence.toHash());
+                    multiDate = multiDate.addDays(1);
+                }
+            } else {
+                d->insertPimOccurence(type, occStartDate, occurrence.toHash());
+            }
         }
     }
+    update();
 }
 
 void CalendarTable::applyConfiguration(KConfigGroup cg)
 {
     setCalendar(cg.readEntry("calendarType", "locale"));
-    setHolidaysRegion(cg.readEntry("holidaysRegion", QString()));
+    setHolidaysRegion(cg.readEntry("holidaysRegion", d->defaultHolidaysRegion()));
     setDisplayHolidays(cg.readEntry("displayHolidays", true));
 }
 
@@ -578,20 +755,30 @@ void CalendarTable::createConfigurationInterface(KConfigDialog *parent)
     }
     d->calendarConfigUi.calendarComboBox->setCurrentIndex( d->calendarConfigUi.calendarComboBox->findData( QVariant( d->calendarType ) ) );
 
-    QStringList regions = dataEngine()->query("holidaysRegions").value("holidaysRegions").toStringList();
-    QMap<QString, QString> regionMap;
-    foreach ( const QString &region, regions ) {
-        QString label = KGlobal::locale()->countryCodeToName( region );
-        if (label.isEmpty()) {
-            label = region;
-        }
-        regionMap.insert(label, region);
-    }
-    d->calendarConfigUi.regionComboBox->addItem(i18n("Do not show holidays"), QString());
-    QMapIterator<QString, QString> i(regionMap);
+    Plasma::DataEngine::Data regions = dataEngine()->query("holidaysRegions");
+    QMap<QString, QString> regionsMap;
+    Plasma::DataEngine::DataIterator i(regions);
     while (i.hasNext()) {
         i.next();
-        d->calendarConfigUi.regionComboBox->addItem( i.key(), QVariant( i.value() ) );
+        Plasma::DataEngine::Data regionData = i.value().toHash();
+        QString name = regionData.value("name").toString();
+        QString languageName = KGlobal::locale()->languageCodeToName(regionData.value("languageCode").toString());
+        QString label;
+        if (languageName.isEmpty()) {
+            label = name;
+        } else {
+            // Need to get permission to break string freeze, in the meantime don't translate!
+            //label = i18nc("Holday region, region language", "%1 (%2)", name, languageName);
+            label = QString("%1 (%2)").arg(name).arg(languageName);
+        }
+        regionsMap.insert(label, i.key());
+    }
+
+    d->calendarConfigUi.regionComboBox->addItem(i18n("Do not show holidays"), QString());
+    QMapIterator<QString, QString> j(regionsMap);
+    while ( j.hasNext() ) {
+        j.next();
+        d->calendarConfigUi.regionComboBox->addItem(j.key(), QVariant(j.value()));
     }
     d->calendarConfigUi.regionComboBox->setCurrentIndex( d->calendarConfigUi.regionComboBox->findData( QVariant( d->holidaysRegion ) ) );
 }
@@ -635,8 +822,12 @@ void CalendarTable::wheelEvent(QGraphicsSceneWheelEvent * event)
 
 void CalendarTable::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
+    d->lastSeenMousePos = event->pos();
+
     event->accept();
-    setDate(d->dateFromPoint(event->pos()));
+    QDate date = d->dateFromPoint(event->pos());
+    setDate(date);
+    emit dateSelected(date);
 }
 
 void CalendarTable::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
@@ -648,11 +839,15 @@ void CalendarTable::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 {
     Q_UNUSED(event);
 
+    d->lastSeenMousePos = event->pos();
+
     emit tableClicked();
 }
 
 void CalendarTable::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
 {
+    d->lastSeenMousePos = event->pos();
+
     d->updateHoveredPainting(event->pos());
 }
 
@@ -693,7 +888,9 @@ void CalendarTable::paintCell(QPainter *p, int cell, int weekRow, int weekdayCol
 
     p->setPen(numberColor);
     QFont font = Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
-    font.setBold(true);
+    if (type & Event) {
+        font.setBold(true);
+    }
     font.setPixelSize(cellArea.height() * 0.7);
     p->setFont(font);
     if (!(type & InvalidDate)) {
@@ -717,8 +914,10 @@ void CalendarTable::paintBorder(QPainter *p, int cell, int weekRow, int weekdayC
         elementId = "today";
     } else if (type & Selected) {
         elementId = "selected";
-    } else if (type & Holiday) {
+    } else if (type & PublicHoliday) {
         elementId = "red";
+    } else if (type & Holiday) {
+        elementId = "green";
     } else {
         return;
     }
@@ -739,7 +938,9 @@ void CalendarTable::paint(QPainter *p, const QStyleOptionGraphicsItem *option, Q
 
     QList<CalendarCellBorder> borders;
     QList<CalendarCellBorder> hovers;
-    QDate currentDate = QDate::currentDate(); //FIXME: calendar timezone
+    if (d->automaticUpdates){
+        d->currentDate = QDate::currentDate();
+    }
 
     //weekRow and weekDaycolumn of table are 0 indexed and are not equivalent to weekday or week
     //numbers.  In LTR mode we count/paint row and column from top-left corner, in RTL mode we
@@ -759,7 +960,8 @@ void CalendarTable::paint(QPainter *p, const QStyleOptionGraphicsItem *option, Q
             QDate cellDate = d->dateFromRowColumn(weekRow, weekdayColumn);
             CalendarTable::CellTypes type(CalendarTable::NoType);
             // get cell info
-            int cellDay = calendar()->day(cellDate);
+            const int cellDay = calendar()->day(cellDate);
+            const int julian = cellDate.toJulianDay();
 
             // check what kind of cell we are
             if (calendar()->month(cellDate) != d->selectedMonth) {
@@ -770,7 +972,7 @@ void CalendarTable::paint(QPainter *p, const QStyleOptionGraphicsItem *option, Q
                 type |= CalendarTable::InvalidDate;
             }
 
-            if (cellDate == currentDate) {
+            if (cellDate == d->currentDate) {
                 type |= CalendarTable::Today;
             }
 
@@ -778,8 +980,16 @@ void CalendarTable::paint(QPainter *p, const QStyleOptionGraphicsItem *option, Q
                 type |= CalendarTable::Selected;
             }
 
-            if (d->specialDates.contains(cellDate.toJulianDay())) {
-                type |= CalendarTable::Holiday;
+            foreach (Plasma::DataEngine::Data holidayData, d->holidays.values(julian)) {
+                if (holidayData.value("observanceType").toString() == "PublicHoliday") {
+                    type |= CalendarTable::PublicHoliday;
+                } else {
+                    type |= CalendarTable::Holiday;
+                }
+            }
+
+            if (d->events.contains(julian) || d->todos.contains(julian)) {
+                type |= CalendarTable::Event;
             }
 
             if (type != CalendarTable::NoType && type != CalendarTable::NotInCurrentMonth) {
@@ -793,6 +1003,9 @@ void CalendarTable::paint(QPainter *p, const QStyleOptionGraphicsItem *option, Q
 
             paintCell(p, cellDay, weekRow, weekdayColumn, type, cellDate);
 
+            // FIXME: modify svg to allow for a wider week number cell
+            // a temporary workaround is to paint only one week number (weekString) when the cell is small
+            // and both week numbers (accurateWeekString) when there is enough room
             if (weekdayColumn == 0) {
                 QRectF cellRect(r.x() + d->centeringSpace, y, d->cellW, d->cellH);
                 p->setPen(Theme::defaultTheme()->color(Plasma::Theme::TextColor));
@@ -801,20 +1014,35 @@ void CalendarTable::paint(QPainter *p, const QStyleOptionGraphicsItem *option, Q
                 p->setFont(font);
                 p->setOpacity(d->opacity);
                 QString weekString;
+                QString accurateWeekString;
                 if (calendar()->isValid(cellDate)) {
                     weekString = calendar()->weekNumberString(cellDate);
+                    accurateWeekString = weekString;
                     if (calendar()->dayOfWeek(cellDate) != Qt::Monday) {
                         QDate nextWeekDate = calendar()->addDays(cellDate, d->daysInWeek);
                         if (calendar()->isValid(nextWeekDate)) {
                             if (layoutDirection() == Qt::RightToLeft) {
-                                weekString.prepend("/").prepend(calendar()->weekNumberString(nextWeekDate));
+                                accurateWeekString.prepend("/").prepend(calendar()->weekNumberString(nextWeekDate));
                             } else {
-                                weekString.append("/").append(calendar()->weekNumberString(nextWeekDate));
+                                accurateWeekString.append("/").append(calendar()->weekNumberString(nextWeekDate));
+                            }
+                        }
+                        // ensure that weekString is the week number that has the most amout of days in the row
+                        QDate middleWeekDate = calendar()->addDays(cellDate, floor(static_cast<float>(d->daysInWeek / 2)));
+                        if (calendar()->isValid(middleWeekDate)) {
+                            QString middleWeekString = calendar()->weekNumberString(middleWeekDate);
+                            if (weekString != middleWeekString) {
+                                weekString = middleWeekString;
                             }
                         }
                     }
                 }
-                p->drawText(cellRect, Qt::AlignCenter, weekString); //draw number
+                QFontMetrics fontMetrics(font);
+                if (fontMetrics.width(accurateWeekString) > d->cellW) {
+                    p->drawText(cellRect, Qt::AlignCenter, weekString); //draw number
+                } else {
+                    p->drawText(cellRect, Qt::AlignCenter, accurateWeekString); //draw number
+                }
                 p->setOpacity(1.0);
             }
         }
