@@ -311,7 +311,7 @@ void GroupManagerPrivate::removeTask(TaskPtr task)
     if (!item) {
         // this can happen if the window hasn't been caught previously,
         // of it it is an ignored type such as a NET::Utility type window
-        kDebug() << "invalid item";
+        //kDebug() << "invalid item";
         return;
     }
 
