@@ -129,9 +129,9 @@ public:
         }
 
         if (m_direction == Plasma::Left || m_direction == Plasma::Right) {
-            p.drawPixmap(QPoint(0, 0), l);
-            p.drawTiledPixmap(QRect(0, l.height(), c.width(), height() - l.height() - r.height()), c);
-            p.drawPixmap(QPoint(0, height() - r.height()), r);
+            p.drawPixmap(pixmapPosition, l);
+            p.drawTiledPixmap(QRect(pixmapPosition.x(), l.height(), c.width(), height() - l.height() - r.height()), c);
+            p.drawPixmap(QPoint(pixmapPosition.x(), height() - r.height()), r);
         } else {
             p.drawPixmap(pixmapPosition, l);
             p.drawTiledPixmap(QRect(l.width(), pixmapPosition.y(), width() - l.width() - r.width(), c.height()), c);
@@ -193,35 +193,137 @@ private:
     QPixmap m_buffer;
 };
 
+
+class ShadowWindow : public QWidget
+{
+public:
+    ShadowWindow(PanelView *panel)
+       : QWidget(0),
+         m_panel(panel),
+         m_valid(false)
+    {
+        setAttribute(Qt::WA_TranslucentBackground);
+        setAttribute(Qt::WA_NoSystemBackground, false);
+        setAutoFillBackground(false);
+#ifdef Q_WS_X11
+        QRegion region(QRect(0,0,1,1));
+        XShapeCombineRegion(QX11Info::display(), winId(), ShapeInput, 0, 0,
+                            region.handle(), ShapeSet);
+#endif
+
+        m_shadow = new Plasma::FrameSvg(this);
+        connect(m_shadow, SIGNAL(repaintNeeded()), m_panel, SLOT(checkShadow()), Qt::QueuedConnection);
+    }
+
+    void setSvg(const QString &path)
+    {
+        m_shadow->setImagePath(path);
+
+        if (!m_shadow->hasElementPrefix("shadow")) {
+            hide();
+            m_valid = false;
+        } else {
+            m_valid = true;
+        }
+
+        m_shadow->setElementPrefix("shadow");
+
+        adjustMargins(geometry());
+    }
+
+    bool isValid() const
+    {
+        return m_valid;
+    }
+
+    void adjustMargins(const QRect &geo)
+    {
+        QRect screenRect = Kephal::ScreenUtils::screenGeometry(m_panel->screen());
+
+        Plasma::FrameSvg::EnabledBorders enabledBorders = Plasma::FrameSvg::AllBorders;
+
+        if (geo.left() <= screenRect.left()) {
+            enabledBorders ^= Plasma::FrameSvg::LeftBorder;
+        }
+        if (geo.top() <= screenRect.top()) {
+            enabledBorders ^= Plasma::FrameSvg::TopBorder;
+        }
+        if (geo.bottom() >= screenRect.bottom()) {
+            enabledBorders ^= Plasma::FrameSvg::BottomBorder;
+        }
+        if (geo.right() >= screenRect.right()) {
+            enabledBorders ^= Plasma::FrameSvg::RightBorder;
+        }
+
+        m_shadow->setEnabledBorders(enabledBorders);
+
+        qreal left, top, right, bottom;
+
+        m_shadow->getMargins(left, top, right, bottom);
+        setContentsMargins(left, top, right, bottom);
+    }
+
+protected:
+    bool event(QEvent *event)
+    {
+        if (event->type() == QEvent::Paint) {
+            QPainter p(this);
+            p.setCompositionMode(QPainter::CompositionMode_Source);
+            p.fillRect(rect(), Qt::transparent);
+        }
+        return QWidget::event(event);
+    }
+
+    void resizeEvent(QResizeEvent *event)
+    {
+        m_shadow->resizeFrame(event->size());
+        adjustMargins(geometry());
+    }
+
+    void paintEvent(QPaintEvent *e)
+    {
+        QPainter p(this);
+        //p.setCompositionMode(QPainter::CompositionMode_Source);
+        m_shadow->paintFrame(&p, e->rect(), e->rect());
+    }
+
+
+private:
+    Plasma::FrameSvg *m_shadow;
+    PanelView *m_panel;
+    bool m_valid;
+};
+
 PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
     : Plasma::View(panel, id, parent),
       m_panelController(0),
       m_glowBar(0),
       m_mousePollTimer(0),
-      m_strutsTimer(0),
+      m_strutsTimer(new QTimer(this)),
+      m_delayedUnhideTimer(new QTimer(this)),
       m_spacer(0),
       m_spacerIndex(-1),
+      m_shadowWindow(0),
 #ifdef Q_WS_X11
       m_unhideTrigger(None),
 #endif
       m_visibilityMode(NormalPanel),
       m_lastHorizontal(true),
-      m_init(false),
-      m_editting(false),
-      m_firstPaint(true),
+      m_editing(false),
       m_triggerEntered(false)
 {
-    kDebug() << "Panel geometry is" << panel->geometry();
+    // KWin setup
+    KWindowSystem::setOnAllDesktops(winId(), true);
+    KWindowSystem::setType(winId(), NET::Dock);
+    setWindowRole(QString("panel_%1").arg(id));
+
+    m_delayedUnhideTimer->setSingleShot(true);
+    m_delayedUnhideTimer->setInterval(200);
+    connect(m_delayedUnhideTimer, SIGNAL(timeout()), this, SLOT(delayedUnhide()));
 
-    m_strutsTimer = new QTimer(this);
     m_strutsTimer->setSingleShot(true);
     connect(m_strutsTimer, SIGNAL(timeout()), this, SLOT(updateStruts()));
 
-    connect(panel, SIGNAL(destroyed(QObject*)), this, SLOT(panelDeleted()));
-    connect(panel, SIGNAL(toolBoxToggled()), this, SLOT(togglePanelController()));
-    connect(panel, SIGNAL(appletAdded(Plasma::Applet *, const QPointF &)), this, SLOT(appletAdded(Plasma::Applet *)));
-    connect(this, SIGNAL(sceneRectAboutToChange()), this, SLOT(pinchContainmentToCurrentScreen()));
-
     // Graphics view setup
     setFrameStyle(QFrame::NoFrame);
     //setAutoFillBackground(true);
@@ -242,20 +344,82 @@ PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
 
     KConfigGroup viewConfig = config();
     KConfigGroup sizes = KConfigGroup(&viewConfig, "Sizes");
-    QRect screenRect = Kephal::ScreenUtils::screenGeometry(panel->screen());
     m_lastHorizontal = isHorizontal();
+
+    const bool onScreen = panel->screen() < Kephal::ScreenUtils::numScreens();
+    const QRect screenRect = onScreen ?  Kephal::ScreenUtils::screenGeometry(panel->screen()) : QRect();
+    if (!onScreen) {
+        resize(panel->size().toSize());
+    }
     m_lastSeenSize = sizes.readEntry("lastsize", m_lastHorizontal ? screenRect.width() : screenRect.height());
+
     m_alignment = alignmentFilter((Qt::Alignment)viewConfig.readEntry("Alignment", (int)Qt::AlignLeft));
     m_offset = viewConfig.readEntry("Offset", 0);
+    setVisibilityMode((VisibilityMode)viewConfig.readEntry("panelVisibility", (int)m_visibilityMode));
 
-    KWindowSystem::setType(winId(), NET::Dock);
-    setWindowRole(QString("panel_%1").arg(id));
+    connect(this, SIGNAL(sceneRectAboutToChange()), this, SLOT(pinchContainmentToCurrentScreen()));
+
+    Kephal::Screens *screens = Kephal::Screens::self();
+    connect(screens, SIGNAL(screenResized(Kephal::Screen *, QSize, QSize)),
+            this, SLOT(pinchContainmentToCurrentScreen()));
+    connect(screens, SIGNAL(screenMoved(Kephal::Screen *, QPoint, QPoint)),
+            this, SLOT(updatePanelGeometry()));
+    connect(screens, SIGNAL(screenAdded(Kephal::Screen *)),
+            this, SLOT(updateStruts()));
+}
+
+PanelView::~PanelView()
+{
+    if (m_panelController) {
+        disconnect(m_panelController, 0, this, 0);
+        delete m_panelController;
+    }
+
+    delete m_glowBar;
+    delete m_shadowWindow;
+    destroyUnhideTrigger();
+#ifdef Q_WS_WIN
+    registerAccessBar(false);
+#endif
+}
+
+void PanelView::setContainment(Plasma::Containment *containment)
+{
+    kDebug() << "Panel geometry is" << containment->geometry();
+
+    Plasma::Containment *oldContainment = this->containment();
+    if (oldContainment) {
+        disconnect(oldContainment, 0, this, 0);
+    }
+
+    connect(containment, SIGNAL(newStatus(Plasma::ItemStatus)), this, SLOT(setStatus(Plasma::ItemStatus)));
+    connect(containment, SIGNAL(destroyed(QObject*)), this, SLOT(panelDeleted()));
+    connect(containment, SIGNAL(toolBoxToggled()), this, SLOT(togglePanelController()));
+    connect(containment, SIGNAL(appletAdded(Plasma::Applet *, const QPointF &)), this, SLOT(appletAdded(Plasma::Applet *)));
+    connect(containment, SIGNAL(showAddWidgetsInterface(QPointF)), this, SLOT(showWidgetExplorer()));
+    connect(containment, SIGNAL(screenChanged(int,int,Plasma::Containment*)), this, SLOT(pinchContainmentToCurrentScreen()));
+
+    KConfigGroup viewIds(KGlobal::config(), "ViewIds");
+
+    if (oldContainment) {
+        viewIds.deleteEntry(QString::number(oldContainment->id()));
+    }
+
+    if (containment) {
+        viewIds.writeEntry(QString::number(containment->id()), id());
+        if (containment->corona()) {
+            containment->corona()->requestConfigSync();
+        }
+    }
 
     // ensure we aren't overlapping other panels
-    const QRegion availGeom = PlasmaApp::self()->corona()->availableScreenRegion(panel->screen());
-    const int w = panel->size().width();
-    const int h = panel->size().height();
-    const int length = panel->formFactor() == Plasma::Horizontal ? w : h;
+    const QRect screenRect = Kephal::ScreenUtils::screenGeometry(containment->screen());
+    const QRegion availGeom = PlasmaApp::self()->corona()->availableScreenRegion(containment->screen());
+    const int w = containment->size().width();
+    const int h = containment->size().height();
+    const int length = containment->formFactor() == Plasma::Horizontal ? w : h;
+
+    View::setContainment(containment);
 
     switch (location()) {
         case Plasma::LeftEdge: {
@@ -288,50 +452,38 @@ PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
 
     // pinchContainment calls updatePanelGeometry for us
     pinchContainment(screenRect);
-    m_lastMin = panel->minimumSize();
-    m_lastMax = panel->maximumSize();
+    m_lastMin = containment->minimumSize();
+    m_lastMax = containment->maximumSize();
 
-    // KWin setup
-    KWindowSystem::setOnAllDesktops(winId(), true);
-    kDebug() << "about to set the containment" << (QObject*)panel;
-    setContainment(panel);
-    init();
-//    QTimer::singleShot(0, this, SLOT(init()));
-}
+    kDebug() << "about to set the containment" << (QObject*)containment;
 
-PanelView::~PanelView()
-{
-    if (m_panelController) {
-        disconnect(m_panelController, 0, this, 0);
-        delete m_panelController;
-    }
-
-    delete m_glowBar;
-    destroyUnhideTrigger();
-#ifdef Q_WS_WIN
-    registerAccessBar(false);
-#endif
-}
-
-void PanelView::init()
-{
-    KConfigGroup viewConfig = config();
-    setVisibilityMode((VisibilityMode)viewConfig.readEntry("panelVisibility", (int)m_visibilityMode));
-
-    m_init = true;
     updateStruts();
 
-    Kephal::Screens *screens = Kephal::Screens::self();
-    connect(screens, SIGNAL(screenResized(Kephal::Screen *, QSize, QSize)),
-            this, SLOT(pinchContainmentToCurrentScreen()));
-    connect(screens, SIGNAL(screenMoved(Kephal::Screen *, QPoint, QPoint)),
-            this, SLOT(updatePanelGeometry()));
-    connect(screens, SIGNAL(screenAdded(Kephal::Screen *)),
-            this, SLOT(updateStruts()));
-    connect(screens, SIGNAL(screenRemoved(int)),
-            this, SLOT(updateStruts()));
-    connect(containment(), SIGNAL(showAddWidgetsInterface(QPointF)),
-	    this, SLOT(showWidgetExplorer()));
+    checkShadow();
+}
+
+void PanelView::checkShadow()
+{
+    if (KWindowSystem::compositingActive() && containment()->property("shadowPath").isValid()) {
+        if (!m_shadowWindow) {
+            m_shadowWindow = new ShadowWindow(this);
+            KWindowSystem::setOnAllDesktops(winId(), true);
+        }
+        KWindowSystem::setType(m_shadowWindow->winId(), NET::Dock);
+        KWindowSystem::setState(m_shadowWindow->winId(), NET::KeepBelow);
+        KWindowSystem::setOnAllDesktops(m_shadowWindow->winId(), true);
+        m_shadowWindow->setSvg(containment()->property("shadowPath").toString());
+        int left, right, top, bottom;
+        m_shadowWindow->adjustMargins(geometry());
+        m_shadowWindow->getContentsMargins(&left, &top, &right, &bottom);
+        m_shadowWindow->setGeometry(geometry().adjusted(-left, -top, right, bottom));
+        if (m_shadowWindow->isValid() && isVisible()) {
+            m_shadowWindow->show();
+        }
+    } else {
+        m_shadowWindow->deleteLater();
+        m_shadowWindow = 0;
+    }
 }
 
 void PanelView::setLocation(Plasma::Location location)
@@ -394,7 +546,7 @@ void PanelView::setLocation(Plasma::Location location)
 #ifdef Q_WS_WIN
     appBarPosChanged();
 #endif
-    QRect screenRect = Kephal::ScreenUtils::screenGeometry(c->screen());
+    const QRect screenRect = Kephal::ScreenUtils::screenGeometry(c->screen());
     pinchContainment(screenRect);
     KWindowSystem::setOnAllDesktops(winId(), true);
     //updatePanelGeometry();
@@ -423,21 +575,28 @@ void PanelView::setVisibilityMode(PanelView::VisibilityMode mode)
     unhide();
 
     disconnect(containment(), SIGNAL(activate()), this, SLOT(unhide()));
+    disconnect(containment(), SIGNAL(newStatus(Plasma::ItemStatus)), this, SLOT(checkUnhide(Plasma::ItemStatus)));
     if (mode == NormalPanel || mode == WindowsGoBelow) {
         //remove the last remnants of hide/unhide
         delete m_mousePollTimer;
         m_mousePollTimer = 0;
     } else {
         connect(containment(), SIGNAL(activate()), this, SLOT(unhide()));
+        connect(containment(), SIGNAL(newStatus(Plasma::ItemStatus)), this, SLOT(checkUnhide(Plasma::ItemStatus)));
     }
 
     config().writeEntry("panelVisibility", (int)mode);
 
     //if the user didn't cause this, hide again in a bit
-    if ((mode == AutoHide || mode == LetWindowsCover) && !m_editting) {
-        m_mousePollTimer->stop();
+    if ((mode == AutoHide || mode == LetWindowsCover) && !m_editing) {
+        if (m_mousePollTimer) {
+            m_mousePollTimer->stop();
+        }
+
         QTimer::singleShot(2000, this, SLOT(startAutoHide()));
     }
+
+    KWindowSystem::setOnAllDesktops(winId(), true);
 }
 
 PanelView::VisibilityMode PanelView::visibilityMode() const
@@ -592,6 +751,12 @@ void PanelView::updatePanelGeometry()
         m_strutsTimer->start(STRUTSTIMERDELAY);
     } else {
         setGeometry(geom);
+        if (m_shadowWindow) {
+            int left, right, top, bottom;
+            m_shadowWindow->adjustMargins(geometry());
+            m_shadowWindow->getContentsMargins(&left, &top, &right, &bottom);
+            m_shadowWindow->setGeometry(geometry().adjusted(-left, -top, right, bottom));
+        }
     }
 
     m_lastMin = c->minimumSize();
@@ -599,10 +764,7 @@ void PanelView::updatePanelGeometry()
 
     //update the panel controller location position and size
     if (m_panelController) {
-        if (m_panelController->isVisible()) {
-            m_panelController->resize(m_panelController->sizeHint());
-            m_panelController->move(m_panelController->positionForPanelGeometry(geometry()));
-        }
+
         m_panelController->setLocation(c->location());
 
         foreach (PanelAppletOverlay *o, m_appletOverlays) {
@@ -622,7 +784,7 @@ bool PanelView::isHorizontal() const
 void PanelView::pinchContainmentToCurrentScreen()
 {
     kDebug() << "pinching to current screen";
-    QRect screenRect = Kephal::ScreenUtils::screenGeometry(containment()->screen());
+    const QRect screenRect = Kephal::ScreenUtils::screenGeometry(containment()->screen());
     pinchContainment(screenRect);
 }
 
@@ -769,13 +931,12 @@ void PanelView::pinchContainment(const QRect &screenGeom)
     updatePanelGeometry();
 
     if (m_panelController) {
-        m_panelController->setContainment(c);
         m_panelController->setOffset(m_offset);
     }
 
     connect(this, SIGNAL(sceneRectAboutToChange()), this, SLOT(pinchContainmentToCurrentScreen()));
     recreateUnhideTrigger();
-    kDebug() << "Done pinching, containement's geom" << c->geometry() << "own geom" << geometry();
+    kDebug() << "Done pinching, containment's geom" << c->geometry() << "own geom" << geometry();
 }
 
 void PanelView::setOffset(int newOffset)
@@ -813,7 +974,7 @@ Qt::Alignment PanelView::alignment() const
 void PanelView::togglePanelController()
 {
     //kDebug();
-    m_editting = false;
+    m_editing = false;
     if (containment()->immutability() != Plasma::Mutable) {
         delete m_panelController;
         m_panelController = 0;
@@ -828,13 +989,14 @@ void PanelView::togglePanelController()
         m_panelController->setOffset(m_offset);
         m_panelController->setVisibilityMode(m_visibilityMode);
 
-        connect(m_panelController, SIGNAL(destroyed(QObject*)), this, SLOT(edittingComplete()));
+        connect(m_panelController, SIGNAL(destroyed(QObject*)), this, SLOT(editingComplete()));
         connect(m_panelController, SIGNAL(offsetChanged(int)), this, SLOT(setOffset(int)));
         connect(m_panelController, SIGNAL(alignmentChanged(Qt::Alignment)), this, SLOT(setAlignment(Qt::Alignment)));
         connect(m_panelController, SIGNAL(locationChanged(Plasma::Location)), this, SLOT(setLocation(Plasma::Location)));
         connect(m_panelController, SIGNAL(panelVisibilityModeChanged(PanelView::VisibilityMode)), this, SLOT(setVisibilityMode(PanelView::VisibilityMode)));
 
-        if (dynamic_cast<QGraphicsLinearLayout*>(containment()->layout())) {
+        if (containment()->containmentType() == Plasma::Containment::PanelContainment && 
+	    dynamic_cast<QGraphicsLinearLayout*>(containment()->layout())) {
             setTabOrder(0, m_panelController);
             QWidget *prior = m_panelController;
 
@@ -861,7 +1023,7 @@ void PanelView::togglePanelController()
     }
 
     if (!m_panelController->isVisible()) {
-        m_editting = true;
+        m_editing = true;
         m_panelController->resize(m_panelController->sizeHint());
         m_panelController->move(m_panelController->positionForPanelGeometry(geometry()));
         Plasma::WindowEffects::slideWindow(m_panelController, location());
@@ -874,12 +1036,11 @@ void PanelView::togglePanelController()
     }
 }
 
-//FIXME "editing"
-void PanelView::edittingComplete()
+void PanelView::editingComplete()
 {
     //kDebug();
     m_panelController = 0;
-    m_editting = false;
+    m_editing = false;
     qDeleteAll(m_appletOverlays);
     m_appletOverlays.clear();
 
@@ -889,10 +1050,9 @@ void PanelView::edittingComplete()
 
     containment()->closeToolBox();
     updateStruts();
-    m_firstPaint = true; // triggers autohide FIXME does not! delete this var.
 
     if (m_visibilityMode == LetWindowsCover || m_visibilityMode == AutoHide) {
-         hideMousePoll();
+        hideIfNotInUse();
     }
 }
 
@@ -921,7 +1081,7 @@ Qt::Alignment PanelView::alignmentFilter(Qt::Alignment align) const
 
 void PanelView::updateStruts()
 {
-    if (!m_init || !containment()) {
+    if (!containment()) {
         return;
     }
 
@@ -1025,6 +1185,7 @@ void PanelView::updateStruts()
         if (m_panelController->isVisible()) {
             m_panelController->resize(m_panelController->sizeHint());
             m_panelController->move(m_panelController->positionForPanelGeometry(geometry()));
+            Plasma::WindowEffects::slideWindow(m_panelController, location());
         }
 
         foreach (PanelAppletOverlay *o, m_appletOverlays) {
@@ -1035,16 +1196,18 @@ void PanelView::updateStruts()
     recreateUnhideTrigger();
 }
 
-/*
-// handy for debugging :)
 void PanelView::enterEvent(QEvent *event)
 {
+    // allow unhiding to happen again even if we were delay-unhidden
+    m_delayedUnhideTs = QTime();
+/*
+// handy for debugging :)
     if (containment()) {
         kDebug() << sceneRect() << containment()->geometry();
     }
+*/
     Plasma::View::enterEvent(event);
 }
-*/
 
 void PanelView::showWidgetExplorer()
 {
@@ -1089,25 +1252,24 @@ void PanelView::resizeEvent(QResizeEvent *event)
     }
 }
 
-//FIXME that's a confusing function name
-void PanelView::hideMousePoll()
+void PanelView::hideIfNotInUse()
 {
-    QPoint mousePos = QCursor::pos();
-
-    if (!geometry().contains(mousePos) && !hasPopup()) {
+    //kDebug() << m_delayedUnhideTs.elapsed() << geometry().contains(QCursor::pos()) << hasPopup();
+    //TODO: is 5s too long? not long enough?
+    if ((m_delayedUnhideTs.isNull() || m_delayedUnhideTs.elapsed() > 5000) &&
+        !geometry().adjusted(-10, -10, 10, 10).contains(QCursor::pos()) && !hasPopup()) {
         startAutoHide();
     }
 }
 
-//FIXME that's a confusing function name
-void PanelView::unhideHintMousePoll()
+void PanelView::updateHinter()
 {
 #ifdef Q_WS_X11
     QPoint mousePos = QCursor::pos();
     m_glowBar->updateStrength(mousePos);
 
     if (!m_unhideTriggerGeom.contains(mousePos)) {
-        unhintHide();
+        hideHinter();
         XMoveResizeWindow(QX11Info::display(), m_unhideTrigger, m_unhideTriggerGeom.x(), m_unhideTriggerGeom.y(), m_unhideTriggerGeom.width(), m_unhideTriggerGeom.height());
     }
 #endif
@@ -1156,8 +1318,7 @@ bool PanelView::hintOrUnhide(const QPoint &point, bool dueToDnd)
             m_mousePollTimer = new QTimer(this);
         }
 
-        disconnect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(unhideHintMousePoll()));
-        connect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(unhideHintMousePoll()));
+        connect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(updateHinter()), Qt::UniqueConnection);
         m_mousePollTimer->start(200);
     }
 
@@ -1167,13 +1328,12 @@ bool PanelView::hintOrUnhide(const QPoint &point, bool dueToDnd)
 #endif
 }
 
-//FIXME that's a confusing function name
-void PanelView::unhintHide()
+void PanelView::hideHinter()
 {
     //kDebug() << "hide the glow";
     if (m_mousePollTimer) {
         m_mousePollTimer->stop();
-        disconnect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(unhideHintMousePoll()));
+        disconnect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(updateHinter()));
     }
 
     delete m_glowBar;
@@ -1200,16 +1360,24 @@ bool PanelView::hasPopup()
 void PanelView::unhide(bool destroyTrigger)
 {
     //kill the unhide stuff
-    unhintHide();
+    m_delayedUnhideTimer->stop();
+    hideHinter();
     if (destroyTrigger) {
         destroyUnhideTrigger();
     }
 
     //ensure it's visible
-    if (! isVisible()) {
+    if (!isVisible()) {
         Plasma::WindowEffects::slideWindow(this, location());
         show();
         KWindowSystem::setOnAllDesktops(winId(), true);
+
+        if (m_shadowWindow && m_shadowWindow->isValid()) {
+            Plasma::WindowEffects::slideWindow(m_shadowWindow, location());
+            m_shadowWindow->show();
+            KWindowSystem::setState(m_shadowWindow->winId(), NET::KeepBelow);
+            KWindowSystem::setOnAllDesktops(m_shadowWindow->winId(), true);
+        }
     }
 
     //non-hiding panels stop here
@@ -1221,11 +1389,8 @@ void PanelView::unhide(bool destroyTrigger)
     if (!m_mousePollTimer) {
         m_mousePollTimer = new QTimer(this);
     }
-    //FIXME investigate whether the mouse poll is really needed all the time or if leave events are
-    //good enough
-    //welll, if we didn't create it here something would have to when a popup dies
-    disconnect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideMousePoll()));
-    connect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideMousePoll()));
+
+    connect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideIfNotInUse()), Qt::UniqueConnection);
     m_mousePollTimer->start(200);
 
     //avoid hide-show loops
@@ -1236,6 +1401,34 @@ void PanelView::unhide(bool destroyTrigger)
     }
 }
 
+void PanelView::setStatus(Plasma::ItemStatus newStatus)
+{
+    if (newStatus == Plasma::AcceptingInputStatus) {
+        KWindowSystem::forceActiveWindow(winId());
+    }
+}
+
+void PanelView::checkUnhide(Plasma::ItemStatus newStatus)
+{
+    //kDebug() << "================= got a new status: " << newStatus << Plasma::ActiveStatus;
+
+    if (newStatus > Plasma::ActiveStatus) {
+        if (!m_delayedUnhideTimer->isActive()) {
+            m_delayedUnhideTimer->start();
+        }
+
+    } else {
+        m_delayedUnhideTimer->stop();
+    }
+}
+
+void PanelView::delayedUnhide()
+{
+    m_delayedUnhideTs.start();
+    //kDebug() << "starting at:" << m_delayedUnhideTs << m_delayedUnhideTs.elapsed();
+    unhide(true);
+}
+
 void PanelView::unhide()
 {
     unhide(true);
@@ -1250,7 +1443,7 @@ void PanelView::startAutoHide()
 {
     if (m_mousePollTimer) {
         m_mousePollTimer->stop();
-        disconnect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideMousePoll()));
+        disconnect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideIfNotInUse()));
     }
 
     if (m_visibilityMode == LetWindowsCover) {
@@ -1259,6 +1452,10 @@ void PanelView::startAutoHide()
     } else {
         Plasma::WindowEffects::slideWindow(this, location());
         createUnhideTrigger();
+        if (m_shadowWindow) {
+            Plasma::WindowEffects::slideWindow(m_shadowWindow, location());
+            m_shadowWindow->hide();
+        }
         hide();
     }
 }
@@ -1269,7 +1466,7 @@ void PanelView::leaveEvent(QEvent *event)
         //this prevents crazy hide-unhide loops that can happen at times
         m_triggerEntered = false;
     } else if (containment() &&
-               (m_visibilityMode == AutoHide || m_visibilityMode == LetWindowsCover) && !m_editting) {
+               (m_visibilityMode == AutoHide || m_visibilityMode == LetWindowsCover) && !m_editing) {
         // even if we dont have a popup, we'll start a timer, so
         // that the panel stays if the mouse only leaves for a
         // few ms
@@ -1277,8 +1474,7 @@ void PanelView::leaveEvent(QEvent *event)
             m_mousePollTimer = new QTimer(this);
         }
 
-        disconnect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideMousePoll()));
-        connect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideMousePoll()));
+        connect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideIfNotInUse()), Qt::UniqueConnection);
         m_mousePollTimer->start(200);
     }
 
@@ -1298,9 +1494,6 @@ void PanelView::drawBackground(QPainter *painter, const QRectF &rect)
 void PanelView::paintEvent(QPaintEvent *event)
 {
     Plasma::View::paintEvent(event);
-    if (m_firstPaint) {
-        m_firstPaint = false;
-    }
 }
 
 bool PanelView::event(QEvent *event)
@@ -1316,7 +1509,7 @@ bool PanelView::event(QEvent *event)
 
 void PanelView::appletAdded(Plasma::Applet *applet)
 {
-    if (m_panelController) {
+    if (m_panelController && containment()->containmentType() == Plasma::Containment::PanelContainment) {
         QColor overlayColor(Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor));
         QBrush overlayBrush(overlayColor);
         QPalette p(palette());
@@ -1565,6 +1758,5 @@ void PanelView::dropEvent(QDropEvent *event)
     }
 }
 
-
 #include "panelview.moc"
 
