@@ -24,6 +24,7 @@
 //Qt
 #include <QtDBus/QDBusInterface>
 #include <QtDBus/QDBusReply>
+#include <QLabel>
 #include <QMetaEnum>
 #include <QGraphicsLinearLayout>
 #include <QGraphicsSceneContextMenuEvent>
@@ -38,6 +39,7 @@
 #include <Plasma/ItemBackground>
 #include <Plasma/ScrollWidget>
 #include <Plasma/Separator>
+#include <Plasma/TextBrowser>
 
 //solid
 #include <solid/device.h>
@@ -56,6 +58,7 @@ NotifierDialog::NotifierDialog(DeviceNotifier * notifier, QObject *parent)
     : QObject(parent),
       m_widget(0),
       m_notifier(notifier),
+      m_deviceCount(0),
       m_collapsing(false)
 {
     buildDialog();
@@ -73,6 +76,8 @@ NotifierDialog::NotifierDialog(DeviceNotifier * notifier, QObject *parent)
     m_clearItemBackgroundTargetTimer.setInterval(100);
     connect(&m_clearItemBackgroundTargetTimer, SIGNAL(timeout()), this, SLOT(clearItemBackgroundTarget()));
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(updateColorsLater()));
+
+    setMenuActionsAt(QPointF(0, 0));
 }
 
 NotifierDialog::~NotifierDialog()
@@ -106,6 +111,9 @@ int NotifierDialog::searchOrCreateDeviceCategory(const QString &categoryName)
         }
     }
 
+    //Poor man's category separator
+    //TODO: make own widget?
+
     Plasma::Separator *separator = new Plasma::Separator();
     separator->setOrientation(Qt::Horizontal);
     separator->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
@@ -113,7 +121,10 @@ int NotifierDialog::searchOrCreateDeviceCategory(const QString &categoryName)
 
     Plasma::Label *category = new Plasma::Label();
     category->setText(categoryName);
-    category->setAlignment(Qt::AlignRight);
+    category->setAlignment(Qt::AlignLeft);
+    // The margins here are needed in order to have the category label
+    // vertically aligned with device icons, which just looks better
+    category->nativeWidget()->setContentsMargins(4+4, 0, 0, 0);
     category->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
     updateCategoryColors(category);
     m_deviceLayout->insertItem(1, category);
@@ -127,6 +138,7 @@ void NotifierDialog::insertDevice(const QString &udi)
         return;
     }
 
+    ++m_deviceCount;
     DeviceItem *devItem = new DeviceItem(udi);
     connect(devItem, SIGNAL(leftActionActivated(DeviceItem *)), this, SLOT(leftActionActivated(DeviceItem *)));
     connect(devItem, SIGNAL(actionActivated(DeviceItem *, const QString &, const QString &)),
@@ -146,6 +158,28 @@ void NotifierDialog::insertDevice(const QString &udi)
 
     m_deviceLayout->insertItem(index + 1, devItem);
 
+    if (device.is<Solid::OpticalDisc>()) {
+        Solid::OpticalDrive *drive = device.parent().as<Solid::OpticalDrive>();
+        if (drive) {
+            connect(drive, SIGNAL(ejectRequested(const QString&)),
+                    this, SLOT(ejectRequested(const QString&)));
+            connect(drive, SIGNAL(ejectDone(Solid::ErrorType, QVariant, const QString &)),
+                    this, SLOT(storageEjectDone(Solid::ErrorType, QVariant , const QString &)));
+        }
+    } else if (device.is<Solid::StorageVolume>()) {
+        Solid::StorageAccess *access = device.as<Solid::StorageAccess>();
+        if (access ) {
+            connect(access, SIGNAL(teardownRequested(const QString &)),
+                    this, SLOT(teardownRequested(const QString &)));
+            connect(access, SIGNAL(teardownDone(Solid::ErrorType, QVariant, const QString &)),
+                    this, SLOT(storageTeardownDone(Solid::ErrorType, QVariant , const QString &)));
+            connect(access, SIGNAL(setupRequested(const QString &)),
+                    this, SLOT(setupRequested(const QString &)));
+            connect(access, SIGNAL(setupDone(Solid::ErrorType, QVariant, const QString &)),
+                    this, SLOT(storageSetupDone(Solid::ErrorType, QVariant , const QString &)));
+        }
+    }
+
     collapseDevices();
     resetSelection();
     updateMainLabelText();
@@ -373,6 +407,25 @@ DeviceItem* NotifierDialog::hoveredItem()
 void NotifierDialog::setMounted(bool mounted, const QString &udi)
 {
     DeviceItem *item = itemForUdi(udi);
+
+    if (mounted) {
+        QString parentUdi = Solid::Device(udi).parent().udi();
+        foreach (DeviceItem* sibling, itemsForParentUdi(parentUdi)) {
+            sibling->setSafelyRemovable(false);
+        }
+    } else {
+        bool safelyRemovable = true;
+        QString parentUdi = Solid::Device(udi).parent().udi();
+        foreach (DeviceItem* sibling, itemsForParentUdi(parentUdi)) {
+            if ((sibling->udi() != udi) && (sibling->isMounted())) {
+                safelyRemovable = false;
+            }
+        }
+        foreach (DeviceItem* sibling, itemsForParentUdi(parentUdi)) {
+                sibling->setSafelyRemovable(safelyRemovable);
+        }
+    }
+
     item->setMounted(mounted);
     updateFreeSpace(item);
 }
@@ -410,12 +463,15 @@ void NotifierDialog::removeDevice(const QString &udi)
         return;
     }
 
+    expireStatusBar(udi);
+
     disconnect(item, 0, this, 0);
     item->removeEventFilter(this);
 
     resetSelection();
     m_deviceLayout->removeItem(item);
     item->deleteLater();
+    --m_deviceCount;
 
     for (int i = 0; i < m_deviceLayout->count(); ++i) {
         Plasma::Separator *separator = dynamic_cast<Plasma::Separator *>(m_deviceLayout->itemAt(i));
@@ -424,9 +480,11 @@ void NotifierDialog::removeDevice(const QString &udi)
             if (!nextItem) {
                 m_deviceLayout->removeAt(i);
                 QGraphicsLayoutItem *category = m_deviceLayout->itemAt(i);
-                m_deviceLayout->removeAt(i);
+                if (category) {
+                    m_deviceLayout->removeAt(i);
+                    delete category;
+                }
                 delete separator;
-                delete category;
                 --i;
             }
         }
@@ -441,8 +499,6 @@ void NotifierDialog::removeDevice(const QString &udi)
     }
 
     updateMainLabelText();
-
-    m_devicesScrollWidget->widget()->adjustSize();
 }
 
 void NotifierDialog::removeAction(const QString &udi, const QString &action)
@@ -456,42 +512,29 @@ void NotifierDialog::buildDialog()
     m_widget->installEventFilter(this);
     m_widget->setFocusPolicy(Qt::ClickFocus);
 
-    QGraphicsLinearLayout *l_layout = new QGraphicsLinearLayout(Qt::Vertical, m_widget);
-    l_layout->setSpacing(0);
+    m_mainLayout = new QGraphicsLinearLayout(Qt::Vertical);
+    m_mainLayout->setSpacing(0);
 
-    Plasma::IconWidget *icon = new Plasma::IconWidget(m_widget);
-    icon->setIcon(KIcon("emblem-mounted"));
-    icon->setMaximumHeight(KIconLoader::SizeMedium);
-    icon->setMinimumHeight(KIconLoader::SizeMedium);
-    icon->setAcceptHoverEvents(false);
-    m_mainLabel = new Plasma::Label(m_widget);
-    m_mainLabel->setMaximumHeight(KIconLoader::SizeMedium);
+    m_mainLabel = new Plasma::Label();
     m_mainLabel->nativeWidget()->setWordWrap(false);
-    m_mainLabel->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
-    QGraphicsLinearLayout *l_layout2 = new QGraphicsLinearLayout;
-    l_layout2->setSpacing(0);
-    l_layout2->setOrientation(Qt::Horizontal);
+    m_mainLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    m_mainLabel->nativeWidget()->setContentsMargins(0, 0, 0, 4);
 
-    l_layout2->addStretch();
-    l_layout2->addItem(icon);
-    l_layout2->addItem(m_mainLabel);
-    l_layout2->addStretch();
+    m_mainLayout->addItem(m_mainLabel);
+    m_mainLayout->setAlignment(m_mainLabel, Qt::AlignCenter);
 
-    QGraphicsWidget *titleWidget = new QGraphicsWidget();
-    titleWidget->setLayout(l_layout2);
-
-    l_layout->addItem(titleWidget);
-
-    m_devicesScrollWidget = new Plasma::ScrollWidget(m_widget);
-    QGraphicsWidget *devicesWidget = new QGraphicsWidget(m_devicesScrollWidget);
+    m_devicesScrollWidget = new Plasma::ScrollWidget();
+    QGraphicsWidget *devicesWidget = new QGraphicsWidget();
     m_devicesScrollWidget->setWidget(devicesWidget);
     m_devicesScrollWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    m_devicesScrollWidget->setMinimumSize(240,250);
-    m_deviceLayout = new QGraphicsLinearLayout(Qt::Vertical, devicesWidget);
+    //These hardcoded sizes should die one day
+    m_devicesScrollWidget->setMinimumSize(240, 250);
+    m_deviceLayout = new QGraphicsLinearLayout(Qt::Vertical);
     m_deviceLayout->setContentsMargins(0, 0, 0, 8);
     devicesWidget->setLayout(m_deviceLayout);
 
-    l_layout->addItem(m_devicesScrollWidget);
+    m_mainLayout->addItem(m_devicesScrollWidget);
+    m_mainLayout->setStretchFactor(m_devicesScrollWidget, 100);
 
     m_itemBackground = new Plasma::ItemBackground(devicesWidget);
     m_selectedItemBackground = new Plasma::ItemBackground(devicesWidget);
@@ -501,60 +544,162 @@ void NotifierDialog::buildDialog()
     connect(m_itemBackground, SIGNAL(animationStep(qreal)), this, SLOT(itemBackgroundMoving(qreal)));
     connect(m_selectedItemBackground, SIGNAL(animationStep(qreal)), this, SLOT(itemBackgroundMoving(qreal)));
 
+    m_statusWidget = new QGraphicsWidget();
+    QGraphicsLinearLayout *statusLayout = new QGraphicsLinearLayout(Qt::Vertical);
+    m_statusWidget->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
+
+    m_statusWidget->setLayout(statusLayout);
+    Plasma::Separator *statusSeparator = new Plasma::Separator();
+    statusSeparator->setOrientation(Qt::Horizontal);
+    statusSeparator->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
+    statusLayout->addItem(statusSeparator);
+
+    m_statusText = new Plasma::Label();
+    m_statusText->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
+
+    Plasma::IconWidget *closeButton = new Plasma::IconWidget();
+    closeButton->setSvg("widgets/configuration-icons", "close");
+    closeButton->setMaximumSize(closeButton->sizeFromIconSize(KIconLoader::SizeSmall));
+    closeButton->setMinimumSize(closeButton->maximumSize());
+
+    m_statusExpandButton = new Plasma::IconWidget();
+    m_statusExpandButton->setMaximumSize(closeButton->sizeFromIconSize(KIconLoader::SizeSmall));
+    m_statusExpandButton->setMinimumSize(closeButton->maximumSize());
+
+    connect(closeButton, SIGNAL(clicked()), this, SLOT(dismissStatusBar()));
+    connect(m_statusExpandButton, SIGNAL(clicked()), this, SLOT(triggerExpandStatusBar()));
+
+    //WORKAROUND: We need to wrap our label layout in a QGW before adding it to our
+    //            main layout to avoid issues with Qt not properly dealing with
+    //            wordwrap enabled labels
+
+    QGraphicsWidget *labelWidget = new QGraphicsWidget();
+    labelWidget->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
+    labelWidget->setContentsMargins(0, 0, 0, 0);
+
+    QGraphicsLinearLayout *labelLayout = new QGraphicsLinearLayout(Qt::Horizontal);
+
+    labelLayout->addItem(m_statusText);
+    labelLayout->setAlignment(m_statusText, Qt::AlignTop);
+    labelLayout->addItem(m_statusExpandButton);
+    labelLayout->setAlignment(m_statusExpandButton, Qt::AlignTop);
+    labelLayout->addItem(closeButton);
+    labelLayout->setAlignment(closeButton, Qt::AlignTop);
+    labelWidget->setLayout(labelLayout);
+
+    m_statusDetailsText = new Plasma::TextBrowser();
+    m_statusDetailsText->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
+    m_statusDetailsText->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    m_statusDetailsText->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    QFont font = m_statusDetailsText->font();
+    font.setPointSize(KGlobalSettings::smallestReadableFont().pointSize());
+    m_statusDetailsText->setFont(font);
+    m_statusDetailsText->hide();
+
+    statusLayout->addItem(labelWidget);
+
     devicesWidget->adjustSize();
     updateMainLabelText();
 
-    m_widget->setLayout(l_layout);
+    m_widget->setLayout(m_mainLayout);
+}
+
+void NotifierDialog::dismissStatusBar()
+{
+    m_statusWidget->hide();
+    m_mainLayout->removeItem(m_statusWidget);
+    m_errorUdi = QString();
+}
+
+void NotifierDialog::expireStatusBar(const QString& udi)
+{
+    if (udi == m_errorUdi) {
+        dismissStatusBar();
+    }
+}
+
+void NotifierDialog::showStatusBarMessage(const QString & message, const QString &details, const QString &udi)
+{
+    m_statusText->setText(message);
+    m_mainLayout->addItem(m_statusWidget);
+    m_statusWidget->show();
+    m_statusDetailsText->setText(details);
+    showStatusBarDetails(false);
+    m_errorUdi = udi;
+}
+
+void NotifierDialog::triggerExpandStatusBar()
+{
+    showStatusBarDetails(!m_statusDetailsText->isVisible());
+}
+
+void NotifierDialog::showStatusBarDetails(bool show)
+{
+    Plasma::Svg* svg = new Plasma::Svg();
+    svg->setImagePath("widgets/configuration-icons");
+    svg->resize();
+
+    if (show) {
+        m_statusDetailsText->show();
+        static_cast<QGraphicsLinearLayout*>(m_statusWidget->layout())->addItem(m_statusDetailsText);
+        m_statusExpandButton->setIcon(QIcon(svg->pixmap("collapse")));
+    } else {
+        m_statusDetailsText->hide();
+        static_cast<QGraphicsLinearLayout*>(m_statusWidget->layout())->removeItem(m_statusDetailsText);
+        m_statusExpandButton->setIcon(QIcon(svg->pixmap("restore")));
+    }
 }
 
 void NotifierDialog::storageTeardownDone(Solid::ErrorType error, QVariant errorData, const QString & udi)
 {
-    if (error && errorData.isValid()) {
-        m_notifier->showErrorMessage(i18n("Could not unmount the device.\nOne or more files on this device are open within an application."));
-    } else {
-        m_notifier->changeNotifierIcon("dialog-ok");
-        m_notifier->update();
-        QTimer::singleShot(5000, this, SLOT(resetNotifierIcon()));
+    DeviceItem* devItem = itemForUdi(udi);
+    if (!devItem) {
+        return;
     }
 
-    //show the message only one time
-    disconnect(sender(), SIGNAL(teardownDone(Solid::ErrorType, QVariant, const QString &)),
-               this, SLOT(storageTeardownDone(Solid::ErrorType, QVariant, const QString &)));
-    itemForUdi(udi)->setReady();
+    if (!error || !errorData.isValid()) {
+        m_notifier->changeNotifierIcon("dialog-ok", 5000);
+        m_notifier->update();
+        expireStatusBar(udi);
+    }
 
+    devItem->setState(DeviceItem::Idle);
 }
 
 void NotifierDialog::storageEjectDone(Solid::ErrorType error, QVariant errorData, const QString &udi)
 {
     Q_UNUSED(udi);
 
-    if (error && errorData.isValid()) {
-        m_notifier->showErrorMessage(i18n("Cannot eject the disc.\nOne or more files on this disc are open within an application."));
-    } else {
-        m_notifier->changeNotifierIcon("dialog-ok");
+    if (!error || !errorData.isValid()) {
+        m_notifier->changeNotifierIcon("dialog-ok", 2000);
         m_notifier->update();
-        QTimer::singleShot(2000, this, SLOT(resetNotifierIcon()));
+        expireStatusBar(udi);
     }
 
-    //show the message only one time
-    disconnect(sender(), SIGNAL(ejectDone(Solid::ErrorType, QVariant, const QString &)),
-               this, SLOT(storageEjectDone(Solid::ErrorType, QVariant, const QString &)));
+    QList<DeviceItem*> deviceList = itemsForParentUdi(udi);
+    if (deviceList.isEmpty()) {
+        kDebug() << "This should just not happen";
+        return;
+    }
+
+    foreach (DeviceItem* item, deviceList) {
+        item->setState(DeviceItem::Idle);
+    }
 }
 
 void NotifierDialog::storageSetupDone(Solid::ErrorType error, QVariant errorData, const QString &udi)
 {
-     if (error && errorData.isValid()) {
-        m_notifier->showErrorMessage(i18n("Cannot mount the disc."));
-    } else {
-        m_notifier->changeNotifierIcon("dialog-ok");
-        m_notifier->update();
-        QTimer::singleShot(2000, this, SLOT(resetNotifierIcon()));
+    DeviceItem *devItem = itemForUdi(udi);
+    if (!devItem) {
+        return;
     }
 
-    //show the message only one time
-    disconnect(sender(), SIGNAL(setupDone(Solid::ErrorType, QVariant, const QString &)),
-               this, SLOT(storageSetupDone(Solid::ErrorType, QVariant, const QString &)));
-    itemForUdi(udi)->setReady();
+     if (!error || !errorData.isValid()) {
+        m_notifier->changeNotifierIcon("dialog-ok", 2000);
+        m_notifier->update();
+        expireStatusBar(udi);
+    }
+    devItem->setState(DeviceItem::Idle);
 }
 
 DeviceItem *NotifierDialog::itemForUdi(const QString &udi) const
@@ -570,6 +715,19 @@ DeviceItem *NotifierDialog::itemForUdi(const QString &udi) const
     return 0;
 }
 
+QList<DeviceItem*> NotifierDialog::itemsForParentUdi(const QString &udi) const
+{
+    QList<DeviceItem*> deviceList;
+    for (int i=0; i<m_deviceLayout->count(); i++) {
+        DeviceItem* item = dynamic_cast<DeviceItem *>(m_deviceLayout->itemAt(i));
+        if (item && Solid::Device(item->udi()).parent().udi() == udi) {
+            deviceList << item;
+        }
+    }
+
+    return deviceList;
+}
+
 void NotifierDialog::collapseDevices()
 {
     for (int i = 0; i < m_deviceLayout->count(); ++i) {
@@ -580,6 +738,42 @@ void NotifierDialog::collapseDevices()
     }
 }
 
+void NotifierDialog::setupRequested(const QString& udi)
+{
+    DeviceItem *item = itemForUdi(udi);
+    if (!item) {
+        kDebug() << "udi not found -- This should just not happen";
+        return;
+    }
+
+    item->setState(DeviceItem::Mounting);
+}
+
+void NotifierDialog::teardownRequested(const QString& udi)
+{
+    DeviceItem *item = itemForUdi(udi);
+    if (!item) {
+        kDebug() << "udi not found -- This should just not happen";
+        return;
+    }
+
+    item->setState(DeviceItem::Umounting);
+}
+
+void NotifierDialog::ejectRequested(const QString& udi)
+{
+    QList<DeviceItem*> deviceList = itemsForParentUdi(udi);
+    if (deviceList.isEmpty()) {
+        kDebug() << "This should just not happen";
+        return;
+   }
+
+    foreach (DeviceItem* item, deviceList) {
+        item->setState(DeviceItem::Umounting);
+    }
+}
+
+
 void NotifierDialog::leftActionActivated(DeviceItem *item)
 {
     Solid::Device device(item->udi());
@@ -588,17 +782,11 @@ void NotifierDialog::leftActionActivated(DeviceItem *item)
         if (device.is<Solid::OpticalDisc>()) {
             Solid::OpticalDrive *drive = device.parent().as<Solid::OpticalDrive>();
             if (drive) {
-                item->setBusy();
-                connect(drive, SIGNAL(ejectDone(Solid::ErrorType, QVariant, const QString &)),
-                        this, SLOT(storageEjectDone(Solid::ErrorType, QVariant, const QString &)));
                 drive->eject();
             }
         } else if (device.is<Solid::StorageVolume>()) {
             Solid::StorageAccess *access = device.as<Solid::StorageAccess>();
             if (access && access->isAccessible()) {
-                item->setBusy();
-                connect(access, SIGNAL(teardownDone(Solid::ErrorType, QVariant, const QString &)),
-                        this, SLOT(storageTeardownDone(Solid::ErrorType, QVariant, const QString &)));
                 access->teardown();
             }
         }
@@ -607,9 +795,6 @@ void NotifierDialog::leftActionActivated(DeviceItem *item)
 
         // only unmounted devices
         if (access && !access->isAccessible()) {
-            item->setBusy();
-            connect(access, SIGNAL(setupDone(Solid::ErrorType, QVariant, const QString &)),
-                    this, SLOT(storageSetupDone(Solid::ErrorType, QVariant , const QString &)));
             access->setup();
         }
     }
@@ -706,12 +891,6 @@ QString NotifierDialog::getCategoryNameOfDevice(const Solid::Device &device)
     return 0;
 }
 
-void NotifierDialog::resetNotifierIcon()
-{
-    m_notifier->changeNotifierIcon();
-    m_notifier->update();
-}
-
 void NotifierDialog::resetSelection()
 {
     m_itemBackground->setTargetItem(0);
@@ -727,7 +906,7 @@ void NotifierDialog::resetSelection()
 
 void NotifierDialog::updateFreeSpace(DeviceItem *item)
 {
-    if (item->isMounted()) {
+    if (item->isMounted() && (item->state() != DeviceItem::Umounting)) {
         Solid::Device device(item->udi());
         Solid::StorageAccess *access = device.as<Solid::StorageAccess>();
         if (access) {
@@ -816,10 +995,10 @@ void NotifierDialog::updateCategoryColors(Plasma::Label * category)
 
 void NotifierDialog::updateMainLabelText()
 {
-    if (m_deviceLayout->count() == 0) {
-        m_mainLabel->setText(i18n("No devices plugged in."));
+    if (m_deviceCount > 0) {
+        m_mainLabel->setText(i18n("Available Devices"));
     } else {
-        m_mainLabel->setText(i18n("Devices recently plugged in:"));
+        m_mainLabel->setText(i18n("No Devices Available"));
     }
 }
 
