@@ -33,6 +33,7 @@
 // KDE
 #include <KIcon>
 #include <KJob>
+#include <KAuthorized>
 #ifndef Q_OS_WIN
 #include <KConfigDialog>
 #include <KSharedConfig>
@@ -44,7 +45,6 @@
 
 // Windows
 #ifdef Q_OS_WIN
-#define _WIN32_WINNT 0x0500 // require NT 5.0 (win 2k pro)
 #include <windows.h>
 #endif // Q_OS_WIN
 
@@ -57,6 +57,7 @@ LockOut::LockOut(QObject *parent, const QVariantList &args)
 #ifndef Q_OS_WIN
     setHasConfigurationInterface(true);
 #endif
+    setAspectRatioMode(Plasma::IgnoreAspectRatio);
 }
 
 void LockOut::init()
@@ -134,52 +135,51 @@ void LockOut::checkLayout()
     getContentsMargins(&left,&top, &right, &bottom);
     int width = geometry().width() - left - right;
     int height = geometry().height() - top - bottom;
-    
-    Qt::Orientation direction;
-    qreal ratioToKeep = 2;
+
+    Qt::Orientation direction = Qt::Vertical;
+    int minWidth = 0;
+    int minHeight = 0;
 
     switch (formFactor()) {
         case Plasma::Vertical:
-            if (width < (MINBUTTONSIZE + MARGINSIZE)* m_visibleButtons ) {
-                direction = Qt::Vertical;
-                ratioToKeep = m_visibleButtons;
-            } else {
+            if (width >= (MINBUTTONSIZE + MARGINSIZE) * m_visibleButtons) {
                 direction = Qt::Horizontal;
-                ratioToKeep = 0.5;
+                height = qMax(width / m_visibleButtons, MINBUTTONSIZE);
+                minHeight = MINBUTTONSIZE;
+            } else {
+                minHeight = MINBUTTONSIZE * m_visibleButtons + top + bottom;
             }
             break;
+
         case Plasma::Horizontal:
-            if (height < (MINBUTTONSIZE + MARGINSIZE)* m_visibleButtons) {
+            if (height < (MINBUTTONSIZE + MARGINSIZE) * m_visibleButtons) {
                 direction = Qt::Horizontal;
-                ratioToKeep = m_visibleButtons;
+                minWidth = MINBUTTONSIZE * m_visibleButtons + left + right;
             } else {
-                direction = Qt::Vertical;
-                ratioToKeep = 0.5;
+                width = qMax(height / m_visibleButtons, MINBUTTONSIZE);
+                minWidth = MINBUTTONSIZE;
             }
             break;
+
         default:
-            direction = Qt::Vertical;
+            if (width > height) {
+                direction = Qt::Horizontal;
+                minWidth = MINBUTTONSIZE * m_visibleButtons + left + right;
+                minHeight = MINBUTTONSIZE + top + bottom;
+            } else {
+                minWidth = MINBUTTONSIZE + left + right;
+                minHeight = MINBUTTONSIZE * m_visibleButtons + top + bottom;
+            }
+            break;
     }
 
-    if (direction != m_layout->orientation()) {
-        m_layout->setOrientation(direction);
-    }
-
-    if (formFactor() == Plasma::Horizontal) {
-        setSizePolicy(QSizePolicy(QSizePolicy::Maximum, QSizePolicy::Expanding));
-        qreal wsize = size().height() * ratioToKeep;
-        setMaximumSize(wsize, QWIDGETSIZE_MAX);
-        setMinimumSize(0,0);
-    } else if (formFactor() == Plasma::Vertical) {
-        setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum));
-        qreal hsize = size().width() * ratioToKeep;
-        setMaximumSize(QWIDGETSIZE_MAX, hsize);
-        setMinimumSize(0,0);
+    m_layout->setOrientation(direction);
+    setMinimumSize(minWidth, minHeight);
+    if (direction == Qt::Horizontal) {
+        setPreferredSize(height * m_visibleButtons + left + right, height + top + bottom);
     } else {
-        setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
-        setMinimumSize(40,40);
+        setPreferredSize(width + left + right, width * m_visibleButtons + top + bottom);
     }
-    
 }
 
 void LockOut::constraintsEvent(Plasma::Constraints constraints)
@@ -208,9 +208,13 @@ void LockOut::clickLock()
 
 void LockOut::clickLogout()
 {
+    if (!KAuthorized::authorizeKAction("logout")) {
+        return;
+    }
+    
     kDebug()<<"LockOut:: logout clicked ";
 #ifndef Q_OS_WIN
-    KWorkSpace::requestShutDown( KWorkSpace::ShutdownConfirmYes,
+    KWorkSpace::requestShutDown( KWorkSpace::ShutdownConfirmDefault,
                                  KWorkSpace::ShutdownTypeDefault,
                                  KWorkSpace::ShutdownModeDefault);
 #endif
